@@ -10,20 +10,17 @@ use MediaWiki\Extension\AbuseFilter\Consequences\ConsequencesLookup;
 use MediaWiki\Extension\AbuseFilter\Consequences\ConsequencesRegistry;
 use MediaWiki\Extension\AbuseFilter\FilterLookup;
 use MediaWiki\Extension\AbuseFilter\Variables\VariableHolder;
+use MediaWiki\User\UserIdentityUtils;
 use MediaWikiUnitTestCase;
 use Psr\Log\NullLogger;
 
 /**
  * @group Test
  * @group AbuseFilter
- * @coversDefaultClass \MediaWiki\Extension\AbuseFilter\Consequences\ConsequencesExecutorFactory
+ * @covers \MediaWiki\Extension\AbuseFilter\Consequences\ConsequencesExecutorFactory
  */
 class ConsequencesExecutorFactoryTest extends MediaWikiUnitTestCase {
 
-	/**
-	 * @covers ::__construct
-	 * @covers ::newExecutor
-	 */
 	public function testNewExecutor() {
 		$factory = new ConsequencesExecutorFactory(
 			$this->createMock( ConsequencesLookup::class ),
@@ -31,6 +28,7 @@ class ConsequencesExecutorFactoryTest extends MediaWikiUnitTestCase {
 			$this->createMock( ConsequencesRegistry::class ),
 			$this->createMock( FilterLookup::class ),
 			new NullLogger(),
+			$this->createMock( UserIdentityUtils::class ),
 			$this->createMock( ServiceOptions::class )
 		);
 		$factory->newExecutor(
