@@ -2,8 +2,8 @@
 
 namespace MediaWiki\Extension\DiscussionTools\Tests;
 
-use ApiTestCase;
-use Title;
+use MediaWiki\Tests\Api\ApiTestCase;
+use MediaWiki\Title\Title;
 
 /**
  * @group medium
