@@ -10,7 +10,7 @@ namespace MediaWiki\Extension\Math\WikiTexVC\MMLnodes;
  */
 class MMLmstyle extends MMLbase {
 
-	public function __construct( string $texclass = "", array $attributes = [] ) {
-		parent::__construct( "mstyle", $texclass, $attributes );
+	public function __construct( string $texclass = "", array $attributes = [], MMLbase ...$children ) {
+		parent::__construct( "mstyle", $texclass, $attributes, ...$children );
 	}
 }
