@@ -3,24 +3,25 @@
 namespace MediaWiki\Extension\Scribunto\Engines\LuaCommon;
 
 use Exception;
-use FormatJson;
-use Html;
 use MediaWiki\Extension\Scribunto\Engines\LuaSandbox\LuaSandboxInterpreter;
 use MediaWiki\Extension\Scribunto\Scribunto;
+use MediaWiki\Extension\Scribunto\ScribuntoContent;
 use MediaWiki\Extension\Scribunto\ScribuntoEngineBase;
 use MediaWiki\Extension\Scribunto\ScribuntoException;
+use MediaWiki\Html\Html;
+use MediaWiki\Json\FormatJson;
 use MediaWiki\MediaWikiServices;
-use MWException;
-use ObjectCache;
-use Parser;
-use PPFrame;
-use Title;
+use MediaWiki\Parser\Parser;
+use MediaWiki\Parser\PPFrame;
+use MediaWiki\Parser\PPNode;
+use MediaWiki\Title\Title;
+use RuntimeException;
 use Wikimedia\ScopedCallback;
 
 abstract class LuaEngine extends ScribuntoEngineBase {
 	/**
 	 * Libraries to load. See also the 'ScribuntoExternalLibraries' hook.
-	 * @var array Maps module names to PHP classes or definition arrays
+	 * @var array<string,class-string<LibraryBase>> Maps module names to PHP classes or definition arrays
 	 */
 	protected static $libraryClasses = [
 		'mw.site' => SiteLibrary::class,
@@ -37,7 +38,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	/**
 	 * Paths for modules that may be loaded from Lua. See also the
 	 * 'ScribuntoExternalLibraryPaths' hook.
-	 * @var array Paths
+	 * @var string[] Paths
 	 */
 	protected static $libraryPaths = [
 		'.',
@@ -59,15 +60,15 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	protected $mw;
 
 	/**
-	 * @var array
+	 * @var array<string,?PPFrame>
 	 */
 	protected $currentFrames = [];
 	/**
-	 * @var array|null
+	 * @var array<string,string>|null
 	 */
 	protected $expandCache = [];
 	/**
-	 * @var array
+	 * @var array<string,array|class-string<LibraryBase>>
 	 */
 	protected $availableLibraries = [];
 
@@ -81,7 +82,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @return LuaEngine
 	 */
 	public static function newAutodetectEngine( array $options ) {
-		global $wgScribuntoEngineConf;
+		$engineConf = MediaWikiServices::getInstance()->getMainConfig()->get( 'ScribuntoEngineConf' );
 		$engine = 'luastandalone';
 		try {
 			LuaSandboxInterpreter::checkLuaSandboxVersion();
@@ -93,7 +94,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 		unset( $options['factory'] );
 
 		// @phan-suppress-next-line PhanTypeMismatchReturnSuperType
-		return Scribunto::newEngine( $options + $wgScribuntoEngineConf[$engine] );
+		return Scribunto::newEngine( $options + $engineConf[$engine] );
 	}
 
 	/**
@@ -189,7 +190,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 *
 	 * @param string $moduleFileName The path to the Lua portion of the library
 	 *         (absolute, or relative to $this->getLuaLibDir())
-	 * @param array $interfaceFuncs Populates mw_interface
+	 * @param array<string,callable> $interfaceFuncs Populates mw_interface
 	 * @param array $setupOptions Passed to the modules setupInterface() method.
 	 * @return array Lua package
 	 */
@@ -253,7 +254,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @param PPFrame|null $frame If null, an empty frame with no parent will be used
 	 * @return ScopedCallback
 	 */
-	private function setupCurrentFrames( PPFrame $frame = null ) {
+	private function setupCurrentFrames( ?PPFrame $frame = null ) {
 		if ( !$frame ) {
 			$frame = $this->getParser()->getPreprocessor()->newFrame();
 		}
@@ -361,13 +362,15 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	protected function loadLibraryFromFile( $fileName ) {
 		static $cache = null;
 
+		$objectcachefactory = MediaWikiServices::getInstance()->getObjectCacheFactory();
+
 		if ( !$cache ) {
-			$cache = ObjectCache::getLocalServerInstance( 'hash' );
+			$cache = $objectcachefactory->getLocalServerInstance( CACHE_HASH );
 		}
 
 		$mtime = filemtime( $fileName );
 		if ( $mtime === false ) {
-			throw new MWException( 'Lua file does not exist: ' . $fileName );
+			throw new RuntimeException( 'Lua file does not exist: ' . $fileName );
 		}
 
 		$cacheKey = $cache->makeGlobalKey( __CLASS__, $fileName );
@@ -375,7 +378,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 
 		$code = false;
 		if ( $fileData ) {
-			list( $code, $cachedMtime ) = $fileData;
+			[ $code, $cachedMtime ] = $fileData;
 			if ( $cachedMtime < $mtime ) {
 				$code = false;
 			}
@@ -383,7 +386,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 		if ( !$code ) {
 			$code = file_get_contents( $fileName );
 			if ( $code === false ) {
-				throw new MWException( 'Lua file does not exist: ' . $fileName );
+				throw new RuntimeException( 'Lua file does not exist: ' . $fileName );
 			}
 			$cache->set( $cacheKey, [ $code, $mtime ], 60 * 5 );
 		}
@@ -472,7 +475,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @param string $funcName The Lua function name, for use in error messages
 	 * @param array $args The argument array
 	 * @param int $index0 The zero-based argument index
-	 * @param string|array $type The allowed type names as given by gettype()
+	 * @param string|string[] $type The allowed type names as given by gettype()
 	 * @param string $msgType The type name used in the error message
 	 * @throws LuaError
 	 */
@@ -511,23 +514,24 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	/**
 	 * Instantiate and register a library.
 	 * @param string $name
-	 * @param array|string $def
+	 * @param array|class-string<LibraryBase> $def
 	 * @param bool $loadDeferred
-	 * @throws MWException
 	 * @return array|null
 	 */
 	private function instantiatePHPLibrary( $name, $def, $loadDeferred ) {
 		$def = $this->availableLibraries[$name];
 		if ( is_string( $def ) ) {
+			/** @var LibraryBase $class */
 			$class = new $def( $this );
 		} else {
 			if ( !$loadDeferred && !empty( $def['deferLoad'] ) ) {
 				return null;
 			}
 			if ( isset( $def['class'] ) ) {
+				/** @var LibraryBase $class */
 				$class = new $def['class']( $this );
 			} else {
-				throw new MWException( "No class for library \"$name\"" );
+				throw new RuntimeException( "No class for library \"$name\"" );
 			}
 		}
 		return $class->register();
@@ -543,8 +547,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @return array
 	 */
 	public function loadPHPLibrary( $name ) {
-		$args = func_get_args();
-		$this->checkString( 'loadPHPLibrary', $args, 0 );
+		$this->checkString( 'loadPHPLibrary', [ $name ], 0 );
 
 		$ret = null;
 		if ( isset( $this->availableLibraries[$name] ) ) {
@@ -564,8 +567,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @return array
 	 */
 	public function loadPackage( $name ) {
-		$args = func_get_args();
-		$this->checkString( 'loadPackage', $args, 0 );
+		$this->checkString( 'loadPackage', [ $name ], 0 );
 
 		# This is what Lua does for its built-in loaders
 		$luaName = str_replace( '.', '/', $name ) . '.lua';
@@ -606,6 +608,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 		if ( $frameId === 'empty' ) {
 			return $this->getParser()->getPreprocessor()->newFrame();
 		} elseif ( isset( $this->currentFrames[$frameId] ) ) {
+			// @phan-suppress-next-line PhanTypeMismatchReturnNullable False positive
 			return $this->currentFrames[$frameId];
 		} else {
 			throw new LuaError( 'invalid frame ID' );
@@ -673,8 +676,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @param int $ttl
 	 */
 	public function setTTL( $ttl ) {
-		$args = func_get_args();
-		$this->checkNumber( 'setTTL', $args, 0 );
+		$this->checkNumber( 'setTTL', [ $ttl ], 0 );
 
 		$frame = $this->getFrameById( 'current' );
 		$frame->setTTL( $ttl );
@@ -688,8 +690,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @return array
 	 */
 	public function getExpandedArgument( $frameId, $name ) {
-		$args = func_get_args();
-		$this->checkString( 'getExpandedArgument', $args, 0 );
+		$this->checkString( 'getExpandedArgument', [ $frameId ], 0 );
 
 		$frame = $this->getFrameById( $frameId );
 		$this->getInterpreter()->pauseUsageTimer();
@@ -736,10 +737,11 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 			throw new LuaError( 'expandTemplate: template inclusion denied' );
 		}
 
-		list( $dom, $finalTitle ) = $this->parser->getTemplateDom( $title );
+		[ $dom, $finalTitle ] = $this->parser->getTemplateDom( $title );
 		if ( $dom === false ) {
 			throw new LuaError( "expandTemplate: template \"$titleText\" does not exist" );
 		}
+		$finalTitle = Title::newFromLinkTarget( $finalTitle );
 		if ( !$frame->loopCheck( $finalTitle ) ) {
 			throw new LuaError( 'expandTemplate: template loop detected' );
 		}
@@ -761,7 +763,6 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @param string $frameId
 	 * @param string $function
 	 * @param array $args
-	 * @throws MWException
 	 * @throws LuaError
 	 * @return array
 	 * @suppress PhanImpossibleCondition
@@ -851,8 +852,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @throws LuaError
 	 */
 	public function preprocess( $frameId, $text ) {
-		$args = func_get_args();
-		$this->checkString( 'preprocess', $args, 0 );
+		$this->checkString( 'preprocess', [ $frameId ], 0 );
 
 		$frame = $this->getFrameById( $frameId );
 
@@ -895,8 +895,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 	 * @param string $text wikitext
 	 */
 	public function addWarning( $text ) {
-		$args = func_get_args();
-		$this->checkString( 'addWarning', $args, 0 );
+		$this->checkString( 'addWarning', [ $text ], 0 );
 
 		// Message localization has to happen on the Lua side
 		$this->getParser()->getOutput()->addWarningMsg(
@@ -918,7 +917,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 
 	/**
 	 * @param PPFrame $frame
-	 * @param string|array $input
+	 * @param string|PPNode $input
 	 * @param mixed $cacheKey
 	 * @return string
 	 */
@@ -964,7 +963,7 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 		}
 
 		$parser = $this->getParser();
-		list( $text, $finalTitle ) = $parser->fetchTemplateAndTitle( $titleObj );
+		[ $text, $finalTitle ] = $parser->fetchTemplateAndTitle( $titleObj );
 
 		$json = FormatJson::decode( $text, true );
 		if ( is_array( $json ) ) {
@@ -974,6 +973,63 @@ abstract class LuaEngine extends ScribuntoEngineBase {
 
 		return [ $json ];
 	}
+
+	/**
+	 * @see Content::updateRedirect
+	 *
+	 * @param ScribuntoContent $content
+	 * @param Title $target
+	 * @return ScribuntoContent
+	 */
+	public function updateRedirect( ScribuntoContent $content, Title $target ): ScribuntoContent {
+		if ( !$content->isRedirect() ) {
+			return $content;
+		}
+		return $this->makeRedirectContent( $target );
+	}
+
+	/**
+	 * @see Content::getRedirectTarget
+	 *
+	 * @param ScribuntoContent $content
+	 * @return Title|null
+	 */
+	public function getRedirectTarget( ScribuntoContent $content ) {
+		$text = $content->getText();
+		preg_match( '/^return require \[\[(.*?)\]\]/', $text, $matches );
+		if ( isset( $matches[1] ) ) {
+			$title = Title::newFromText( $matches[1] );
+			// Can only redirect to other Scribunto pages
+			if ( $title && $title->hasContentModel( CONTENT_MODEL_SCRIBUNTO ) ) {
+				// Have a title, check that the current content equals what
+				// the redirect content should be
+				if ( $content->equals( $this->makeRedirectContent( $title ) ) ) {
+					return $title;
+				}
+			}
+		}
+		return null;
+	}
+
+	/**
+	 * @see ContentHandler::makeRedirectContent
+	 * @param Title $destination
+	 * @param string $text
+	 * @return ScribuntoContent
+	 */
+	public function makeRedirectContent( Title $destination, $text = '' ) {
+		$targetPage = $destination->getPrefixedText();
+		$redirectText = "return require [[$targetPage]]";
+		return new ScribuntoContent( $redirectText );
+	}
+
+	/**
+	 * @see ContentHandler::supportsRedirects
+	 * @return bool true
+	 */
+	public function supportsRedirects(): bool {
+		return true;
+	}
 }
 
 class_alias( LuaEngine::class, 'Scribunto_LuaEngine' );
