@@ -242,10 +242,6 @@ class CommentParser {
 						']]'
 					], '', $section );
 
-					// We don't want any links in the auto text to be linked, but we still
-					// want to show any [[ ]]
-					$sectionText = str_replace( '[[', '&#91;[', $auto );
-
 					$section = substr( Parser::guessSectionNameFromStrippedText( $section ), 1 );
 					if ( $section !== '' ) {
 						if ( $samePage ) {
@@ -256,7 +252,7 @@ class CommentParser {
 						$auto = $this->makeSectionLink(
 							$sectionTitle,
 							$this->userLang->getArrow() .
-								Html::rawElement( 'bdi', [ 'dir' => $this->userLang->getDir() ], $sectionText ),
+								Html::rawElement( 'bdi', [ 'dir' => $this->userLang->getDir() ], $auto ),
 							$wikiId,
 							$selfLinkTarget
 						);
@@ -273,7 +269,10 @@ class CommentParser {
 				if ( $auto ) {
 					$auto = Html::rawElement( 'span', [ 'class' => 'autocomment' ], $auto );
 				}
-				return $pre . $auto;
+
+				// Make sure any brackets (which the user could have input in the edit summary)
+				// in the generated autocomment HTML don't trigger additional link processing (T406664).
+				return str_replace( [ '[', ']' ], [ '&#91;', '&#93;' ], $pre . $auto );
 			},
 			$comment
 		);
