@@ -22,9 +22,9 @@
 
 namespace MediaWiki\Revision;
 
-use Content;
 use InvalidArgumentException;
 use LogicException;
+use MediaWiki\Content\Content;
 use OutOfBoundsException;
 use Wikimedia\Assert\Assert;
 use Wikimedia\NonSerializable\NonSerializableTrait;
@@ -256,7 +256,7 @@ class SlotRecord {
 	 *        in revision history.
 	 */
 	public function __construct( \stdClass $row, $content, bool $derived = false ) {
-		Assert::parameterType( 'Content|callable', $content, '$content' );
+		Assert::parameterType( [ 'Content', 'callable' ], $content, '$content' );
 
 		Assert::parameter(
 			property_exists( $row, 'slot_revision_id' ),
@@ -310,6 +310,8 @@ class SlotRecord {
 	 *
 	 * @throws SuppressedDataException if access to the content is not allowed according
 	 * to the audience check performed by RevisionRecord::getSlot().
+	 * @throws BadRevisionException if the revision is permanently missing
+	 * @throws RevisionAccessException for other storage access errors
 	 *
 	 * @return Content The slot's content. This is a direct reference to the internal instance,
 	 * copy before exposing to application logic!
@@ -319,7 +321,7 @@ class SlotRecord {
 			return $this->content;
 		}
 
-		$obj = call_user_func( $this->content, $this );
+		$obj = ( $this->content )( $this );
 
 		Assert::postcondition(
 			$obj instanceof Content,
@@ -358,7 +360,7 @@ class SlotRecord {
 
 		// NOTE: allow callbacks, but don't trust plain string callables from the database!
 		if ( !is_string( $value ) && is_callable( $value ) ) {
-			$value = call_user_func( $value, $this );
+			$value = $value( $this );
 			$this->setField( $name, $value );
 		}
 
@@ -686,9 +688,3 @@ class SlotRecord {
 	}
 
 }
-
-/**
- * Retain the old class name for backwards compatibility.
- * @deprecated since 1.32
- */
-class_alias( SlotRecord::class, 'MediaWiki\Storage\SlotRecord' );
