@@ -75,6 +75,13 @@ class ApiResult implements ApiSerializable {
 	 */
 	public const NO_VALIDATE = self::NO_SIZE_CHECK | 8;
 
+	/**
+	 * For addValue(), setValue() and similar functions, do allow override
+	 * of conflicting keys.
+	 * @since 1.45 (also backported to 1.43.6, 1.44.3)
+	 */
+	public const IGNORE_CONFLICT_KEYS = 16;
+
 	/**
 	 * Key for the 'indexed tag name' metadata item. Value is string.
 	 * @since 1.25
@@ -311,7 +318,7 @@ class ApiResult implements ApiSerializable {
 			}
 		} elseif ( is_array( $arr[$name] ) && is_array( $value ) ) {
 			$conflicts = array_intersect_key( $arr[$name], $value );
-			if ( !$conflicts ) {
+			if ( !$conflicts || ( $flags & self::IGNORE_CONFLICT_KEYS ) ) {
 				$arr[$name] += $value;
 			} else {
 				$keys = implode( ', ', array_keys( $conflicts ) );
@@ -792,10 +799,7 @@ class ApiResult implements ApiSerializable {
 	 * @return bool
 	 */
 	public static function isMetadataKey( $key ) {
-		// Optimization: This is a very hot and highly optimized code path. Note that ord() only
-		// considers the first character and also works with empty strings and integers.
-		// 95 corresponds to the '_' character.
-		return ord( $key ) === 95;
+		return str_starts_with( $key, '_' );
 	}
 
 	/**
