@@ -22,26 +22,28 @@
 
 namespace MediaWiki\Deferred\LinksUpdate;
 
-use AutoCommitUpdate;
-use BacklinkCache;
-use DataUpdate;
-use DeferredUpdates;
-use DeprecationHelper;
-use Job;
+use InvalidArgumentException;
+use MediaWiki\Cache\BacklinkCache;
+use MediaWiki\Deferred\AutoCommitUpdate;
+use MediaWiki\Deferred\DataUpdate;
+use MediaWiki\Deferred\DeferredUpdates;
 use MediaWiki\HookContainer\ProtectedHookAccessorTrait;
+use MediaWiki\JobQueue\Job;
+use MediaWiki\JobQueue\Jobs\RefreshLinksJob;
 use MediaWiki\Logger\LoggerFactory;
+use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Page\PageIdentity;
 use MediaWiki\Page\PageReference;
 use MediaWiki\Page\PageReferenceValue;
+use MediaWiki\Parser\ParserOutput;
 use MediaWiki\Revision\RevisionRecord;
+use MediaWiki\Title\Title;
 use MediaWiki\User\UserIdentity;
-use MWException;
-use ParserOutput;
-use RefreshLinksJob;
 use RuntimeException;
-use Title;
+use Wikimedia\Rdbms\IConnectionProvider;
 use Wikimedia\Rdbms\IDatabase;
+use Wikimedia\Rdbms\IDBAccessObject;
 use Wikimedia\ScopedCallback;
 
 /**
@@ -53,9 +55,6 @@ use Wikimedia\ScopedCallback;
  */
 class LinksUpdate extends DataUpdate {
 	use ProtectedHookAccessorTrait;
-	use DeprecationHelper;
-
-	// @todo make members protected, but make sure extensions don't break
 
 	/** @var int Page ID of the article linked from */
 	protected $mId;
@@ -69,6 +68,9 @@ class LinksUpdate extends DataUpdate {
 	/** @var bool Whether to queue jobs for recursive updates */
 	protected $mRecursive;
 
+	/** @var bool Whether the page's redirect target may have changed in the latest revision */
+	protected $mMaybeRedirectChanged;
+
 	/** @var RevisionRecord Revision for which this update has been triggered */
 	private $mRevisionRecord;
 
@@ -83,91 +85,27 @@ class LinksUpdate extends DataUpdate {
 	/** @var LinksTableGroup */
 	private $tableFactory;
 
+	private IConnectionProvider $dbProvider;
+
 	/**
 	 * @param PageIdentity $page The page we're updating
 	 * @param ParserOutput $parserOutput Output from a full parse of this page
 	 * @param bool $recursive Queue jobs for recursive updates?
-	 *
-	 * @throws MWException
+	 * @param bool $maybeRedirectChanged True if the page's redirect target may have changed in the
+	 *   latest revision. If false, this is used as a hint to skip some unnecessary updates.
 	 */
-	public function __construct( PageIdentity $page, ParserOutput $parserOutput, $recursive = true ) {
+	public function __construct(
+		PageIdentity $page,
+		ParserOutput $parserOutput,
+		$recursive = true,
+		$maybeRedirectChanged = true
+	) {
 		parent::__construct();
 
-		$this->mTitle = Title::castFromPageIdentity( $page );
+		$this->mTitle = Title::newFromPageIdentity( $page );
 		$this->mParserOutput = $parserOutput;
-
-		$this->deprecatePublicProperty( 'mId', '1.38', __CLASS__ );
-		$this->deprecatePublicProperty( 'mTitle', '1.38', __CLASS__ );
-		$this->deprecatePublicProperty( 'mParserOutput', '1.38', __CLASS__ );
-
-		$this->deprecatePublicPropertyFallback( 'mLinks', '1.38',
-			function () {
-				return $this->getParserOutput()->getLinks();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mImages', '1.38',
-			function () {
-				return $this->getParserOutput()->getImages();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mTemplates', '1.38',
-			function () {
-				return $this->getParserOutput()->getTemplates();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mExternals', '1.38',
-			function () {
-				return $this->getParserOutput()->getExternalLinks();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mCategories', '1.38',
-			function () {
-				return $this->getParserOutput()->getCategories();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mProperties', '1.38',
-			function () {
-				return $this->getParserOutput()->getPageProperties();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mInterwikis', '1.38',
-			function () {
-				return $this->getParserOutput()->getInterwikiLinks();
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mInterlangs', '1.38',
-			function () {
-				$ill = $this->getParserOutput()->getLanguageLinks();
-				$res = [];
-				foreach ( $ill as $link ) {
-					list( $key, $title ) = explode( ':', $link, 2 );
-					$res[$key] = $title;
-				}
-				return $res;
-			},
-			null, __CLASS__
-		);
-		$this->deprecatePublicPropertyFallback( 'mCategories', '1.38',
-			function () {
-				$cats = $this->getParserOutput()->getCategories();
-				foreach ( $cats as &$sortkey ) {
-					# If the sortkey is longer then 255 bytes, it is truncated by DB, and then doesn't match
-					# when comparing existing vs current categories, causing T27254.
-					$sortkey = mb_strcut( $sortkey, 0, 255 );
-				}
-			},
-			null, __CLASS__
-		);
-
 		$this->mRecursive = $recursive;
-		$this->deprecatePublicProperty( 'mRecursive', '1.38', __CLASS__ );
+		$this->mMaybeRedirectChanged = $maybeRedirectChanged;
 
 		$services = MediaWikiServices::getInstance();
 		$config = $services->getMainConfig();
@@ -177,16 +115,12 @@ class LinksUpdate extends DataUpdate {
 			$services->getCollationFactory(),
 			$page,
 			$services->getLinkTargetLookup(),
-			$config->get( 'UpdateRowsPerQuery' ),
-			function ( $table, $rows ) {
-				$this->getHookRunner()->onLinksUpdateAfterInsert( $this, $table, $rows );
-			},
-			$config->get( 'TempCategoryCollations' )
+			$config->get( MainConfigNames::UpdateRowsPerQuery ),
+			$config->get( MainConfigNames::TempCategoryCollations )
 		);
 		// TODO: this does not have to be called in LinksDeletionUpdate
 		$this->tableFactory->setParserOutput( $parserOutput );
-
-		$this->getHookRunner()->onLinksUpdateConstructed( $this );
+		$this->dbProvider = $services->getDBLoadBalancerFactory();
 	}
 
 	public function setTransactionTicket( $ticket ) {
@@ -197,8 +131,6 @@ class LinksUpdate extends DataUpdate {
 	/**
 	 * Notify LinksUpdate that a move has just been completed and set the
 	 * original title
-	 *
-	 * @param PageReference $oldPage
 	 */
 	public function setMoveDetails( PageReference $oldPage ) {
 		$this->tableFactory->setMoveDetails( $oldPage );
@@ -212,13 +144,13 @@ class LinksUpdate extends DataUpdate {
 	public function doUpdate() {
 		if ( !$this->mId ) {
 			// NOTE: subclasses may initialize mId directly!
-			$this->mId = $this->mTitle->getArticleID( Title::READ_LATEST );
+			$this->mId = $this->mTitle->getArticleID( IDBAccessObject::READ_LATEST );
 		}
 
 		if ( !$this->mId ) {
 			// Probably due to concurrent deletion or renaming of the page
 			$logger = LoggerFactory::getInstance( 'SecondaryDataUpdate' );
-			$logger->notice(
+			$logger->warning(
 				'LinksUpdate: The Title object yields no ID. Perhaps the page was deleted?',
 				[
 					'page_title' => $this->mTitle->getPrefixedDBkey(),
@@ -318,8 +250,8 @@ class LinksUpdate extends DataUpdate {
 		self::queueRecursiveJobsForTable(
 			$this->mTitle, 'templatelinks', $action, $agent, $backlinkCache
 		);
-		if ( $this->mTitle->getNamespace() === NS_FILE ) {
-			// Process imagelinks in case the title is or was a redirect
+		if ( $this->mMaybeRedirectChanged && $this->mTitle->getNamespace() === NS_FILE ) {
+			// Process imagelinks in case the redirect target has changed
 			self::queueRecursiveJobsForTable(
 				$this->mTitle, 'imagelinks', $action, $agent, $backlinkCache
 			);
@@ -353,9 +285,9 @@ class LinksUpdate extends DataUpdate {
 	 * @param BacklinkCache|null $backlinkCache
 	 */
 	public static function queueRecursiveJobsForTable(
-		PageIdentity $page, $table, $action = 'unknown', $userName = 'unknown', ?BacklinkCache $backlinkCache = null
+		PageIdentity $page, $table, $action = 'LinksUpdate', $userName = 'unknown', ?BacklinkCache $backlinkCache = null
 	) {
-		$title = Title::castFromPageIdentity( $page );
+		$title = Title::newFromPageIdentity( $page );
 		if ( !$backlinkCache ) {
 			wfDeprecatedMsg( __METHOD__ . " needs a BacklinkCache object, null passed", '1.37' );
 			$backlinkCache = MediaWikiServices::getInstance()->getBacklinkCacheFactory()
@@ -420,8 +352,10 @@ class LinksUpdate extends DataUpdate {
 	/**
 	 * Return the list of images used as generated by the parser
 	 * @return array
+	 * @deprecated since 1.44
 	 */
 	public function getImages() {
+		wfDeprecated( __METHOD__, '1.44' ); // unused
 		return $this->getParserOutput()->getImages();
 	}
 
@@ -464,25 +398,16 @@ class LinksUpdate extends DataUpdate {
 		return $this->user;
 	}
 
-	/**
-	 * @return PageLinksTable
-	 */
 	protected function getPageLinksTable(): PageLinksTable {
 		// @phan-suppress-next-line PhanTypeMismatchReturnSuperType
 		return $this->tableFactory->get( 'pagelinks' );
 	}
 
-	/**
-	 * @return ExternalLinksTable
-	 */
 	protected function getExternalLinksTable(): ExternalLinksTable {
 		// @phan-suppress-next-line PhanTypeMismatchReturnSuperType
 		return $this->tableFactory->get( 'externallinks' );
 	}
 
-	/**
-	 * @return PagePropsTable
-	 */
 	protected function getPagePropsTable(): PagePropsTable {
 		// @phan-suppress-next-line PhanTypeMismatchReturnSuperType
 		return $this->tableFactory->get( 'page_props' );
@@ -492,10 +417,11 @@ class LinksUpdate extends DataUpdate {
 	 * Fetch page links added by this LinksUpdate.  Only available after the update is complete.
 	 *
 	 * @since 1.22
-	 * @deprecated since 1.38 use getPageReferenceIterator() or getPageReferenceArray()
+	 * @deprecated since 1.38 use getPageReferenceIterator() or getPageReferenceArray(), hard-deprecated since 1.43
 	 * @return Title[] Array of Titles
 	 */
 	public function getAddedLinks() {
+		wfDeprecated( __METHOD__, '1.43' );
 		return $this->getPageLinksTable()->getTitleArray( LinksTable::INSERTED );
 	}
 
@@ -503,10 +429,11 @@ class LinksUpdate extends DataUpdate {
 	 * Fetch page links removed by this LinksUpdate.  Only available after the update is complete.
 	 *
 	 * @since 1.22
-	 * @deprecated since 1.38 use getPageReferenceIterator() or getPageReferenceArray()
+	 * @deprecated since 1.38 use getPageReferenceIterator() or getPageReferenceArray(), hard-deprecated since 1.43
 	 * @return Title[] Array of Titles
 	 */
 	public function getRemovedLinks() {
+		wfDeprecated( __METHOD__, '1.43' );
 		return $this->getPageLinksTable()->getTitleArray( LinksTable::DELETED );
 	}
 
@@ -570,7 +497,7 @@ class LinksUpdate extends DataUpdate {
 		if ( $table instanceof TitleLinksTable ) {
 			return $table->getPageReferenceIterator( $setType );
 		} else {
-			throw new \InvalidArgumentException(
+			throw new InvalidArgumentException(
 				__METHOD__ . ": $tableName does not have a list of titles" );
 		}
 	}
@@ -595,11 +522,11 @@ class LinksUpdate extends DataUpdate {
 		if ( $this->mId ) {
 			// The link updates made here only reflect the freshness of the parser output
 			$timestamp = $this->mParserOutput->getCacheTime();
-			$this->getDB()->update( 'page',
-				[ 'page_links_updated' => $this->getDB()->timestamp( $timestamp ) ],
-				[ 'page_id' => $this->mId ],
-				__METHOD__
-			);
+			$this->getDB()->newUpdateQueryBuilder()
+				->update( 'page' )
+				->set( [ 'page_links_updated' => $this->getDB()->timestamp( $timestamp ) ] )
+				->where( [ 'page_id' => $this->mId ] )
+				->caller( __METHOD__ )->execute();
 		}
 	}
 
@@ -608,7 +535,7 @@ class LinksUpdate extends DataUpdate {
 	 */
 	protected function getDB() {
 		if ( !$this->db ) {
-			$this->db = wfGetDB( DB_PRIMARY );
+			$this->db = $this->dbProvider->getPrimaryDatabase();
 		}
 
 		return $this->db;
@@ -624,6 +551,3 @@ class LinksUpdate extends DataUpdate {
 		return $this->mRecursive;
 	}
 }
-
-/** @deprecated since 1.38 */
-class_alias( LinksUpdate::class, 'LinksUpdate' );
