@@ -158,7 +158,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - reqTimeout      : post-connection timeout per request (seconds)
 	 *   - usePipelining   : whether to use HTTP pipelining if possible (for all hosts)
 	 *   - maxConnsPerHost : maximum number of concurrent connections (per host)
-	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2' or 'v2.0'. Leave empty to use
+	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2', 'v2.0', 'v3' or 'v3.0'. Leave empty to use
 	 *                       PHP/curl's default
 	 * @param string $caller The method making this request, for attribution in logs
 	 * @return array Response array for request
@@ -193,7 +193,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 	 *   - reqTimeout      : post-connection timeout per request (seconds)
 	 *   - usePipelining   : whether to use HTTP pipelining if possible (for all hosts)
 	 *   - maxConnsPerHost : maximum number of concurrent connections (per host)
-	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2' or 'v2.0'. Leave empty to use
+	 *   - httpVersion     : One of 'v1.0', 'v1.1', 'v2', 'v2.0', 'v3' or 'v3.0'. Leave empty to use
 	 *                       PHP/curl's default
 	 * @param string $caller The method making these requests, for attribution in logs
 	 * @return array[] $reqs With response array populated for each
@@ -222,6 +222,10 @@ class MultiHttpClient implements LoggerAwareInterface {
 				case 'v2.0':
 					$opts['httpVersion'] = CURL_HTTP_VERSION_2_0;
 					break;
+				case 'v3':
+				case 'v3.0':
+					$opts['httpVersion'] = CURL_HTTP_VERSION_3;
+					break;
 				default:
 					$opts['httpVersion'] = CURL_HTTP_VERSION_NONE;
 			}
@@ -470,7 +474,7 @@ class MultiHttpClient implements LoggerAwareInterface {
 				}
 				$length = strlen( $header );
 				$matches = [];
-				if ( preg_match( "/^(HTTP\/(?:1\.[01]|2)) (\d{3}) (.*)/", $header, $matches ) ) {
+				if ( preg_match( "/^(HTTP\/(?:1\.[01]|2|3)) (\d{3}) (.*)/", $header, $matches ) ) {
 					$req['response']['code'] = (int)$matches[2];
 					$req['response']['reason'] = trim( $matches[3] );
 					// After a redirect we will receive this again, but we already stored headers
