@@ -1,7 +1,5 @@
 <?php
 /**
- * Storage medium specific cache for storing items.
- *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -18,29 +16,34 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup Cache
  */
+namespace Wikimedia\ObjectCache;
 
+use InvalidArgumentException;
+use JsonSerializable;
+use stdClass;
 use Wikimedia\WaitConditionLoop;
 
 /**
- * Storage medium specific cache for storing items (e.g. redis, memcached, ...)
+ * Helper classs that implements most of BagOStuff for a backend.
  *
- * This should not be used for proxy classes that simply wrap other cache instances
+ * This should be used by concrete implementations only. Wrapper classes that
+ * proxy another BagOStuff should extend and implement BagOStuff directly.
  *
  * @ingroup Cache
  * @since 1.34
  */
 abstract class MediumSpecificBagOStuff extends BagOStuff {
-	/** @var array<string,array> Map of (key => (class, depth, expiry) */
+	/** @var array<string,array> Map of (key => (class LOCK_* constant => value) */
 	protected $locks = [];
-	/** @var int Seconds */
-	protected $syncTimeout;
 	/** @var int Bytes; chunk size of segmented cache values */
 	protected $segmentationSize;
 	/** @var int Bytes; maximum total size of a segmented cache value */
 	protected $segmentedValueMaxSize;
 
+	/** @var float Seconds; maximum expected seconds for a lock ping to reach the backend */
+	protected $maxLockSendDelay = 0.05;
+
 	/** @var array */
 	private $duplicateKeyLookups = [];
 	/** @var bool */
@@ -48,16 +51,10 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	/** @var bool */
 	private $dupeTrackScheduled = false;
 
-	/** @var callable[] */
-	protected $busyCallbacks = [];
-
-	/** @var array[] Map of (key => (PHP variable value, serialized value)) */
-	protected $preparedValues = [];
-
-	/** @var string Component to use for key construction of blob segment keys */
+	/** Component to use for key construction of blob segment keys */
 	private const SEGMENT_COMPONENT = 'segment';
 
-	/** @var int Idiom for doGet() to return extra information by reference */
+	/** Idiom for doGet() to return extra information by reference */
 	protected const PASS_BY_REF = -1;
 
 	protected const METRIC_OP_GET = 'get';
@@ -80,27 +77,29 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *   - logger: Psr\Log\LoggerInterface instance
 	 *   - reportDupes: Whether to emit warning log messages for all keys that were
 	 *      requested more than once (requires an asyncHandler).
-	 *   - syncTimeout: How long to wait with WRITE_SYNC in seconds.
 	 *   - segmentationSize: The chunk size, in bytes, of segmented values. The value should
 	 *      not exceed the maximum size of values in the storage backend, as configured by
 	 *      the site administrator.
 	 *   - segmentedValueMaxSize: The maximum total size, in bytes, of segmented values.
 	 *      This should be configured to a reasonable size give the site traffic and the
 	 *      amount of I/O between application and cache servers that the network can handle.
+	 *
 	 * @param array $params
+	 *
 	 * @phpcs:ignore Generic.Files.LineLength
-	 * @phan-param array{logger?:Psr\Log\LoggerInterface,asyncHandler?:callable,reportDupes?:bool,syncTimeout?:int,segmentationSize?:int,segmentedValueMaxSize?:int} $params
+	 * @phan-param array{logger?:\Psr\Log\LoggerInterface,asyncHandler?:callable,reportDupes?:bool,segmentationSize?:int|float,segmentedValueMaxSize?:int} $params
 	 */
 	public function __construct( array $params = [] ) {
 		parent::__construct( $params );
 
-		if ( !empty( $params['reportDupes'] ) && is_callable( $this->asyncHandler ) ) {
+		if ( !empty( $params['reportDupes'] ) && $this->asyncHandler ) {
 			$this->reportDupes = true;
 		}
 
-		$this->syncTimeout = $params['syncTimeout'] ?? 3;
-		$this->segmentationSize = $params['segmentationSize'] ?? 8388608; // 8MiB
-		$this->segmentedValueMaxSize = $params['segmentedValueMaxSize'] ?? 67108864; // 64MiB
+		// Default to 8MiB if segmentationSize is not set
+		$this->segmentationSize = $params['segmentationSize'] ?? 8_388_608;
+		// Default to 64MiB if segmentedValueMaxSize is not set
+		$this->segmentedValueMaxSize = $params['segmentedValueMaxSize'] ?? 67_108_864;
 	}
 
 	/**
@@ -114,6 +113,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *
 	 * @param string $key
 	 * @param int $flags Bitfield of BagOStuff::READ_* constants [optional]
+	 *
 	 * @return mixed Returns false on failure or if the item does not exist
 	 */
 	public function get( $key, $flags = 0 ) {
@@ -124,6 +124,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	/**
 	 * Track the number of times that a given key has been used.
+	 *
 	 * @param string $key
 	 */
 	private function trackDuplicateKeys( $key ) {
@@ -136,12 +137,12 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 			// easy filtering with array_filter() later.
 			$this->duplicateKeyLookups[$key] = 0;
 		} else {
-			$this->duplicateKeyLookups[$key] += 1;
+			$this->duplicateKeyLookups[$key]++;
 
 			if ( $this->dupeTrackScheduled === false ) {
 				$this->dupeTrackScheduled = true;
 				// Schedule a callback that logs keys processed more than once by get().
-				call_user_func( $this->asyncHandler, function () {
+				( $this->asyncHandler )( function () {
 					$dups = array_filter( $this->duplicateKeyLookups );
 					foreach ( $dups as $key => $count ) {
 						$this->logger->warning(
@@ -156,26 +157,23 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	}
 
 	/**
+	 * Get an item
+	 *
+	 * The CAS token should be null if the key does not exist or the value is corrupt
+	 *
 	 * @param string $key
 	 * @param int $flags Bitfield of BagOStuff::READ_* constants [optional]
-	 * @param mixed|null &$casToken cas() token if MediumSpecificBagOStuff::PASS_BY_REF [returned]
+	 * @param mixed &$casToken CAS token if MediumSpecificBagOStuff::PASS_BY_REF [returned]
+	 *
 	 * @return mixed Returns false on failure or if the item does not exist
 	 */
 	abstract protected function doGet( $key, $flags = 0, &$casToken = null );
 
-	/**
-	 * Set an item
-	 *
-	 * @param string $key
-	 * @param mixed $value
-	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
-	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
-	 * @return bool Success
-	 */
 	public function set( $key, $value, $exptime = 0, $flags = 0 ) {
-		list( $entry, $usable ) = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags );
+		$entry = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags, $ok );
+
 		// Only when all segments (if any) are stored should the main key be changed
-		return $usable ? $this->doSet( $key, $entry, $exptime, $flags ) : false;
+		return $ok && $this->doSet( $key, $entry, $exptime, $flags );
 	}
 
 	/**
@@ -185,23 +183,13 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param mixed $value
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	abstract protected function doSet( $key, $value, $exptime = 0, $flags = 0 );
 
-	/**
-	 * Delete an item
-	 *
-	 * For large values written using WRITE_ALLOW_SEGMENTS, this only deletes the main
-	 * segment list key unless WRITE_PRUNE_SEGMENTS is in the flags. While deleting the segment
-	 * list key has the effect of functionally deleting the key, it leaves unused blobs in cache.
-	 *
-	 * @param string $key
-	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
-	 * @return bool True if the item was deleted or not found, false on failure
-	 */
 	public function delete( $key, $flags = 0 ) {
-		if ( !$this->fieldHasFlags( $flags, self::WRITE_PRUNE_SEGMENTS ) ) {
+		if ( !$this->fieldHasFlags( $flags, self::WRITE_ALLOW_SEGMENTS ) ) {
 			return $this->doDelete( $key, $flags );
 		}
 
@@ -211,7 +199,8 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		}
 
 		if ( !SerializedValueContainer::isSegmented( $mainValue ) ) {
-			return true; // no segments to delete
+			// no segments to delete
+			return true;
 		}
 
 		$orderedKeys = array_map(
@@ -221,7 +210,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 			$mainValue->{SerializedValueContainer::SEGMENTED_HASHES}
 		);
 
-		return $this->deleteMulti( $orderedKeys, $flags & ~self::WRITE_PRUNE_SEGMENTS );
+		return $this->deleteMulti( $orderedKeys, $flags & ~self::WRITE_ALLOW_SEGMENTS );
 	}
 
 	/**
@@ -229,14 +218,16 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *
 	 * @param string $key
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool True if the item was deleted or not found, false on failure
 	 */
 	abstract protected function doDelete( $key, $flags = 0 );
 
 	public function add( $key, $value, $exptime = 0, $flags = 0 ) {
-		list( $entry, $usable ) = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags );
+		$entry = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags, $ok );
+
 		// Only when all segments (if any) are stored should the main key be changed
-		return $usable ? $this->doAdd( $key, $entry, $exptime, $flags ) : false;
+		return $ok && $this->doAdd( $key, $entry, $exptime, $flags );
 	}
 
 	/**
@@ -246,6 +237,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param mixed $value
 	 * @param int $exptime
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
+	 *
 	 * @return bool Success
 	 */
 	abstract protected function doAdd( $key, $value, $exptime = 0, $flags = 0 );
@@ -264,6 +256,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $attempts The amount of times to attempt a merge in case of failure
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	public function merge( $key, callable $callback, $exptime = 0, $attempts = 10, $flags = 0 ) {
@@ -276,13 +269,14 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $attempts The amount of times to attempt a merge in case of failure
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 * @see BagOStuff::merge()
 	 */
 	final protected function mergeViaCas( $key, callable $callback, $exptime, $attempts, $flags ) {
 		$attemptsLeft = $attempts;
 		do {
-			$token = self::PASS_BY_REF; // passed by reference
+			$token = self::PASS_BY_REF;
 			// Get the old value and CAS token from cache
 			$watchPoint = $this->watchErrors();
 			$currentValue = $this->resolveSegments(
@@ -292,8 +286,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 			if ( $this->getLastError( $watchPoint ) ) {
 				// Don't spam slow retries due to network problems (retry only on races)
 				$this->logger->warning(
-					__METHOD__ . ' failed due to read I/O error on get() for {key}.',
-					[ 'key' => $key ]
+					__METHOD__ . ' failed due to read I/O error on get() for {key}.', [ 'key' => $key ]
 				);
 				$success = false;
 				break;
@@ -301,16 +294,19 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 			// Derive the new value from the old value
 			$value = $callback( $this, $key, $currentValue, $exptime );
-			$keyWasNonexistant = ( $currentValue === false );
+			$keyWasNonexistent = ( $currentValue === false );
 			$valueMatchesOldValue = ( $value === $currentValue );
-			unset( $currentValue ); // free RAM in case the value is large
+			// free RAM in case the value is large
+			unset( $currentValue );
 
 			$watchPoint = $this->watchErrors();
 			if ( $value === false || $exptime < 0 ) {
-				$success = true; // do nothing
+				// do nothing
+				$success = true;
 			} elseif ( $valueMatchesOldValue && $attemptsLeft !== $attempts ) {
-				$success = true; // recently set by another thread to the same value
-			} elseif ( $keyWasNonexistant ) {
+				// recently set by another thread to the same value
+				$success = true;
+			} elseif ( $keyWasNonexistent ) {
 				// Try to create the key, failing if it gets created in the meantime
 				$success = $this->add( $key, $value, $exptime, $flags );
 			} else {
@@ -333,13 +329,14 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	}
 
 	/**
-	 * Check and set an item
+	 * Set an item if the current CAS token matches the provided CAS token
 	 *
-	 * @param mixed $casToken
+	 * @param mixed $casToken Only set the item if it still has this CAS token
 	 * @param string $key
 	 * @param mixed $value
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	protected function cas( $casToken, $key, $value, $exptime = 0, $flags = 0 ) {
@@ -349,37 +346,37 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 				[ 'key' => $key ]
 			);
 
-			return false; // caller may have meant to use add()?
+			// caller may have meant to use add()?
+			return false;
 		}
 
-		list( $entry, $usable ) = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags );
+		$entry = $this->makeValueOrSegmentList( $key, $value, $exptime, $flags, $ok );
+
 		// Only when all segments (if any) are stored should the main key be changed
-		return $usable ? $this->doCas( $casToken, $key, $entry, $exptime, $flags ) : false;
+		return $ok && $this->doCas( $casToken, $key, $entry, $exptime, $flags );
 	}
 
 	/**
-	 * Check and set an item
+	 * Set an item if the current CAS token matches the provided CAS token
 	 *
-	 * @param mixed $casToken
+	 * @param mixed $casToken CAS token from an existing version of the key
 	 * @param string $key
 	 * @param mixed $value
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	protected function doCas( $casToken, $key, $value, $exptime = 0, $flags = 0 ) {
 		// @TODO: the use of lock() assumes that all other relevant sets() use a lock
 		if ( !$this->lock( $key, 0 ) ) {
-			return false; // non-blocking
+			// non-blocking
+			return false;
 		}
 
-		$curCasToken = self::PASS_BY_REF; // passed by reference
+		$curCasToken = self::PASS_BY_REF;
 		$watchPoint = $this->watchErrors();
-		$this->doGet( $key, self::READ_LATEST, $curCasToken );
-		if ( is_object( $curCasToken ) ) {
-			// Using === does not work with objects since it checks for instance identity
-			throw new UnexpectedValueException( "CAS token cannot be an object" );
-		}
+		$exists = ( $this->doGet( $key, self::READ_LATEST, $curCasToken ) !== false );
 		if ( $this->getLastError( $watchPoint ) ) {
 			// Fail if the old CAS token could not be read
 			$success = false;
@@ -387,13 +384,14 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 				__METHOD__ . ' failed due to write I/O error for {key}.',
 				[ 'key' => $key ]
 			);
-		} elseif ( $casToken === $curCasToken ) {
+		} elseif ( $exists && $this->tokensMatch( $casToken, $curCasToken ) ) {
 			$success = $this->doSet( $key, $value, $exptime, $flags );
 		} else {
-			$success = false; // mismatched or failed
+			// mismatched or failed
+			$success = false;
 			$this->logger->info(
 				__METHOD__ . ' failed due to race condition for {key}.',
-				[ 'key' => $key ]
+				[ 'key' => $key, 'key_exists' => $exists ]
 			);
 		}
 
@@ -402,6 +400,29 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		return $success;
 	}
 
+	/**
+	 * @param mixed $value CAS token for an existing key
+	 * @param mixed $otherValue CAS token for an existing key
+	 *
+	 * @return bool Whether the two tokens match
+	 */
+	final protected function tokensMatch( $value, $otherValue ) {
+		$type = gettype( $value );
+		// Ideally, tokens are counters, timestamps, hashes, or serialized PHP values.
+		// However, some classes might use the PHP values themselves.
+		if ( $type !== gettype( $otherValue ) ) {
+			return false;
+		}
+		// Serialize both tokens to strictly compare objects or arrays (which might objects
+		// nested inside). Note that this will not apply if integer/string CAS tokens are used.
+		if ( $type === 'array' || $type === 'object' ) {
+			return ( serialize( $value ) === serialize( $otherValue ) );
+		}
+
+		// For string/integer tokens, use a simple comparison
+		return ( $value === $otherValue );
+	}
+
 	/**
 	 * Change the expiration on a key if it exists
 	 *
@@ -416,6 +437,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param string $key
 	 * @param int $exptime TTL or UNIX timestamp
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
+	 *
 	 * @return bool Success Returns false on failure or if the item does not exist
 	 * @since 1.28
 	 */
@@ -427,6 +449,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param string $key
 	 * @param int $exptime
 	 * @param int $flags
+	 *
 	 * @return bool
 	 */
 	protected function doChangeTTL( $key, $exptime, $flags ) {
@@ -468,6 +491,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param int $step
 	 * @param int $init
 	 * @param int $flags
+	 *
 	 * @return int|bool New value or false on failure
 	 */
 	abstract protected function doIncrWithInit( $key, $exptime, $step, $init, $flags );
@@ -477,6 +501,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param int $timeout
 	 * @param int $exptime
 	 * @param string $rclass
+	 *
 	 * @return bool
 	 */
 	public function lock( $key, $timeout = 6, $exptime = 6, $rclass = '' ) {
@@ -513,6 +538,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param string $key
 	 * @param int $timeout Lock wait timeout; 0 for non-blocking [optional]
 	 * @param int $exptime Lock time-to-live 1 day maximum [optional]
+	 *
 	 * @return float|null UNIX timestamp of acquisition; null on failure
 	 */
 	protected function doLock( $key, $timeout, $exptime ) {
@@ -525,14 +551,14 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 				if ( $this->add( $this->makeLockKey( $key ), 1, $exptime ) ) {
 					$lockTsUnix = microtime( true );
 
-					return WaitConditionLoop::CONDITION_REACHED; // locked!
+					return WaitConditionLoop::CONDITION_REACHED;
 				} elseif ( $this->getLastError( $watchPoint ) ) {
 					$this->logger->warning(
 						"$fname failed due to I/O error for {key}.",
 						[ 'key' => $key ]
 					);
 
-					return WaitConditionLoop::CONDITION_ABORTED; // network partition?
+					return WaitConditionLoop::CONDITION_ABORTED;
 				}
 
 				return WaitConditionLoop::CONDITION_CONTINUE;
@@ -555,6 +581,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * Release an advisory lock on a key string
 	 *
 	 * @param string $key
+	 *
 	 * @return bool Success
 	 */
 	public function unlock( $key ) {
@@ -587,24 +614,28 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @see MediumSpecificBagOStuff::unlock()
 	 *
 	 * @param string $key
+	 *
 	 * @return bool Success
 	 */
 	protected function doUnlock( $key ) {
-		// Estimate the remaining TTL of the lock key
-		$curTTL = $this->locks[$key][self::LOCK_EXPIRY] - $this->getCurrentTime();
-		// Maximum expected one-way-delay for a query to reach the backend
-		$maxOWD = 0.050;
-
 		$released = false;
 
-		if ( ( $curTTL - $maxOWD ) > 0 ) {
-			// The lock key is extremely unlikely to expire before a deletion operation
-			// sent from this method arrives on the relevant backend server
+		// Estimate the remaining TTL of the lock key
+		$curTTL = $this->locks[$key][self::LOCK_EXPIRY] - $this->getCurrentTime();
+
+		// Check the risk of race conditions for key deletion
+		if ( $this->getQoS( self::ATTR_DURABILITY ) <= self::QOS_DURABILITY_SCRIPT ) {
+			// Lock (and data) keys use memory specific to this request (e.g. HashBagOStuff)
+			$isSafe = true;
+		} else {
+			// It is unsafe to delete the lock key if there is a serious risk of the key already
+			// being claimed by another thread before the delete operation reaches the backend
+			$isSafe = ( $curTTL > $this->maxLockSendDelay );
+		}
+
+		if ( $isSafe ) {
 			$released = $this->doDelete( $this->makeLockKey( $key ) );
 		} else {
-			// It is unsafe for this method to delete the lock key due to the risk of it
-			// expiring and being claimed by another thread before the deletion operation
-			// arrives on the backend server
 			$this->logger->warning(
 				"Lock for {key} held too long ({age} sec).",
 				[ 'key' => $key, 'curTTL' => $curTTL ]
@@ -616,6 +647,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	/**
 	 * @param string $key
+	 *
 	 * @return string
 	 */
 	protected function makeLockKey( $key ) {
@@ -624,17 +656,19 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	public function deleteObjectsExpiringBefore(
 		$timestamp,
-		callable $progress = null,
+		?callable $progress = null,
 		$limit = INF,
-		string $tag = null
+		?string $tag = null
 	) {
 		return false;
 	}
 
 	/**
 	 * Get an associative array containing the item for each of the keys that have items.
+	 *
 	 * @param string[] $keys List of keys; can be a map of (unused => key) for convenience
 	 * @param int $flags Bitfield; supports READ_LATEST [optional]
+	 *
 	 * @return mixed[] Map of (key => value) for existing keys; preserves the order of $keys
 	 */
 	public function getMulti( array $keys, $flags = 0 ) {
@@ -657,8 +691,10 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	/**
 	 * Get an associative array containing the item for each of the keys that have items.
+	 *
 	 * @param string[] $keys List of keys
 	 * @param int $flags Bitfield; supports READ_LATEST [optional]
+	 *
 	 * @return array Map of (key => value) for existing keys; preserves the order of $keys
 	 */
 	protected function doGetMulti( array $keys, $flags = 0 ) {
@@ -681,6 +717,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param mixed[] $valueByKey Map of (key => value)
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
+	 *
 	 * @return bool Success
 	 * @since 1.24
 	 */
@@ -696,6 +733,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param mixed[] $data Map of (key => value)
 	 * @param int $exptime Either an interval in seconds or a unix timestamp for expiry
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	protected function doSetMulti( array $data, $exptime = 0, $flags = 0 ) {
@@ -707,19 +745,9 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		return $res;
 	}
 
-	/**
-	 * Batch deletion
-	 *
-	 * This does not support WRITE_ALLOW_SEGMENTS to avoid excessive read I/O
-	 *
-	 * @param string[] $keys List of keys
-	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
-	 * @return bool Success
-	 * @since 1.33
-	 */
 	public function deleteMulti( array $keys, $flags = 0 ) {
-		if ( $this->fieldHasFlags( $flags, self::WRITE_PRUNE_SEGMENTS ) ) {
-			throw new InvalidArgumentException( __METHOD__ . ' got WRITE_PRUNE_SEGMENTS' );
+		if ( $this->fieldHasFlags( $flags, self::WRITE_ALLOW_SEGMENTS ) ) {
+			throw new InvalidArgumentException( __METHOD__ . ' got WRITE_ALLOW_SEGMENTS' );
 		}
 
 		return $this->doDeleteMulti( $keys, $flags );
@@ -728,6 +756,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	/**
 	 * @param string[] $keys List of keys
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	protected function doDeleteMulti( array $keys, $flags = 0 ) {
@@ -735,6 +764,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		foreach ( $keys as $key ) {
 			$res = $this->doDelete( $key, $flags ) && $res;
 		}
+
 		return $res;
 	}
 
@@ -744,6 +774,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param string[] $keys List of keys
 	 * @param int $exptime TTL or UNIX timestamp
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants (since 1.33)
+	 *
 	 * @return bool Success
 	 *
 	 * @since 1.34
@@ -756,6 +787,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * @param string[] $keys List of keys
 	 * @param int $exptime TTL or UNIX timestamp
 	 * @param int $flags Bitfield of BagOStuff::WRITE_* constants
+	 *
 	 * @return bool Success
 	 */
 	protected function doChangeTTLMulti( array $keys, $exptime, $flags = 0 ) {
@@ -772,13 +804,10 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *
 	 * @param string $key
 	 * @param mixed $mainValue
+	 *
 	 * @return string|null|bool The combined string, false if missing, null on error
 	 */
 	final protected function resolveSegments( $key, $mainValue ) {
-		if ( SerializedValueContainer::isUnified( $mainValue ) ) {
-			return $this->unserialize( $mainValue->{SerializedValueContainer::UNIFIED_DATA} );
-		}
-
 		if ( SerializedValueContainer::isSegmented( $mainValue ) ) {
 			$orderedKeys = array_map(
 				function ( $segmentHash ) use ( $key ) {
@@ -794,7 +823,8 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 				if ( isset( $segmentsByKey[$segmentKey] ) ) {
 					$parts[] = $segmentsByKey[$segmentKey];
 				} else {
-					return false; // missing segment
+					// missing segment
+					return false;
 				}
 			}
 
@@ -804,32 +834,63 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		return $mainValue;
 	}
 
-	final public function addBusyCallback( callable $workCallback ) {
-		$this->busyCallbacks[] = $workCallback;
+	/**
+	 * Check if a value should use a segmentation wrapper due to its size
+	 *
+	 * In order to avoid extra serialization and/or twice-serialized wrappers, just check if
+	 * the value is a large string. Support cache wrappers (e.g. WANObjectCache) that use 2D
+	 * arrays to wrap values. This does not recurse in order to avoid overhead from complex
+	 * structures and the risk of infinite loops (due to references).
+	 *
+	 * @param mixed $value
+	 * @param int $flags
+	 *
+	 * @return bool
+	 */
+	private function useSegmentationWrapper( $value, $flags ) {
+		if (
+			$this->segmentationSize === INF ||
+			!$this->fieldHasFlags( $flags, self::WRITE_ALLOW_SEGMENTS )
+		) {
+			return false;
+		}
+
+		if ( is_string( $value ) ) {
+			return ( strlen( $value ) >= $this->segmentationSize );
+		}
+
+		if ( is_array( $value ) ) {
+			// Expect that the contained value will be one of the first array entries
+			foreach ( array_slice( $value, 0, 4 ) as $v ) {
+				if ( is_string( $v ) && strlen( $v ) >= $this->segmentationSize ) {
+					return true;
+				}
+			}
+		}
+
+		// Avoid breaking functions for incrementing/decrementing integer key values
+		return false;
 	}
 
 	/**
-	 * Determine the entry (inline or segment list) to store under a key to save the value
+	 * Make the entry to store at a key (inline or segment list), storing any segments
 	 *
 	 * @param string $key
 	 * @param mixed $value
 	 * @param int $exptime
 	 * @param int $flags
-	 * @return array (inline value or segment list, whether the entry is usable)
+	 * @param mixed|null &$ok Whether the entry is usable (e.g. no missing segments) [returned]
+	 *
+	 * @return mixed The entry (inline value, wrapped inline value, or wrapped segment list)
 	 * @since 1.34
 	 */
-	final protected function makeValueOrSegmentList( $key, $value, $exptime, $flags ) {
+	final protected function makeValueOrSegmentList( $key, $value, $exptime, $flags, &$ok ) {
 		$entry = $value;
-		$usable = true;
+		$ok = true;
 
-		if (
-			$this->fieldHasFlags( $flags, self::WRITE_ALLOW_SEGMENTS ) &&
-			!is_int( $value ) && // avoid breaking incr()/decr()
-			is_finite( $this->segmentationSize )
-		) {
+		if ( $this->useSegmentationWrapper( $value, $flags ) ) {
 			$segmentSize = $this->segmentationSize;
 			$maxTotalSize = $this->segmentedValueMaxSize;
-
 			$serialized = $this->getSerialized( $value, $key );
 			$size = strlen( $serialized );
 			if ( $size > $maxTotalSize ) {
@@ -837,9 +898,6 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 					"Value for {key} exceeds $maxTotalSize bytes; cannot segment.",
 					[ 'key' => $key ]
 				);
-			} elseif ( $size <= $segmentSize ) {
-				// The serialized value was already computed, so just use it inline
-				$entry = SerializedValueContainer::newUnified( $serialized );
 			} else {
 				// Split the serialized value into chunks and store them at different keys
 				$chunksByKey = [];
@@ -853,16 +911,17 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 					$segmentHashes[] = $hash;
 				}
 				$flags &= ~self::WRITE_ALLOW_SEGMENTS;
-				$usable = $this->setMulti( $chunksByKey, $exptime, $flags );
+				$ok = $this->setMulti( $chunksByKey, $exptime, $flags );
 				$entry = SerializedValueContainer::newSegmented( $segmentHashes );
 			}
 		}
 
-		return [ $entry, $usable ];
+		return $entry;
 	}
 
 	/**
 	 * @param int|float $exptime
+	 *
 	 * @return bool Whether the expiry is non-infinite, and, negative or not a UNIX timestamp
 	 * @since 1.34
 	 */
@@ -880,6 +939,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *   - positive (>= 10 years): absolute UNIX timestamp; return this value
 	 *
 	 * @param int $exptime
+	 *
 	 * @return int Expiration timestamp or TTL_INDEFINITE for indefinite
 	 * @since 1.34
 	 */
@@ -904,6 +964,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 *   - positive (>= 10 years): absolute UNIX timestamp; return offset to current time
 	 *
 	 * @param int $exptime
+	 *
 	 * @return int Relative TTL or TTL_INDEFINITE for indefinite
 	 * @since 1.34
 	 */
@@ -921,6 +982,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 	 * Check if a value is an integer
 	 *
 	 * @param mixed $value
+	 *
 	 * @return bool
 	 */
 	final protected function isInteger( $value ) {
@@ -935,106 +997,43 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		return ( $value === (string)$integer );
 	}
 
-	public function makeGlobalKey( $collection, ...$components ) {
-		return $this->makeKeyInternal( self::GLOBAL_KEYSPACE, func_get_args() );
-	}
-
-	public function makeKey( $collection, ...$components ) {
-		return $this->makeKeyInternal( $this->keyspace, func_get_args() );
-	}
-
-	protected function convertGenericKey( $key ) {
-		$components = $this->componentsFromGenericKey( $key );
-		if ( count( $components ) < 2 ) {
-			// Legacy key not from makeKey()/makeGlobalKey(); keep it as-is
-			return $key;
-		}
-
-		$keyspace = array_shift( $components );
-
-		return $this->makeKeyInternal( $keyspace, $components );
-	}
-
 	public function getQoS( $flag ) {
 		return $this->attrMap[$flag] ?? self::QOS_UNKNOWN;
 	}
 
+	/**
+	 * @deprecated since 1.43, not used anywhere.
+	 */
 	public function getSegmentationSize() {
+		wfDeprecated( __METHOD__, '1.43' );
+
 		return $this->segmentationSize;
 	}
 
+	/**
+	 * @deprecated since 1.43, not used anywhere.
+	 */
 	public function getSegmentedValueMaxSize() {
+		wfDeprecated( __METHOD__, '1.43' );
+
 		return $this->segmentedValueMaxSize;
 	}
 
-	public function setNewPreparedValues( array $valueByKey ) {
-		$this->preparedValues = [];
-
-		$sizes = [];
-		foreach ( $valueByKey as $key => $value ) {
-			if ( $value === false ) {
-				$sizes[] = null; // not storable, don't bother
-				continue;
-			}
-
-			$serialized = $this->serialize( $value );
-			$sizes[] = ( $serialized !== false ) ? strlen( $serialized ) : null;
-
-			$this->preparedValues[$key] = [ $value, $serialized ];
-		}
-
-		return $sizes;
-	}
-
 	/**
-	 * Get the serialized form a value, using any applicable prepared value
-	 *
-	 * @see BagOStuff::setNewPreparedValues()
+	 * Get the serialized form a value, logging a warning if it involves custom classes
 	 *
 	 * @param mixed $value
 	 * @param string $key
-	 * @return string|int String/integer representation
+	 *
+	 * @return string|int String/integer representation of value
 	 * @since 1.35
 	 */
 	protected function getSerialized( $value, $key ) {
-		// Reuse any available prepared (serialized) value
-		if ( array_key_exists( $key, $this->preparedValues ) ) {
-			list( $prepValue, $prepSerialized ) = $this->preparedValues[$key];
-			// Normally, this comparison should only take a few microseconds to confirm a match.
-			// Using "===" on variables of different types is always fast. It is also fast for
-			// variables of matching type int, float, bool, null, and object. Lastly, it is fast
-			// for comparing arrays/strings if they are copy-on-write references, which should be
-			// the case at this point, assuming prepareValues() was called correctly.
-			if ( $prepValue === $value ) {
-				unset( $this->preparedValues[$key] );
-
-				return $prepSerialized;
-			}
-		}
-
 		$this->checkValueSerializability( $value, $key );
 
 		return $this->serialize( $value );
 	}
 
-	/**
-	 * Estimate the size of a variable once serialized
-	 *
-	 * @param mixed $value
-	 * @param int $depth Current stack depth
-	 * @param int &$loops Number of iterable nodes visited
-	 * @return int|null Size in bytes; null for unsupported variable types
-	 * @since 1.35
-	 */
-	protected function guessSerialValueSize( $value, $depth = 0, &$loops = 0 ) {
-		if ( is_string( $value ) ) {
-			// E.g. "<type><delim1><quote><value><quote><delim2>"
-			return strlen( $value ) + 5;
-		} else {
-			return strlen( serialize( $value ) );
-		}
-	}
-
 	/**
 	 * Log if a new cache value does not appear suitable for serialization at a quick glance
 	 *
@@ -1080,8 +1079,8 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 			if ( is_object( $entry ) ) {
 				// Note that Closure instances count as objects
 				if (
-					!( $entry instanceof stdClass ) &&
-					!( $entry instanceof JsonSerializable )
+					!( $entry instanceof \stdClass ) &&
+					!( $entry instanceof \JsonSerializable )
 				) {
 					$this->logger->warning(
 						"{class} value for '{cachekey}' at '$index'; serialization is suspect.",
@@ -1096,6 +1095,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	/**
 	 * @param mixed $value
+	 *
 	 * @return string|int|false String/integer representation
 	 * @note Special handling is usually needed for integers so incr()/decr() work
 	 */
@@ -1105,6 +1105,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 
 	/**
 	 * @param string|int|false $value
+	 *
 	 * @return mixed Original value or false on error
 	 * @note Special handling is usually needed for integers so incr()/decr() work
 	 */
@@ -1119,6 +1120,21 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		$this->logger->debug( "{class} debug: $text", [ 'class' => static::class ] );
 	}
 
+	/**
+	 * @param string $key Key generated by BagOStuff::makeKeyInternal
+	 *
+	 * @return string A stats prefix to describe this class of key (e.g. "objectcache.file")
+	 */
+	private function determinekeyGroupForStats( $key ): string {
+		// Key came directly from BagOStuff::makeKey() or BagOStuff::makeGlobalKey()
+		// and thus has the format of "<scope>:<collection>[:<constant or variable>]..."
+		$components = explode( ':', $key, 3 );
+		// Handle legacy callers that fail to use the key building methods
+		$keygroup = $components[1] ?? 'UNKNOWN';
+
+		return strtr( $keygroup, '.', '_' );
+	}
+
 	/**
 	 * @param string $op Operation name as a MediumSpecificBagOStuff::METRIC_OP_* constant
 	 * @param array<int,string>|array<string,int[]> $keyInfo Key list, if payload sizes are not
@@ -1132,7 +1148,7 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 		foreach ( $keyInfo as $indexOrKey => $keyOrSizes ) {
 			if ( is_array( $keyOrSizes ) ) {
 				$key = $indexOrKey;
-				list( $sPayloadSize, $rPayloadSize ) = $keyOrSizes;
+				[ $sPayloadSize, $rPayloadSize ] = $keyOrSizes;
 			} else {
 				$key = $keyOrSizes;
 				$sPayloadSize = 0;
@@ -1140,30 +1156,61 @@ abstract class MediumSpecificBagOStuff extends BagOStuff {
 			}
 
 			// Metric prefix for the cache wrapper and key collection name
-			$prefix = $this->determineKeyPrefixForStats( $key );
+			$keygroup = $this->determinekeyGroupForStats( $key );
 
 			if ( $op === self::METRIC_OP_GET ) {
 				// This operation was either a "hit" or "miss" for this key
-				$name = "{$prefix}.{$op}_" . ( $rPayloadSize === false ? 'miss_rate' : 'hit_rate' );
+				if ( $rPayloadSize === false ) {
+					$statsdName = "objectcache.{$keygroup}.{$op}_miss_rate";
+					$statsName = "bagostuff_miss_total";
+				} else {
+					$statsdName = "objectcache.{$keygroup}.{$op}_hit_rate";
+					$statsName = "bagostuff_hit_total";
+				}
 			} else {
 				// There is no concept of "hit" or "miss" for this operation
-				$name = "{$prefix}.{$op}_call_rate";
+				$statsdName = "objectcache.{$keygroup}.{$op}_call_rate";
+				$statsName = "bagostuff_call_total";
 			}
-			$deltasByMetric[$name] = ( $deltasByMetric[$name] ?? 0 ) + 1;
+			$deltasByMetric[$statsdName] = [
+				'delta' => ( $deltasByMetric[$statsdName]['delta'] ?? 0 ) + 1,
+				'metric' => $statsName,
+				'keygroup' => $keygroup,
+				'operation' => $op,
+			];
 
 			if ( $sPayloadSize > 0 ) {
-				$name = "{$prefix}.{$op}_bytes_sent";
-				$deltasByMetric[$name] = ( $deltasByMetric[$name] ?? 0 ) + $sPayloadSize;
+				$statsdName = "objectcache.{$keygroup}.{$op}_bytes_sent";
+				$statsName = "bagostuff_bytes_sent_total";
+				$deltasByMetric[$statsdName] = [
+					'delta' => ( $deltasByMetric[$statsdName]['delta'] ?? 0 ) + $sPayloadSize,
+					'metric' => $statsName,
+					'keygroup' => $keygroup,
+					'operation' => $op,
+				];
 			}
 
 			if ( $rPayloadSize > 0 ) {
-				$name = "{$prefix}.{$op}_bytes_read";
-				$deltasByMetric[$name] = ( $deltasByMetric[$name] ?? 0 ) + $rPayloadSize;
+				$statsdName = "objectcache.{$keygroup}.{$op}_bytes_read";
+				$statsName = "bagostuff_bytes_read_total";
+				$deltasByMetric[$statsdName] = [
+					'delta' => ( $deltasByMetric[$statsdName]['delta'] ?? 0 ) + $rPayloadSize,
+					'metric' => $statsName,
+					'keygroup' => $keygroup,
+					'operation' => $op,
+				];
 			}
 		}
 
-		foreach ( $deltasByMetric as $name => $delta ) {
-			$this->stats->updateCount( $name, $delta );
+		foreach ( $deltasByMetric as $statsdName => $delta ) {
+			$this->stats->getCounter( $delta['metric'] )
+				->setLabel( 'keygroup', $delta['keygroup'] )
+				->setLabel( 'operation', $delta['operation'] )
+				->copyToStatsdAt( $statsdName )
+				->incrementBy( $delta['delta'] );
 		}
 	}
 }
+
+/** @deprecated class alias since 1.43 */
+class_alias( MediumSpecificBagOStuff::class, 'MediumSpecificBagOStuff' );
