@@ -16,9 +16,16 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup Media
  */
+
+use MediaWiki\Context\IContextSource;
+use MediaWiki\FileRepo\File\File;
+use MediaWiki\FileRepo\File\LocalFile;
+use MediaWiki\HookContainer\HookRunner;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Parser\Parser;
+use MediaWiki\Status\Status;
+use Wikimedia\FileBackend\FSFile\FSFile;
 
 /**
  * @defgroup Media Media
@@ -31,7 +38,6 @@ use MediaWiki\MediaWikiServices;
  * Base media handler class
  *
  * @stable to extend
- *
  * @ingroup Media
  */
 abstract class MediaHandler {
@@ -48,7 +54,7 @@ abstract class MediaHandler {
 	 * Get a MediaHandler for a given MIME type from the instance cache
 	 *
 	 * @param string $type
-	 * @return MediaHandler|bool
+	 * @return MediaHandler|false
 	 */
 	public static function getHandler( $type ) {
 		return MediaWikiServices::getInstance()
@@ -58,6 +64,7 @@ abstract class MediaHandler {
 	/**
 	 * Get an associative array mapping magic word IDs to parameter names.
 	 * Will be used by the parser to identify parameters.
+	 * @return string[]
 	 */
 	abstract public function getParamMap();
 
@@ -68,6 +75,7 @@ abstract class MediaHandler {
 	 *
 	 * @param string $name
 	 * @param mixed $value
+	 * @return bool
 	 */
 	abstract public function validateParam( $name, $value );
 
@@ -83,7 +91,7 @@ abstract class MediaHandler {
 	 * Parse a param string made with makeParamString back into an array
 	 *
 	 * @param string $str The parameter string without file name (e.g. 122px)
-	 * @return array|bool Array of parameters or false on failure.
+	 * @return array|false Array of parameters or false on failure.
 	 */
 	abstract public function parseParamString( $str );
 
@@ -93,6 +101,7 @@ abstract class MediaHandler {
 	 * Returns false if the parameters are unacceptable and the transform should fail
 	 * @param File $image
 	 * @param array &$params
+	 * @return bool
 	 */
 	abstract public function normaliseParams( $image, &$params );
 
@@ -109,7 +118,7 @@ abstract class MediaHandler {
 	 *
 	 * @deprecated since 1.37, override getSizeAndMetadata instead
 	 *
-	 * @param File|FSFile $image The image object, or false if there isn't one.
+	 * @param File|FSFile|false $image The image object, or false if there isn't one.
 	 *   Warning, FSFile::getPropsFromPath might pass an FSFile instead of File (!)
 	 * @param string $path The filename
 	 * @return array|false Follow the format of PHP getimagesize() internal function.
@@ -155,7 +164,7 @@ abstract class MediaHandler {
 	 * Get handler-specific metadata which will be saved in the img_metadata field.
 	 * @deprecated since 1.37 override getSizeAndMetadata() instead
 	 *
-	 * @param File|FSFile $image The image object, or false if there isn't one.
+	 * @param File|FSFile|false $image The image object, or false if there isn't one.
 	 *   Warning, FSFile::getPropsFromPath might pass an FSFile instead of File (!)
 	 * @param string $path The filename
 	 * @return string A string of metadata in php serialized form (Run through serialize())
@@ -226,7 +235,7 @@ abstract class MediaHandler {
 				$info += [ 'width' => 0, 'height' => 0, 'metadata' => [] ];
 				if ( !is_array( $info['metadata'] ) ) {
 					throw new InvalidArgumentException( 'Media handler ' .
-						static::class . ' returned ' . gettype( $info['metadata'] ) .
+						static::class . ' returned ' . get_debug_type( $info['metadata'] ) .
 						' for metadata, should be array' );
 				}
 				return $info;
@@ -289,7 +298,7 @@ abstract class MediaHandler {
 	 */
 	public static function getMetadataVersion() {
 		$version = [ '2' ]; // core metadata version
-		Hooks::runner()->onGetMetadataVersion( $version );
+		( new HookRunner( MediaWikiServices::getInstance()->getHookContainer() ) )->onGetMetadataVersion( $version );
 
 		return implode( ';', $version );
 	}
@@ -302,7 +311,7 @@ abstract class MediaHandler {
 	 * @stable to override
 	 *
 	 * @param array $metadata Metadata array
-	 * @param int $version Target version
+	 * @param int|string $version Target version
 	 * @return array Serialized metadata in specified version, or $metadata on fail.
 	 */
 	public function convertMetadataVersion( $metadata, $version = 1 ) {
@@ -315,7 +324,7 @@ abstract class MediaHandler {
 	 *
 	 * @note This method is currently unused.
 	 * @param File $image
-	 * @return string
+	 * @return string|false
 	 */
 	public function getMetadataType( $image ) {
 		return false;
@@ -403,7 +412,7 @@ abstract class MediaHandler {
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return array|bool False if interface not supported
+	 * @return array|false False if interface not supported
 	 * @since 1.23
 	 */
 	public function getCommonMetaArray( File $file ) {
@@ -423,7 +432,7 @@ abstract class MediaHandler {
 	 * @param File $image
 	 * @param string $script
 	 * @param array $params
-	 * @return bool|ThumbnailImage
+	 * @return ThumbnailImage|false
 	 */
 	public function getScriptedTransform( $image, $script, $params ) {
 		return false;
@@ -472,7 +481,7 @@ abstract class MediaHandler {
 	 * @return array Thumbnail extension and MIME type
 	 */
 	public function getThumbType( $ext, $mime, $params = null ) {
-		$magic = MediaWiki\MediaWikiServices::getInstance()->getMimeAnalyzer();
+		$magic = MediaWikiServices::getInstance()->getMimeAnalyzer();
 		if ( !$ext || $magic->isMatchingExtension( $ext, $mime ) === false ) {
 			// The extension is not valid for this MIME type and we do
 			// recognize the MIME type
@@ -600,7 +609,7 @@ abstract class MediaHandler {
 	 *
 	 * @param File $image
 	 * @param int $page What page to get dimensions of
-	 * @return array|bool
+	 * @return array|false
 	 */
 	public function getPageDimensions( File $image, $page ) {
 		return false;
@@ -613,7 +622,7 @@ abstract class MediaHandler {
 	 *
 	 * @param File $image
 	 * @param int $page Page number to get information for
-	 * @return bool|string Page text or false when no text found or if
+	 * @return string|false Page text or false when no text found or if
 	 *   unsupported.
 	 */
 	public function getPageText( File $image, $page ) {
@@ -623,7 +632,7 @@ abstract class MediaHandler {
 	/**
 	 * Get the text of the entire document.
 	 * @param File $file
-	 * @return bool|string The text of the document or false if unsupported.
+	 * @return string|false The text of the document or false if unsupported.
 	 */
 	public function getEntireText( File $file ) {
 		$numPages = $file->pageCount();
@@ -739,7 +748,7 @@ abstract class MediaHandler {
 	 *
 	 * @param string $key The metadata field key
 	 * @param string|array $vals The unformatted value of this metadata field
-	 * @param bool|IContextSource $context Context to use (optional)
+	 * @param IContextSource|false $context Context to use (optional)
 	 * @return false|null|string|array False to use default formatting, null
 	 *   to remove this tag from the formatted list; otherwise return
 	 *   a formatted HTML string (or array of them).
@@ -784,7 +793,7 @@ abstract class MediaHandler {
 	 * Note, everything here is passed through the parser later on (!)
 	 */
 	protected static function addMeta( &$array, $visibility, $type, $id, $value, $param = false ) {
-		$msg = wfMessage( "$type-$id", $param );
+		$msg = wfMessage( "$type-$id", (string)$param );
 		if ( $msg->exists() ) {
 			$name = $msg->text();
 		} else {
@@ -809,19 +818,19 @@ abstract class MediaHandler {
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return string
+	 * @return string HTML
 	 */
 	public function getShortDesc( $file ) {
 		return self::getGeneralShortDesc( $file );
 	}
 
 	/**
-	 * Long description. Shown under image on image description page surounded by ().
+	 * Long description. Shown under image on image description page surrounded by ().
 	 *
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return string
+	 * @return string HTML
 	 */
 	public function getLongDesc( $file ) {
 		return self::getGeneralLongDesc( $file );
@@ -831,7 +840,7 @@ abstract class MediaHandler {
 	 * Used instead of getShortDesc if there is no handler registered for file.
 	 *
 	 * @param File $file
-	 * @return string
+	 * @return string HTML
 	 */
 	public static function getGeneralShortDesc( $file ) {
 		global $wgLang;
@@ -843,7 +852,7 @@ abstract class MediaHandler {
 	 * Used instead of getLongDesc if there is no handler registered for file.
 	 *
 	 * @param File $file
-	 * @return string
+	 * @return string HTML
 	 */
 	public static function getGeneralLongDesc( $file ) {
 		return wfMessage( 'file-info' )->sizeParams( $file->getSize() )
@@ -862,10 +871,9 @@ abstract class MediaHandler {
 		$idealWidth = $boxWidth * $maxHeight / $boxHeight;
 		$roundedUp = ceil( $idealWidth );
 		if ( round( $roundedUp * $boxHeight / $boxWidth ) > $maxHeight ) {
-			return floor( $idealWidth );
-		} else {
-			return $roundedUp;
+			return (int)floor( $idealWidth );
 		}
+		return $roundedUp;
 	}
 
 	/**
@@ -874,7 +882,7 @@ abstract class MediaHandler {
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return string Dimensions
+	 * @return string Dimensions (plain text)
 	 */
 	public function getDimensionsString( $file ) {
 		return '';
@@ -1019,7 +1027,7 @@ abstract class MediaHandler {
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return string[] Array of language codes, or empty array if unsupported.
+	 * @return string[] Array of IETF language codes, or empty array if unsupported.
 	 * @since 1.23
 	 */
 	public function getAvailableLanguages( File $file ) {
@@ -1033,9 +1041,9 @@ abstract class MediaHandler {
 	 *
 	 * @since 1.32
 	 *
-	 * @param string $userPreferredLanguage Language code requesed
-	 * @param string[] $availableLanguages Languages present in the file
-	 * @return string|null Language code picked or null if not supported/available
+	 * @param string $userPreferredLanguage IETF Language code requested
+	 * @param string[] $availableLanguages IETF Languages present in the file
+	 * @return string|null IETF Language code picked or null if not supported/available
 	 */
 	public function getMatchedLanguage( $userPreferredLanguage, array $availableLanguages ) {
 		return null;
@@ -1048,11 +1056,12 @@ abstract class MediaHandler {
 	 * If getAvailableLanguages returns a non-empty array, this must return
 	 * a valid language code. Otherwise can return null if files of this
 	 * type do not support alternative language renderings.
+	 * It can also return 'und' for explicitly requesting an undetermined language
 	 *
 	 * @stable to override
 	 *
 	 * @param File $file
-	 * @return string|null Language code or null if multi-language not supported for filetype.
+	 * @return string|null IETF Language code or null if multi-language not supported for filetype.
 	 * @since 1.23
 	 */
 	public function getDefaultRenderLanguage( File $file ) {
@@ -1060,7 +1069,7 @@ abstract class MediaHandler {
 	}
 
 	/**
-	 * If its an audio file, return the length of the file. Otherwise 0.
+	 * If it's an audio file, return the length of the file. Otherwise 0.
 	 *
 	 * File::getLength() existed for a long time, but was calling a method
 	 * that only existed in some subclasses of this class (The TMH ones).
@@ -1157,7 +1166,7 @@ abstract class MediaHandler {
 
 			foreach ( $pageList as $page ) {
 				if ( $page > $lastPage + 1 ) {
-					if ( $firstPage != $lastPage ) {
+					if ( $firstPage !== $lastPage ) {
 						$ranges[] = "$firstPage-$lastPage";
 					} else {
 						$ranges[] = "$firstPage";
