@@ -2,7 +2,7 @@
 /**
  * HTML sanitizer for %MediaWiki.
  *
- * Copyright © 2002-2005 Brion Vibber <brion@pobox.com> et al
+ * Copyright © 2002-2005 Brooke Vibber <bvibber@wikimedia.org> et al
  * https://www.mediawiki.org/
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,15 +24,20 @@
  * @ingroup Parser
  */
 
+namespace MediaWiki\Parser;
+
+use InvalidArgumentException;
+use LogicException;
+use MediaWiki\HookContainer\HookRunner;
 use MediaWiki\MediaWikiServices;
-use MediaWiki\Parser\RemexRemoveTagHandler;
-use MediaWiki\Parser\RemexStripTagHandler;
 use MediaWiki\Tidy\RemexCompatFormatter;
+use UnexpectedValueException;
 use Wikimedia\RemexHtml\HTMLData;
 use Wikimedia\RemexHtml\Serializer\Serializer as RemexSerializer;
 use Wikimedia\RemexHtml\Tokenizer\Tokenizer as RemexTokenizer;
 use Wikimedia\RemexHtml\TreeBuilder\Dispatcher as RemexDispatcher;
 use Wikimedia\RemexHtml\TreeBuilder\TreeBuilder as RemexTreeBuilder;
+use Wikimedia\StringUtils\StringUtils;
 
 /**
  * HTML sanitizer for MediaWiki
@@ -47,9 +52,9 @@ class Sanitizer {
 	 */
 	private const CHAR_REFS_REGEX =
 		'/&([A-Za-z0-9\x80-\xff]+;)
-		 |&\#([0-9]+);
-		 |&\#[xX]([0-9A-Fa-f]+);
-		 |(&)/x';
+		|&\#([0-9]+);
+		|&\#[xX]([0-9A-Fa-f]+);
+		|&/x';
 
 	/**
 	 * Acceptable tag name charset from HTML5 parsing spec
@@ -96,15 +101,14 @@ class Sanitizer {
 	/**
 	 * Lazy-initialised attributes regex, see getAttribsRegex()
 	 */
-	private static $attribsRegex;
+	private static ?string $attribsRegex = null;
 
 	/**
 	 * Regular expression to match HTML/XML attribute pairs within a tag.
 	 * Based on https://www.w3.org/TR/html5/syntax.html#before-attribute-name-state
 	 * Used in Sanitizer::decodeTagAttributes
-	 * @return string
 	 */
-	private static function getAttribsRegex() {
+	private static function getAttribsRegex(): string {
 		if ( self::$attribsRegex === null ) {
 			$spaceChars = '\x09\x0a\x0c\x0d\x20';
 			$space = "[{$spaceChars}]";
@@ -127,13 +131,12 @@ class Sanitizer {
 	/**
 	 * Lazy-initialised attribute name regex, see getAttribNameRegex()
 	 */
-	private static $attribNameRegex;
+	private static ?string $attribNameRegex = null;
 
 	/**
 	 * Used in Sanitizer::decodeTagAttributes to filter attributes.
-	 * @return string
 	 */
-	private static function getAttribNameRegex() {
+	private static function getAttribNameRegex(): string {
 		if ( self::$attribNameRegex === null ) {
 			$attribFirst = "[:_\p{L}\p{N}]";
 			$attrib = "[:_\.\-\p{L}\p{N}]";
@@ -149,21 +152,17 @@ class Sanitizer {
 	 * @return array
 	 * @internal
 	 */
-	public static function getRecognizedTagData( $extratags = [], $removetags = [] ) {
-		global $wgAllowImageTag;
-		static $commonCase, $staticInitialised;
+	public static function getRecognizedTagData( array $extratags = [], array $removetags = [] ): array {
+		static $commonCase, $staticInitialised = false;
 		$isCommonCase = ( $extratags === [] && $removetags === [] );
-		if ( $staticInitialised === $wgAllowImageTag && $isCommonCase && $commonCase ) {
+		if ( $staticInitialised && $isCommonCase && $commonCase ) {
 			return $commonCase;
 		}
 
 		static $htmlpairsStatic, $htmlsingle, $htmlsingleonly, $htmlnest, $tabletags,
 			$htmllist, $listtags, $htmlsingleallowed, $htmlelementsStatic;
 
-		// Base our staticInitialised variable off of the global config state so that if the globals
-		// are changed (like in the screwed up test system) we will re-initialise the settings.
-		$globalContext = $wgAllowImageTag;
-		if ( !$staticInitialised || $staticInitialised !== $globalContext ) {
+		if ( !$staticInitialised ) {
 			$htmlpairsStatic = [ # Tags that must be closed
 				'b', 'bdi', 'del', 'i', 'ins', 'u', 'font', 'big', 'small', 'sub', 'sup', 'h1',
 				'h2', 'h3', 'h4', 'h5', 'h6', 'cite', 'code', 'em', 's',
@@ -203,13 +202,6 @@ class Sanitizer {
 				'li',
 			];
 
-			if ( $wgAllowImageTag ) {
-				wfDeprecatedMsg( 'Setting $wgAllowImageTag to true ' .
-					'is deprecated since MediaWiki 1.35', '1.35', false, false );
-				$htmlsingle[] = 'img';
-				$htmlsingleonly[] = 'img';
-			}
-
 			$htmlsingleallowed = array_unique( array_merge( $htmlsingle, $tabletags ) );
 			$htmlelementsStatic = array_unique( array_merge( $htmlsingle, $htmlpairsStatic, $htmlnest ) );
 
@@ -219,7 +211,7 @@ class Sanitizer {
 			foreach ( $vars as $var ) {
 				$$var = array_fill_keys( $$var, true );
 			}
-			$staticInitialised = $globalContext;
+			$staticInitialised = true;
 		}
 
 		# Populate $htmlpairs and $htmlelements with the $extratags and $removetags arrays
@@ -245,45 +237,6 @@ class Sanitizer {
 		return $result;
 	}
 
-	/**
-	 * Cleans up HTML, removes dangerous tags and attributes, and
-	 * removes HTML comments; BEWARE there may be unmatched HTML
-	 * tags in the result.
-	 *
-	 * @note Callers are recommended to use `::removeSomeTags()`
-	 * instead of this method.  `Sanitizer::removeSomeTags()` is safer
-	 * and will always return well-formed HTML; however, it is
-	 * significantly slower (especially for short strings where setup
-	 * costs predominate).  This method, although faster, should only
-	 * be used where we know the result be cleaned up in a subsequent
-	 * tidy pass.
-	 *
-	 * @param string $text Original string; see T268353 for why untainted.
-	 * @param-taint $text none
-	 * @param callable|null $processCallback Callback to do any variable or
-	 *   parameter replacements in HTML attribute values.
-	 *   This argument should be considered @internal.
-	 * @param-taint $processCallback exec_shell
-	 * @param array|bool $args Arguments for the processing callback
-	 * @param-taint $args none
-	 * @param array $extratags For any extra tags to include
-	 * @param-taint $extratags tainted
-	 * @param array $removetags For any tags (default or extra) to exclude
-	 * @param-taint $removetags none
-	 * @return string
-	 * @return-taint escaped
-	 * @deprecated since 1.38. Use ::removeSomeTags(), which always gives
-	 * balanced/tidy HTML.
-	 */
-	public static function removeHTMLtags( $text, $processCallback = null,
-		$args = [], $extratags = [], $removetags = []
-	) {
-		wfDeprecated( __METHOD__, '1.38' );
-		return self::internalRemoveHtmlTags(
-			$text, $processCallback, $args, $extratags, $removetags
-		);
-	}
-
 	/**
 	 * Cleans up HTML, removes dangerous tags and attributes, and
 	 * removes HTML comments; BEWARE there may be unmatched HTML
@@ -312,9 +265,9 @@ class Sanitizer {
 	 * @return-taint escaped
 	 * @internal
 	 */
-	public static function internalRemoveHtmlTags( $text, $processCallback = null,
-		$args = [], $extratags = [], $removetags = []
-	) {
+	public static function internalRemoveHtmlTags( string $text, ?callable $processCallback = null,
+		$args = [], array $extratags = [], array $removetags = []
+	): string {
 		$tagData = self::getRecognizedTagData( $extratags, $removetags );
 		$htmlsingle = $tagData['htmlsingle'];
 		$htmlsingleonly = $tagData['htmlsingleonly'];
@@ -328,13 +281,13 @@ class Sanitizer {
 		# this might be possible using remex tidy itself
 		foreach ( $bits as $x ) {
 			if ( preg_match( self::ELEMENT_BITS_REGEX, $x, $regs ) ) {
-				list( /* $qbar */, $slash, $t, $params, $brace, $rest ) = $regs;
+				[ /* $qbar */, $slash, $t, $params, $brace, $rest ] = $regs;
 
 				$badtag = false;
 				$t = strtolower( $t );
 				if ( isset( $htmlelements[$t] ) ) {
 					if ( is_callable( $processCallback ) ) {
-						call_user_func_array( $processCallback, [ &$params, $args ] );
+						$processCallback( $params, $args );
 					}
 
 					if ( $brace == '/>' && !( isset( $htmlsingle[$t] ) || isset( $htmlsingleonly[$t] ) ) ) {
@@ -395,7 +348,6 @@ class Sanitizer {
 		// These options are @internal:
 		$attrCallback = $options['attrCallback'] ?? null;
 		$attrCallbackArgs = $options['attrCallbackArgs'] ?? [];
-		$tidy = $options['tidy'] ?? true;
 
 		// This disallows HTML5-style "missing trailing semicolon" attributes
 		// In wikitext "clean&copy" does *not* contain an entity.
@@ -433,11 +385,9 @@ class Sanitizer {
 	 * To avoid leaving blank lines, when a comment is both preceded
 	 * and followed by a newline (ignoring spaces), trim leading and
 	 * trailing spaces and one of the newlines.
-	 *
-	 * @param string $text
-	 * @return string
 	 */
-	public static function removeHTMLcomments( $text ) {
+	public static function removeHTMLcomments( string $text ): string {
+		// phpcs:ignore Generic.CodeAnalysis.AssignmentInCondition.FoundInWhileCondition
 		while ( ( $start = strpos( $text, '<!--' ) ) !== false ) {
 			$end = strpos( $text, '-->', $start + 4 );
 			if ( $end === false ) {
@@ -479,13 +429,9 @@ class Sanitizer {
 	 * where we may want to allow a tag within content but ONLY when it has
 	 * specific attributes set.
 	 *
-	 * @param string $params
-	 * @param string $element
-	 * @return bool
-	 *
 	 * @see RemexRemoveTagHandler::validateTag()
 	 */
-	private static function validateTag( $params, $element ) {
+	private static function validateTag( string $params, string $element ): bool {
 		$params = self::decodeTagAttributes( $params );
 
 		if ( $element == 'meta' || $element == 'link' ) {
@@ -514,14 +460,10 @@ class Sanitizer {
 	 * - Unsafe style attributes are discarded
 	 * - Invalid id attributes are re-encoded
 	 *
-	 * @param array $attribs
-	 * @param string $element
-	 * @return array
-	 *
 	 * @todo Check for legal values where the DTD limits things.
 	 * @todo Check for unique id attribute :P
 	 */
-	public static function validateTagAttributes( $attribs, $element ) {
+	public static function validateTagAttributes( array $attribs, string $element ): array {
 		return self::validateAttributes( $attribs,
 			self::attributesAllowedInternal( $element ) );
 	}
@@ -544,14 +486,15 @@ class Sanitizer {
 	 * @todo Check for legal values where the DTD limits things.
 	 * @todo Check for unique id attribute :P
 	 */
-	public static function validateAttributes( $attribs, $allowed ) {
+	public static function validateAttributes( array $attribs, array $allowed ): array {
 		if ( isset( $allowed[0] ) ) {
 			// Calling this function with a sequential array is
 			// deprecated.  For now just convert it.
 			wfDeprecated( __METHOD__ . ' with sequential array', '1.35' );
 			$allowed = array_fill_keys( $allowed, true );
 		}
-		$hrefExp = '/^(' . wfUrlProtocols() . ')[^\s]+$/';
+		$validProtocols = MediaWikiServices::getInstance()->getUrlUtils()->validProtocols();
+		$hrefExp = '/^(' . $validProtocols . ')[^\s]+$/';
 
 		$out = [];
 		foreach ( $attribs as $attribute => $value ) {
@@ -569,8 +512,13 @@ class Sanitizer {
 			# * Disallow data attributes used by MediaWiki code
 			# * Ensure that the attribute is not namespaced by banning
 			#   colons.
+			# * Ensure attribute name will be accepted by the HTML
+			#   parser; see
+			#   https://github.com/whatwg/dom/issues/849#issuecomment-1007541209
+			# * Underscore and double-wide underscore (U+FF3F) is disallowed
+			#   here (but not in Parsoid): T407131
 			if ( (
-				!preg_match( '/^data-[^:]*$/i', $attribute ) &&
+				!preg_match( '|^data-[^:= \t\r\n/>\0_＿]*$|i', $attribute ) &&
 				!array_key_exists( $attribute, $allowed )
 			) || self::isReservedDataAttribute( $attribute ) ) {
 				continue;
@@ -585,6 +533,9 @@ class Sanitizer {
 			# Escape HTML id attributes
 			if ( $attribute === 'id' ) {
 				$value = self::escapeIdForAttribute( $value, self::ID_PRIMARY );
+				if ( $value === false || $value === '' ) {
+					continue;
+				}
 			}
 
 			# Escape HTML id reference lists
@@ -650,7 +601,7 @@ class Sanitizer {
 	 * @param string $attr Attribute name.
 	 * @return bool
 	 */
-	public static function isReservedDataAttribute( $attr ) {
+	public static function isReservedDataAttribute( string $attr ): bool {
 		// data-ooui is reserved for ooui.
 		// data-mw and data-parsoid are reserved for parsoid.
 		// data-mw-<name here> is reserved for extensions (or core) if
@@ -667,11 +618,10 @@ class Sanitizer {
 	 * will be combined (if they're both strings).
 	 *
 	 * @todo implement merging for other attributes such as style
-	 * @param array $a
-	 * @param array $b
-	 * @return array
+	 * @deprecated since 1.44, use array_merge() for attribute arrays and
+	 *   Html::addClass() / Html::expandClassList() for class lists
 	 */
-	public static function mergeAttributes( $a, $b ) {
+	public static function mergeAttributes( array $a, array $b ): array {
 		$out = array_merge( $a, $b );
 		if ( isset( $a['class'] ) && isset( $b['class'] )
 			&& is_string( $a['class'] ) && is_string( $b['class'] )
@@ -692,7 +642,7 @@ class Sanitizer {
 	 * @param string $value the css string
 	 * @return string normalized css
 	 */
-	public static function normalizeCss( $value ) {
+	public static function normalizeCss( string $value ): string {
 		// Decode character references like &#123;
 		$value = self::decodeCharReferences( $value );
 
@@ -759,6 +709,8 @@ class Sanitizer {
 	 * clever input strings. These character references must
 	 * be escaped before the return value is embedded in HTML.
 	 *
+	 * @warning This method is intended to sanitize style attributes on
+	 *  html tags only. It is not safe to use on full CSS files.
 	 * @param string $value
 	 * @return string
 	 */
@@ -767,36 +719,33 @@ class Sanitizer {
 
 		// Reject problematic keywords and control characters
 		if ( preg_match( '/[\000-\010\013\016-\037\177]/', $value ) ||
-			strpos( $value, UtfNormal\Constants::UTF8_REPLACEMENT ) !== false ) {
+			strpos( $value, \UtfNormal\Constants::UTF8_REPLACEMENT ) !== false ) {
 			return '/* invalid control char */';
 		} elseif ( preg_match(
 			'! expression
-				| filter\s*:
 				| accelerator\s*:
 				| -o-link\s*:
 				| -o-link-source\s*:
 				| -o-replace\s*:
 				| url\s*\(
+				| src\s*\(
 				| image\s*\(
 				| image-set\s*\(
 				| attr\s*\([^)]+[\s,]+url
-				| var\s*\(
 			!ix', $value ) ) {
 			return '/* insecure input */';
 		}
 		return $value;
 	}
 
-	/**
-	 * @param array $matches
-	 * @return string
-	 */
-	private static function cssDecodeCallback( $matches ) {
+	private static function cssDecodeCallback( array $matches ): string {
 		if ( $matches[1] !== '' ) {
 			// Line continuation
 			return '';
 		} elseif ( $matches[2] !== '' ) {
-			$char = UtfNormal\Utils::codepointToUtf8( hexdec( $matches[2] ) );
+			# hexdec could return a float if the match is too long, but the
+			# regexp in question limits the string length to 6.
+			$char = \UtfNormal\Utils::codepointToUtf8( hexdec( $matches[2] ) );
 		} elseif ( $matches[3] !== '' ) {
 			$char = $matches[3];
 		} else {
@@ -833,7 +782,7 @@ class Sanitizer {
 	 * @param bool $sorted Whether to sort the attributes (default: false)
 	 * @return string
 	 */
-	public static function fixTagAttributes( $text, $element, $sorted = false ) {
+	public static function fixTagAttributes( string $text, string $element, bool $sorted = false ): string {
 		if ( trim( $text ) == '' ) {
 			return '';
 		}
@@ -851,9 +800,11 @@ class Sanitizer {
 	/**
 	 * Encode an attribute value for HTML output.
 	 * @param string $text
+	 * @param-taint $text escapes_html
 	 * @return string HTML-encoded text fragment
+	 * @return-taint escaped
 	 */
-	public static function encodeAttribute( $text ) {
+	public static function encodeAttribute( string $text ): string {
 		$encValue = htmlspecialchars( $text, ENT_QUOTES );
 
 		// Whitespace is normalized during attribute decoding,
@@ -876,7 +827,7 @@ class Sanitizer {
 	 * @param string $space Space character for the French spaces, defaults to '&#160;'
 	 * @return string Armored text
 	 */
-	public static function armorFrenchSpaces( $text, $space = '&#160;' ) {
+	public static function armorFrenchSpaces( string $text, string $space = '&#160;' ): string {
 		// Replace $ with \$ and \ with \\
 		$space = preg_replace( '#(?<!\\\\)(\\$|\\\\)#', '\\\\$1', $space );
 		$fixtags = [
@@ -893,17 +844,21 @@ class Sanitizer {
 	 * Encode an attribute value for HTML tags, with extra armoring
 	 * against further wiki processing.
 	 * @param string $text
+	 * @param-taint $text escapes_html
 	 * @return string HTML-encoded text fragment
+	 * @return-taint escaped
 	 */
-	public static function safeEncodeAttribute( $text ) {
+	public static function safeEncodeAttribute( string $text ): string {
 		$encValue = self::encodeAttribute( $text );
 
 		# Templates and links may be expanded in later parsing,
 		# creating invalid or dangerous output. Suppress this.
 		$encValue = strtr( $encValue, [
-			'<'    => '&lt;',   // This should never happen,
-			'>'    => '&gt;',   // we've received invalid input
-			'"'    => '&quot;', // which should have been escaped.
+			// '<', '>', and '"' should never happen, as they indicate that we've received invalid input which should
+			// have been escaped.
+			'<'    => '&lt;',
+			'>'    => '&gt;',
+			'"'    => '&quot;',
 			'{'    => '&#123;',
 			'}'    => '&#125;', // prevent unpaired language conversion syntax
 			'['    => '&#91;',
@@ -913,12 +868,14 @@ class Sanitizer {
 			'RFC'  => '&#82;FC',
 			'PMID' => '&#80;MID',
 			'|'    => '&#124;',
-			'__'   => '&#95;_',
+			'_'    => '&#95;',
+			'＿'    => '&#xFF3F;', // Japanese magic words
 		] );
 
 		# Stupid hack
+		$validProtocols = MediaWikiServices::getInstance()->getUrlUtils()->validProtocols();
 		$encValue = preg_replace_callback(
-			'/((?i)' . wfUrlProtocols() . ')/',
+			'/((?i)' . $validProtocols . ')/',
 			static function ( $matches ) {
 				return str_replace( ':', '&#58;', $matches[1] );
 			},
@@ -936,12 +893,12 @@ class Sanitizer {
 	 * @param string $id String to escape
 	 * @param int $mode One of ID_* constants, specifying whether the primary or fallback encoding
 	 *     should be used.
-	 * @return string|bool Escaped ID or false if fallback encoding is requested but it's not
+	 * @return string|false Escaped ID or false if fallback encoding is requested but it's not
 	 *     configured.
 	 *
 	 * @since 1.30
 	 */
-	public static function escapeIdForAttribute( $id, $mode = self::ID_PRIMARY ) {
+	public static function escapeIdForAttribute( string $id, int $mode = self::ID_PRIMARY ) {
 		global $wgFragmentMode;
 
 		if ( !isset( $wgFragmentMode[$mode] ) ) {
@@ -968,7 +925,7 @@ class Sanitizer {
 	 *
 	 * @since 1.30
 	 */
-	public static function escapeIdForLink( $id ) {
+	public static function escapeIdForLink( string $id ): string {
 		global $wgFragmentMode;
 
 		if ( !isset( $wgFragmentMode[self::ID_PRIMARY] ) ) {
@@ -991,7 +948,7 @@ class Sanitizer {
 	 *
 	 * @since 1.30
 	 */
-	public static function escapeIdForExternalInterwiki( $id ) {
+	public static function escapeIdForExternalInterwiki( string $id ): string {
 		global $wgExternalInterwikiFragmentMode;
 
 		$id = self::escapeIdInternalUrl( $id, $wgExternalInterwikiFragmentMode );
@@ -1008,7 +965,7 @@ class Sanitizer {
 	 * @param string $mode One of modes from $wgFragmentMode
 	 * @return string
 	 */
-	private static function escapeIdInternalUrl( $id, $mode ) {
+	private static function escapeIdInternalUrl( string $id, string $mode ): string {
 		$id = self::escapeIdInternal( $id, $mode );
 		if ( $mode === 'html5' ) {
 			$id = preg_replace( '/%([a-fA-F0-9]{2})/', '%25$1', $id );
@@ -1023,7 +980,7 @@ class Sanitizer {
 	 * @param string $mode One of modes from $wgFragmentMode
 	 * @return string
 	 */
-	private static function escapeIdInternal( $id, $mode ) {
+	private static function escapeIdInternal( string $id, string $mode ): string {
 		// Truncate overly-long IDs.  This isn't an HTML limit, it's just
 		// griefer protection. [T251506]
 		$id = mb_substr( $id, 0, 1024 );
@@ -1057,25 +1014,10 @@ class Sanitizer {
 	 * Given a string containing a space delimited list of ids, escape each id
 	 * to match ids escaped by the escapeIdForAttribute() function.
 	 *
-	 * @since 1.27
-	 * @deprecated since 1.36. Unused outside this class, will be made private.
-	 *
 	 * @param string $referenceString Space delimited list of ids
 	 * @return string
 	 */
-	public static function escapeIdReferenceList( $referenceString ) {
-		wfDeprecated( __METHOD__, '1.36' );
-		return self::escapeIdReferenceListInternal( $referenceString );
-	}
-
-	/**
-	 * Given a string containing a space delimited list of ids, escape each id
-	 * to match ids escaped by the escapeIdForAttribute() function.
-	 *
-	 * @param string $referenceString Space delimited list of ids
-	 * @return string
-	 */
-	private static function escapeIdReferenceListInternal( $referenceString ) {
+	private static function escapeIdReferenceListInternal( string $referenceString ): string {
 		# Explode the space delimited list string into an array of tokens
 		$references = preg_split( '/\s+/', "{$referenceString}", -1, PREG_SPLIT_NO_EMPTY );
 
@@ -1098,11 +1040,8 @@ class Sanitizer {
 	 * @todo For extra validity, input should be validated UTF-8.
 	 *
 	 * @see https://www.w3.org/TR/CSS21/syndata.html Valid characters/format
-	 *
-	 * @param string $class
-	 * @return string
 	 */
-	public static function escapeClass( $class ) {
+	public static function escapeClass( string $class ): string {
 		// Convert ugly stuff to underscores and kill underscores in ugly places
 		return rtrim( preg_replace(
 			[ '/(^[0-9\\-])|[\\x00-\\x20!"#$%&\'()*+,.\\/:;<=>?@[\\]^`{|}~]|\\xC2\\xA0/', '/_+/' ],
@@ -1110,31 +1049,36 @@ class Sanitizer {
 			$class ), '_' );
 	}
 
+	public static function escapeCombiningChar( string $html ): string {
+		return strtr( $html, [
+			"\u{0338}" => '&#x338;', # T387130
+		] );
+	}
+
 	/**
 	 * Given HTML input, escape with htmlspecialchars but un-escape entities.
 	 * This allows (generally harmless) entities like &#160; to survive.
 	 *
 	 * @param string $html HTML to escape
+	 * @param-taint $html escapes_htmlnoent
 	 * @return string Escaped input
+	 * @return-taint escaped
 	 */
-	public static function escapeHtmlAllowEntities( $html ) {
+	public static function escapeHtmlAllowEntities( string $html ): string {
 		$html = self::decodeCharReferences( $html );
 		# It seems wise to escape ' as well as ", as a matter of course.  Can't
 		# hurt. Use ENT_SUBSTITUTE so that incorrectly truncated multibyte characters
 		# don't cause the entire string to disappear.
 		$html = htmlspecialchars( $html, ENT_QUOTES | ENT_SUBSTITUTE );
-		return $html;
+		return self::escapeCombiningChar( $html );
 	}
 
 	/**
 	 * Return an associative array of attribute names and values from
 	 * a partial tag string. Attribute names are forced to lowercase,
 	 * character references are decoded to UTF-8 text.
-	 *
-	 * @param string $text
-	 * @return array
 	 */
-	public static function decodeTagAttributes( $text ) {
+	public static function decodeTagAttributes( string $text ): array {
 		if ( trim( $text ) == '' ) {
 			return [];
 		}
@@ -1172,11 +1116,8 @@ class Sanitizer {
 	/**
 	 * Build a partial tag string from an associative array of attribute
 	 * names and values as returned by decodeTagAttributes.
-	 *
-	 * @param array $assoc_array
-	 * @return string
 	 */
-	public static function safeEncodeTagAttributes( $assoc_array ) {
+	public static function safeEncodeTagAttributes( array $assoc_array ): string {
 		$attribs = [];
 		foreach ( $assoc_array as $attribute => $value ) {
 			$encAttribute = htmlspecialchars( $attribute, ENT_COMPAT );
@@ -1190,12 +1131,8 @@ class Sanitizer {
 	/**
 	 * Pick the appropriate attribute value from a match set from the
 	 * attribs regex matches.
-	 *
-	 * @param array $set
-	 * @throws MWException When tag conditions are not met.
-	 * @return string
 	 */
-	private static function getTagAttributeCallback( $set ) {
+	private static function getTagAttributeCallback( array $set ): string {
 		if ( isset( $set[5] ) ) {
 			# No quotes.
 			return $set[5];
@@ -1211,31 +1148,31 @@ class Sanitizer {
 			# https://www.w3.org/TR/html5/syntax.html#syntax-attribute-name
 			return "";
 		} else {
-			throw new MWException( "Tag conditions not met. This should never happen and is a bug." );
+			throw new LogicException( "Tag conditions not met. This should never happen and is a bug." );
 		}
 	}
 
-	/**
-	 * @param string $text
-	 * @return string
-	 */
-	private static function normalizeWhitespace( $text ) {
-		return trim( preg_replace(
-			'/(?:\r\n|[\x20\x0d\x0a\x09])+/',
-			' ',
-			$text ) );
+	private static function normalizeWhitespace( string $text ): string {
+		$normalized = preg_replace( '/[ \r\n\t]+/', ' ', $text );
+		if ( $normalized === null ) {
+			wfLogWarning( __METHOD__ . ': Failed to normalize whitespace: ' . preg_last_error() );
+			return '';
+		}
+		return trim( $normalized );
 	}
 
 	/**
 	 * Normalizes whitespace in a section name, such as might be returned
 	 * by Parser::stripSectionName(), for use in the id's that are used for
 	 * section links.
-	 *
-	 * @param string $section
-	 * @return string
 	 */
-	public static function normalizeSectionNameWhitespace( $section ) {
-		return trim( preg_replace( '/[ _]+/', ' ', $section ) );
+	public static function normalizeSectionNameWhitespace( string $section ): string {
+		$normalized = preg_replace( '/[ _]+/', ' ', $section );
+		if ( $normalized === null ) {
+			wfLogWarning( __METHOD__ . ': Failed to normalize whitespace: ' . preg_last_error() );
+			return '';
+		}
+		return trim( $normalized );
 	}
 
 	/**
@@ -1249,28 +1186,23 @@ class Sanitizer {
 	 * c. use lower cased "&#x", not "&#X"
 	 * d. fix or reject non-valid attributes
 	 *
-	 * @param string $text
-	 * @return string
 	 * @internal
 	 */
-	public static function normalizeCharReferences( $text ) {
+	public static function normalizeCharReferences( string $text ): string {
 		return preg_replace_callback(
 			self::CHAR_REFS_REGEX,
 			[ self::class, 'normalizeCharReferencesCallback' ],
-			$text );
+			$text, -1, $count, PREG_UNMATCHED_AS_NULL
+		);
 	}
 
-	/**
-	 * @param string $matches
-	 * @return string
-	 */
-	private static function normalizeCharReferencesCallback( $matches ) {
+	private static function normalizeCharReferencesCallback( array $matches ): string {
 		$ret = null;
-		if ( $matches[1] != '' ) {
+		if ( isset( $matches[1] ) ) {
 			$ret = self::normalizeEntity( $matches[1] );
-		} elseif ( $matches[2] != '' ) {
+		} elseif ( isset( $matches[2] ) ) {
 			$ret = self::decCharReference( $matches[2] );
-		} elseif ( $matches[3] != '' ) {
+		} elseif ( isset( $matches[3] ) ) {
 			$ret = self::hexCharReference( $matches[3] );
 		}
 		if ( $ret === null ) {
@@ -1290,7 +1222,7 @@ class Sanitizer {
 	 * @param string $name Semicolon-terminated name
 	 * @return string
 	 */
-	private static function normalizeEntity( $name ) {
+	private static function normalizeEntity( string $name ): string {
 		if ( isset( self::MW_ENTITY_ALIASES[$name] ) ) {
 			// Non-standard MediaWiki-specific entities
 			return '&' . self::MW_ENTITY_ALIASES[$name];
@@ -1300,33 +1232,28 @@ class Sanitizer {
 		} elseif ( isset( HTMLData::$namedEntityTranslations[$name] ) ) {
 			// Beware: some entities expand to more than 1 codepoint
 			return preg_replace_callback( '/./Ssu', static function ( $m ) {
-				return '&#' . UtfNormal\Utils::utf8ToCodepoint( $m[0] ) . ';';
+				return '&#' . \UtfNormal\Utils::utf8ToCodepoint( $m[0] ) . ';';
 			}, HTMLData::$namedEntityTranslations[$name] );
 		} else {
 			return "&amp;$name";
 		}
 	}
 
-	/**
-	 * @param int $codepoint
-	 * @return null|string
-	 */
-	private static function decCharReference( $codepoint ) {
+	private static function decCharReference( string $codepoint ): ?string {
+		# intval() will (safely) saturate at the maximum signed integer
+		# value if $codepoint is too many digits
 		$point = intval( $codepoint );
 		if ( self::validateCodepoint( $point ) ) {
-			return sprintf( '&#%d;', $point );
+			return "&#$point;";
 		} else {
 			return null;
 		}
 	}
 
-	/**
-	 * @param int $codepoint
-	 * @return null|string
-	 */
-	private static function hexCharReference( $codepoint ) {
+	private static function hexCharReference( string $codepoint ): ?string {
 		$point = hexdec( $codepoint );
-		if ( self::validateCodepoint( $point ) ) {
+		// hexdec() might return a float if the string is too long
+		if ( is_int( $point ) && self::validateCodepoint( $point ) ) {
 			return sprintf( '&#x%x;', $point );
 		} else {
 			return null;
@@ -1336,10 +1263,8 @@ class Sanitizer {
 	/**
 	 * Returns true if a given Unicode codepoint is a valid character in
 	 * both HTML5 and XML.
-	 * @param int $codepoint
-	 * @return bool
 	 */
-	private static function validateCodepoint( $codepoint ) {
+	private static function validateCodepoint( int $codepoint ): bool {
 		# U+000C is valid in HTML5 but not allowed in XML.
 		# U+000D is valid in XML but not allowed in HTML5.
 		# U+007F - U+009F are disallowed in HTML5 (control characters).
@@ -1354,15 +1279,13 @@ class Sanitizer {
 	/**
 	 * Decode any character references, numeric or named entities,
 	 * in the text and return a UTF-8 string.
-	 *
-	 * @param string $text
-	 * @return string
 	 */
-	public static function decodeCharReferences( $text ) {
+	public static function decodeCharReferences( string $text ): string {
 		return preg_replace_callback(
 			self::CHAR_REFS_REGEX,
 			[ self::class, 'decodeCharReferencesCallback' ],
-			$text );
+			$text, -1, $count, PREG_UNMATCHED_AS_NULL
+		);
 	}
 
 	/**
@@ -1375,13 +1298,11 @@ class Sanitizer {
 	 * @param string $text Already normalized, containing entities
 	 * @return string Still normalized, without entities
 	 */
-	public static function decodeCharReferencesAndNormalize( $text ) {
+	public static function decodeCharReferencesAndNormalize( string $text ): string {
 		$text = preg_replace_callback(
 			self::CHAR_REFS_REGEX,
 			[ self::class, 'decodeCharReferencesCallback' ],
-			$text,
-			-1, // limit
-			$count
+			$text, -1, $count, PREG_UNMATCHED_AS_NULL
 		);
 
 		if ( $count ) {
@@ -1391,17 +1312,19 @@ class Sanitizer {
 		}
 	}
 
-	/**
-	 * @param string $matches
-	 * @return string
-	 */
-	private static function decodeCharReferencesCallback( $matches ) {
-		if ( $matches[1] != '' ) {
+	private static function decodeCharReferencesCallback( array $matches ): string {
+		if ( isset( $matches[1] ) ) {
 			return self::decodeEntity( $matches[1] );
-		} elseif ( $matches[2] != '' ) {
+		} elseif ( isset( $matches[2] ) ) {
 			return self::decodeChar( intval( $matches[2] ) );
-		} elseif ( $matches[3] != '' ) {
-			return self::decodeChar( hexdec( $matches[3] ) );
+		} elseif ( isset( $matches[3] ) ) {
+			$point = hexdec( $matches[3] );
+			// hexdec() might return a float if the string is too long
+			if ( !is_int( $point ) ) {
+				// Invalid character reference.
+				return \UtfNormal\Constants::UTF8_REPLACEMENT;
+			}
+			return self::decodeChar( $point );
 		}
 		# Last case should be an ampersand by itself
 		return $matches[0];
@@ -1410,15 +1333,13 @@ class Sanitizer {
 	/**
 	 * Return UTF-8 string for a codepoint if that is a valid
 	 * character reference, otherwise U+FFFD REPLACEMENT CHARACTER.
-	 * @param int $codepoint
-	 * @return string
 	 * @internal
 	 */
-	private static function decodeChar( $codepoint ) {
+	private static function decodeChar( int $codepoint ): string {
 		if ( self::validateCodepoint( $codepoint ) ) {
-			return UtfNormal\Utils::codepointToUtf8( $codepoint );
+			return \UtfNormal\Utils::codepointToUtf8( $codepoint );
 		} else {
-			return UtfNormal\Constants::UTF8_REPLACEMENT;
+			return \UtfNormal\Constants::UTF8_REPLACEMENT;
 		}
 	}
 
@@ -1430,7 +1351,7 @@ class Sanitizer {
 	 * @param string $name Semicolon-terminated entity name
 	 * @return string
 	 */
-	private static function decodeEntity( $name ) {
+	private static function decodeEntity( string $name ): string {
 		// These are MediaWiki-specific entities, not in the HTML standard
 		if ( isset( self::MW_ENTITY_ALIASES[$name] ) ) {
 			$name = self::MW_ENTITY_ALIASES[$name];
@@ -1446,7 +1367,7 @@ class Sanitizer {
 	 * @return array An associative array where keys are acceptable attribute
 	 *   names
 	 */
-	private static function attributesAllowedInternal( $element ) {
+	private static function attributesAllowedInternal( string $element ): array {
 		$list = self::setupAttributesAllowedInternal();
 		return $list[$element] ?? [];
 	}
@@ -1458,7 +1379,7 @@ class Sanitizer {
 	 *   values are associative arrays where the keys are allowed attribute
 	 *   names.
 	 */
-	private static function setupAttributesAllowedInternal() {
+	private static function setupAttributesAllowedInternal(): array {
 		static $allowed;
 
 		if ( $allowed !== null ) {
@@ -1489,6 +1410,7 @@ class Sanitizer {
 			'aria-hidden',
 			'aria-label',
 			'aria-labelledby',
+			'aria-level',
 			'aria-owns',
 			'role',
 
@@ -1627,8 +1549,7 @@ class Sanitizer {
 
 			# 13.2
 			# Not usually allowed, but may be used for extension-style hooks
-			# such as <math> when it is rasterized, or if $wgAllowImageTag is
-			# true
+			# such as <math> when it is rasterized
 			'img'        => $merge( $common, [ 'alt', 'src', 'width', 'height', 'srcset' ] ),
 			# Attributes for A/V tags added in T163583 / T133673
 			'audio'      => $merge( $common, [ 'controls', 'preload', 'width', 'height' ] ),
@@ -1706,7 +1627,7 @@ class Sanitizer {
 	 * @return string
 	 * @return-taint tainted
 	 */
-	public static function stripAllTags( $html ) {
+	public static function stripAllTags( string $html ): string {
 		// Use RemexHtml to tokenize $html and extract the text
 		$handler = new RemexStripTagHandler;
 		$tokenizer = new RemexTokenizer( $handler, $html, [
@@ -1714,12 +1635,11 @@ class Sanitizer {
 			// don't ignore char refs, we want them to be decoded
 			'ignoreNulls' => true,
 			'skipPreprocess' => true,
+			// We ignore all attributes, don't bother to parse them
+			'lazyAttributes' => true,
 		] );
 		$tokenizer->execute();
-		$text = $handler->getResult();
-
-		$text = self::normalizeWhitespace( $text );
-		return $text;
+		return self::normalizeWhitespace( $handler->getResult() );
 	}
 
 	/**
@@ -1729,11 +1649,10 @@ class Sanitizer {
 	 *
 	 * Use for passing XHTML fragments to PHP's XML parsing functions
 	 *
-	 * @return string
 	 * @deprecated since 1.36; will be made private or removed in a future
 	 *    release.
 	 */
-	public static function hackDocType() {
+	public static function hackDocType(): string {
 		$out = "<!DOCTYPE html [\n";
 		foreach ( HTMLData::$namedEntityTranslations as $entity => $translation ) {
 			if ( substr( $entity, -1 ) !== ';' ) {
@@ -1753,23 +1672,19 @@ class Sanitizer {
 		return $out;
 	}
 
-	/**
-	 * @param string $url
-	 * @return mixed|string
-	 */
-	public static function cleanUrl( $url ) {
+	public static function cleanUrl( string $url ): string {
 		# Normalize any HTML entities in input. They will be
 		# re-escaped by makeExternalLink().
 		$url = self::decodeCharReferences( $url );
 
 		# Escape any control characters introduced by the above step
-		$url = preg_replace_callback( '/[\][<>"\\x00-\\x20\\x7F\|]/',
-			[ __CLASS__, 'cleanUrlCallback' ], $url );
+		$url = preg_replace_callback( '/[\][<>"\\x00-\\x20\\x7F\|]+/',
+			static fn ( $m ) => urlencode( $m[0] ), $url );
 
 		# Validate hostname portion
 		$matches = [];
 		if ( preg_match( '!^([^:]+:)(//[^/]+)?(.*)$!iD', $url, $matches ) ) {
-			list( /* $whole */, $protocol, $host, $rest ) = $matches;
+			[ /* $whole */, $protocol, $host, $rest ] = $matches;
 
 			// Characters that will be ignored in IDNs.
 			// https://datatracker.ietf.org/doc/html/rfc8264#section-9.13
@@ -1824,7 +1739,7 @@ class Sanitizer {
 			$host = preg_replace( $strip, '', $host );
 
 			// IPv6 host names are bracketed with [].  Url-decode these.
-			if ( substr_compare( "//%5B", $host, 0, 5 ) === 0 &&
+			if ( str_starts_with( $host, "//%5B" ) &&
 				preg_match( '!^//%5B([0-9A-Fa-f:.]+)%5D((:\d+)?)$!', $host, $matches )
 			) {
 				$host = '//[' . $matches[1] . ']' . $matches[2];
@@ -1838,14 +1753,6 @@ class Sanitizer {
 		}
 	}
 
-	/**
-	 * @param array $matches
-	 * @return string
-	 */
-	private static function cleanUrlCallback( $matches ) {
-		return urlencode( $matches[0] );
-	}
-
 	/**
 	 * Does a string look like an e-mail address?
 	 *
@@ -1874,10 +1781,11 @@ class Sanitizer {
 	 * @param string $addr E-mail address
 	 * @return bool
 	 */
-	public static function validateEmail( $addr ) {
+	public static function validateEmail( string $addr ): bool {
 		$result = null;
 		// TODO This method should be non-static, and have a HookRunner injected
-		if ( !Hooks::runner()->onIsValidEmailAddr( $addr, $result ) ) {
+		$hookRunner = new HookRunner( MediaWikiServices::getInstance()->getHookContainer() );
+		if ( !$hookRunner->onIsValidEmailAddr( $addr, $result ) ) {
 			return $result;
 		}
 
@@ -1899,3 +1807,6 @@ class Sanitizer {
 		return (bool)preg_match( $html5_email_regexp, $addr );
 	}
 }
+
+/** @deprecated class alias since 1.41 */
+class_alias( Sanitizer::class, 'Sanitizer' );
