@@ -568,6 +568,12 @@ class RecentChange implements Taggable {
 	 * @param array|null $feeds Optional feeds to send to, defaults to $wgRCFeeds
 	 */
 	public function notifyRCFeeds( ?array $feeds = null ) {
+		// T403757: Don't send 'suppressed from creation' recent changes entries to the RCFeeds as they do not
+		// have systems to appropriately redact suppressed / deleted material
+		if ( $this->mAttribs['rc_deleted'] != 0 ) {
+			return;
+		}
+
 		$feeds ??=
 			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::RCFeeds );
 
@@ -937,13 +943,14 @@ class RecentChange implements Taggable {
 	 * @param bool $isPatrollable Whether this log entry is patrollable
 	 * @param bool|null $forceBotFlag Override the default behavior and set bot flag to
 	 * 	the value of the argument. When omitted or null, it falls back to the global state.
+	 * @param int|null $deleted Value to set as rc_deleted (one of the LogPage::DELETED_* constants or 0)
 	 *
 	 * @return RecentChange
 	 */
 	public static function newLogEntry( $timestamp,
 		$logPage, $user, $actionComment, $ip,
 		$type, $action, $target, $logComment, $params, $newId = 0, $actionCommentIRC = '',
-		$revId = 0, $isPatrollable = false, $forceBotFlag = null
+		$revId = 0, $isPatrollable = false, $forceBotFlag = null, $deleted = 0
 	) {
 		global $wgRequest;
 
@@ -1013,7 +1020,7 @@ class RecentChange implements Taggable {
 			'rc_new' => 0, # obsolete
 			'rc_old_len' => null,
 			'rc_new_len' => null,
-			'rc_deleted' => 0,
+			'rc_deleted' => $deleted,
 			'rc_logid' => $newId,
 			'rc_log_type' => $type,
 			'rc_log_action' => $action,
