@@ -1,12 +1,18 @@
 <?php
 
+namespace MediaWiki\Registration;
+
+use Exception;
+use InvalidArgumentException;
 use MediaWiki\MainConfigNames;
 use MediaWiki\ResourceLoader\FilePath;
+use RuntimeException;
+use UnexpectedValueException;
 
 /**
- * Utility class for loading extension manifests and aggregating
- * their contents.
+ * Load extension manifests and then aggregate their contents.
  *
+ * @ingroup ExtensionRegistry
  * @newable since 1.39
  */
 class ExtensionProcessor implements Processor {
@@ -29,6 +35,7 @@ class ExtensionProcessor implements Processor {
 		MainConfigNames::AvailableRights,
 		MainConfigNames::CentralIdLookupProviders,
 		MainConfigNames::ChangeCredentialsBlacklist,
+		MainConfigNames::ConditionalUserOptions,
 		MainConfigNames::ConfigRegistry,
 		MainConfigNames::ContentHandlers,
 		MainConfigNames::DefaultUserOptions,
@@ -39,6 +46,7 @@ class ExtensionProcessor implements Processor {
 		MainConfigNames::FilterLogTypes,
 		MainConfigNames::GrantPermissionGroups,
 		MainConfigNames::GrantPermissions,
+		MainConfigNames::GrantRiskGroups,
 		MainConfigNames::GroupPermissions,
 		MainConfigNames::GroupsAddToSelf,
 		MainConfigNames::GroupsRemoveFromSelf,
@@ -52,7 +60,9 @@ class ExtensionProcessor implements Processor {
 		MainConfigNames::LogRestrictions,
 		MainConfigNames::LogTypes,
 		MainConfigNames::MediaHandlers,
+		MainConfigNames::OutputPipelineStages,
 		MainConfigNames::PasswordPolicy,
+		MainConfigNames::PrivilegedGroups,
 		MainConfigNames::RateLimits,
 		MainConfigNames::RawHtmlMessages,
 		MainConfigNames::ReauthenticateTime,
@@ -62,13 +72,12 @@ class ExtensionProcessor implements Processor {
 		MainConfigNames::ResourceLoaderSources,
 		MainConfigNames::RevokePermissions,
 		MainConfigNames::SessionProviders,
-		MainConfigNames::SpecialPages
+		MainConfigNames::SpecialPages,
+		MainConfigNames::UserRegistrationProviders,
 	];
 
 	/**
 	 * Top-level attributes that come from MW core
-	 *
-	 * @var string[]
 	 */
 	protected const CORE_ATTRIBS = [
 		'ParsoidModules',
@@ -80,6 +89,10 @@ class ExtensionProcessor implements Processor {
 		'LateJSConfigVarNames',
 		'TempUserSerialProviders',
 		'TempUserSerialMappings',
+		'DatabaseVirtualDomains',
+		'UserOptionsStoreProviders',
+		'NotificationHandlers',
+		'NotificationMiddleware',
 	];
 
 	/**
@@ -87,9 +100,9 @@ class ExtensionProcessor implements Processor {
 	 *
 	 * @see ExtensionRegistry::exportExtractedData
 	 * @see getExtractedInfo
-	 * @var array
 	 */
 	protected const MERGE_STRATEGIES = [
+		'wgAddGroups' => 'array_merge_recursive',
 		'wgAuthManagerAutoConfig' => 'array_plus_2d',
 		'wgCapitalLinkOverrides' => 'array_plus',
 		'wgExtraGenderNamespaces' => 'array_plus',
@@ -101,13 +114,12 @@ class ExtensionProcessor implements Processor {
 		'wgNamespacesWithSubpages' => 'array_plus',
 		'wgPasswordPolicy' => 'array_merge_recursive',
 		'wgRateLimits' => 'array_plus_2d',
+		'wgRemoveGroups' => 'array_merge_recursive',
 		'wgRevokePermissions' => 'array_plus_2d',
 	];
 
 	/**
 	 * Keys that are part of the extension credits
-	 *
-	 * @var array
 	 */
 	protected const CREDIT_ATTRIBS = [
 		'type',
@@ -124,8 +136,6 @@ class ExtensionProcessor implements Processor {
 	/**
 	 * Things that are not 'attributes', and are not in
 	 * $globalSettings or CREDIT_ATTRIBS.
-	 *
-	 * @var array
 	 */
 	protected const NOT_ATTRIBS = [
 		'callback',
@@ -138,7 +148,10 @@ class ExtensionProcessor implements Processor {
 		'AutoloadClasses',
 		'AutoloadNamespaces',
 		'ExtensionMessagesFiles',
+		'TranslationAliasesDirs',
+		'ForeignResourcesDir',
 		'Hooks',
+		'DomainEventIngresses',
 		'MessagePosterModule',
 		'MessagesDirs',
 		'OOUIThemePaths',
@@ -152,13 +165,15 @@ class ExtensionProcessor implements Processor {
 	/**
 	 * Stuff that is going to be set to $GLOBALS
 	 *
-	 * Some keys are pre-set to arrays so we can += to them
+	 * Some keys are pre-set to arrays, so we can += to them
 	 *
 	 * @var array
 	 */
 	protected $globals = [
 		'wgExtensionMessagesFiles' => [],
+		'wgRestAPIAdditionalRouteFiles' => [],
 		'wgMessagesDirs' => [],
+		'TranslationAliasesDirs' => [],
 	];
 
 	/**
@@ -169,8 +184,9 @@ class ExtensionProcessor implements Processor {
 	protected $defines = [];
 
 	/**
-	 * Things to be called once registration of these extensions are done
-	 * keyed by the name of the extension that it belongs to
+	 * Things to be called once the registration of these extensions is done
+	 *
+	 * Keyed by the name of the extension that it belongs to
 	 *
 	 * @var callable[]
 	 */
@@ -207,7 +223,7 @@ class ExtensionProcessor implements Processor {
 	];
 
 	/**
-	 * Any thing else in the $info that hasn't
+	 * Anything else in the $info that hasn't
 	 * already been processed
 	 *
 	 * @var array
@@ -237,15 +253,6 @@ class ExtensionProcessor implements Processor {
 			throw new RuntimeException( "Failed to load JSON data from $path" );
 		}
 
-		if ( !isset( $info['manifest_version'] ) ) {
-			wfDeprecatedMsg(
-				"{$info['name']}'s extension.json or skin.json does not have manifest_version, " .
-				'this is deprecated since MediaWiki 1.29',
-				'1.29', false, false
-			);
-			$info['manifest_version'] = 1;
-		}
-
 		$this->extractInfo( $path, $info, $info['manifest_version'] );
 	}
 
@@ -257,12 +264,17 @@ class ExtensionProcessor implements Processor {
 	public function extractInfo( $path, array $info, $version ) {
 		$dir = dirname( $path );
 		$this->extractHooks( $info, $path );
+		$this->extractDomainEventIngresses( $info, $path );
 		$this->extractExtensionMessagesFiles( $dir, $info );
+		$this->extractRestModuleFiles( $dir, $info );
 		$this->extractMessagesDirs( $dir, $info );
+		$this->extractTranslationAliasesDirs( $dir, $info );
 		$this->extractSkins( $dir, $info );
 		$this->extractSkinImportPaths( $dir, $info );
 		$this->extractNamespaces( $info );
+		$this->extractImplicitRights( $info );
 		$this->extractResourceLoaderModules( $dir, $info );
+		$this->extractInstallerTasks( $dir, $info );
 		if ( isset( $info['ServiceWiringFiles'] ) ) {
 			$this->extractPathBasedGlobal(
 				'wgServiceWiringFiles',
@@ -295,10 +307,12 @@ class ExtensionProcessor implements Processor {
 						'class' => $className,
 					];
 				}
-				$module['name'] = $name;
+				$module['name'] ??= $name;
 			}
 		}
 
+		$this->extractForeignResourcesDir( $info, $name, $dir );
+
 		if ( $version >= 2 ) {
 			$this->extractAttributes( $path, $info );
 		}
@@ -368,7 +382,8 @@ class ExtensionProcessor implements Processor {
 		}
 
 		$autoload = $this->getExtractedAutoloadInfo( $includeDev );
-		$info = [
+
+		return [
 			'globals' => $this->globals,
 			'defines' => $this->defines,
 			'callbacks' => $this->callbacks,
@@ -378,7 +393,6 @@ class ExtensionProcessor implements Processor {
 			'autoloaderClasses' => $autoload['classes'],
 			'autoloaderNS' => $autoload['namespaces'],
 		];
-		return $info;
 	}
 
 	public function getRequirements( array $info, $includeDev ) {
@@ -430,7 +444,7 @@ class ExtensionProcessor implements Processor {
 					);
 				} else {
 					// Prefer dev value, but these should be constant
-					// anyways (ext-* and ability-*)
+					// anyway (ext-* and ability-*)
 					$value = $dev['platform'][$pkey] ?? $req['platform'][$pkey];
 				}
 				$merged['platform'][$pkey] = $value;
@@ -449,6 +463,7 @@ class ExtensionProcessor implements Processor {
 				);
 			}
 		}
+
 		return $merged;
 	}
 
@@ -461,6 +476,7 @@ class ExtensionProcessor implements Processor {
 	 * @param array $hookHandlersAttr handler definitions from 'HookHandler' attribute
 	 * @param string $name
 	 * @param string $path extension.json file path
+	 *
 	 * @throws UnexpectedValueException
 	 */
 	private function setArrayHookHandler(
@@ -560,9 +576,21 @@ class ExtensionProcessor implements Processor {
 	}
 
 	/**
-	 * Register namespaces with the appropriate global settings
+	 * Extract domain event subscribers.
 	 *
-	 * @param array $info
+	 * @param array $info attributes and associated values from extension.json
+	 * @param string $path path to extension.json
+	 */
+	protected function extractDomainEventIngresses( array $info, string $path ) {
+		$this->attributes['DomainEventIngresses'] ??= [];
+		foreach ( $info['DomainEventIngresses'] ?? [] as $subscriber ) {
+			$subscriber['extensionPath'] = $path;
+			$this->attributes['DomainEventIngresses'][] = $subscriber;
+		}
+	}
+
+	/**
+	 * Register namespaces with the appropriate global settings
 	 */
 	protected function extractNamespaces( array $info ) {
 		if ( isset( $info['namespaces'] ) ) {
@@ -648,8 +676,6 @@ class ExtensionProcessor implements Processor {
 					$data['localBasePath'] = "$dir/{$data['localBasePath']}";
 				}
 			}
-			// Satisfy PHPUnit ResourcesTest::testUnsatisfiableDependencies
-			$data['targets'] = [ 'test' ];
 			$this->attributes['QUnitTestModules']["test.{$info['name']}"] = $data;
 		}
 
@@ -676,6 +702,15 @@ class ExtensionProcessor implements Processor {
 		}
 	}
 
+	protected function extractRestModuleFiles( $dir, array $info ) {
+		$var = MainConfigNames::RestAPIAdditionalRouteFiles;
+		if ( isset( $info['RestModuleFiles'] ) ) {
+			foreach ( $info['RestModuleFiles'] as &$file ) {
+				$this->globals["wg$var"][] = "$dir/$file";
+			}
+		}
+	}
+
 	/**
 	 * Set message-related settings, which need to be expanded to use
 	 * absolute paths
@@ -693,6 +728,21 @@ class ExtensionProcessor implements Processor {
 		}
 	}
 
+	/**
+	 * Set localization related settings, which need to be expanded to use
+	 * absolute paths
+	 *
+	 * @param string $dir
+	 * @param array $info
+	 */
+	protected function extractTranslationAliasesDirs( $dir, array $info ) {
+		foreach ( $info['TranslationAliasesDirs'] ?? [] as $name => $files ) {
+			foreach ( (array)$files as $file ) {
+				$this->globals['wgTranslationAliasesDirs'][$name][] = "$dir/$file";
+			}
+		}
+	}
+
 	/**
 	 * Extract skins and handle path correction for templateDirectory.
 	 *
@@ -702,32 +752,37 @@ class ExtensionProcessor implements Processor {
 	protected function extractSkins( $dir, array $info ) {
 		if ( isset( $info['ValidSkinNames'] ) ) {
 			foreach ( $info['ValidSkinNames'] as $skinKey => $data ) {
-				if ( isset( $data['args'][0]['templateDirectory'] ) ) {
-					$templateDirectory = $data['args'][0]['templateDirectory'];
-					$correctedPath = $dir . '/' . $templateDirectory;
-					// Historically the template directory was relative to core
-					// but it really should've been relative to the skin directory.
-					// If the path exists relative to the skin directory, assume that
-					// is what was intended. Otherwise fall back on the previous behavior
-					// of having it relative to core.
-					if ( is_dir( $correctedPath ) ) {
-						$data['args'][0]['templateDirectory'] = $correctedPath;
-					} else {
-						$data['args'][0]['templateDirectory'] = $templateDirectory;
-						wfDeprecatedMsg(
-							'Template directory should be relative to skin or omitted for skin ' . $skinKey,
-							'1.37'
-						);
-					}
-				} elseif ( isset( $data['args'][0] ) ) {
-					// If not set, we set a sensible default.
-					$data['args'][0]['templateDirectory'] = $dir . '/templates';
+				if ( isset( $data['args'][0] ) ) {
+					$templateDirectory = $data['args'][0]['templateDirectory'] ?? 'templates';
+					$data['args'][0]['templateDirectory'] = $dir . '/' . $templateDirectory;
 				}
 				$this->globals['wgValidSkinNames'][$skinKey] = $data;
 			}
 		}
 	}
 
+	/**
+	 * Extract any user rights that should be granted implicitly.
+	 */
+	protected function extractImplicitRights( array $info ) {
+		// Rate limits are only configurable for rights that are either in wgImplicitRights
+		// or in wgAvailableRights. Extensions that define rate limits should not have to
+		// explicitly add them to wgImplicitRights as well, we can do that automatically.
+
+		if ( isset( $info['RateLimits'] ) ) {
+			$rights = array_keys( $info['RateLimits'] );
+
+			if ( isset( $info['AvailableRights'] ) ) {
+				$rights = array_diff( $rights, $info['AvailableRights'] );
+			}
+
+			$this->globals['wgImplicitRights'] = array_merge(
+				$this->globals['wgImplicitRights'] ?? [],
+				$rights
+			);
+		}
+	}
+
 	/**
 	 * @param string $dir
 	 * @param array $info
@@ -743,6 +798,7 @@ class ExtensionProcessor implements Processor {
 	/**
 	 * @param string $path
 	 * @param array $info
+	 *
 	 * @return string Name of thing
 	 * @throws Exception
 	 */
@@ -764,7 +820,9 @@ class ExtensionProcessor implements Processor {
 		if ( isset( $this->credits[$name] ) ) {
 			$firstPath = $this->credits[$name]['path'];
 			$secondPath = $credits['path'];
-			throw new Exception( "It was attempted to load $name twice, from $firstPath and $secondPath." );
+			throw new InvalidArgumentException(
+				"It was attempted to load $name twice, from $firstPath and $secondPath."
+			);
 		}
 
 		$this->credits[$name] = $credits;
@@ -772,8 +830,30 @@ class ExtensionProcessor implements Processor {
 		return $name;
 	}
 
+	protected function extractForeignResourcesDir( array $info, string $name, string $dir ): void {
+		if ( array_key_exists( 'ForeignResourcesDir', $info ) ) {
+			if ( !is_string( $info['ForeignResourcesDir'] ) ) {
+				throw new InvalidArgumentException( "Incorrect ForeignResourcesDir type, must be a string (in $name)" );
+			}
+			$this->attributes['ForeignResourcesDir'][$name] = "{$dir}/{$info['ForeignResourcesDir']}";
+		}
+	}
+
+	protected function extractInstallerTasks( string $path, array $info ): void {
+		if ( isset( $info['InstallerTasks'] ) ) {
+			// Use a fixed path for the schema base path for now. This could be
+			// made configurable if there were a use case for that.
+			$schemaBasePath = $path . '/sql';
+			foreach ( $info['InstallerTasks'] as $taskSpec ) {
+				$this->attributes['InstallerTasks'][]
+					= $taskSpec + [ 'schemaBasePath' => $schemaBasePath ];
+			}
+		}
+	}
+
 	/**
 	 * Set configuration settings for manifest_version == 1
+	 *
 	 * @todo In the future, this should be done via Config interfaces
 	 *
 	 * @param array $info
@@ -808,11 +888,13 @@ class ExtensionProcessor implements Processor {
 		foreach ( $value as $k => $v ) {
 			$result[$k] = $dir . '/' . $v;
 		}
+
 		return $result;
 	}
 
 	/**
 	 * Set configuration settings for manifest_version == 2
+	 *
 	 * @todo In the future, this should be done via Config interfaces
 	 *
 	 * @param array $info
@@ -847,7 +929,7 @@ class ExtensionProcessor implements Processor {
 	}
 
 	/**
-	 * Helper function to set a value to a specific global, if it isn't set already.
+	 * Helper function to set a value to a specific global config variable if it isn't set already.
 	 *
 	 * @param string $key The config key with the prefix and anything
 	 * @param mixed $value The value of the config
@@ -859,6 +941,14 @@ class ExtensionProcessor implements Processor {
 				"The configuration setting '$key' was already set by MediaWiki core or"
 				. " another extension, and cannot be set again by $extName." );
 		}
+		if ( isset( $value[ExtensionRegistry::MERGE_STRATEGY] ) &&
+			$value[ExtensionRegistry::MERGE_STRATEGY] === 'array_merge_recursive' ) {
+			wfDeprecatedMsg(
+				"Using the array_merge_recursive merge strategy in extension.json and skin.json" .
+				" was deprecated in MediaWiki 1.42",
+				"1.42"
+			);
+		}
 		$this->globals[$key] = $value;
 	}
 
@@ -875,7 +965,7 @@ class ExtensionProcessor implements Processor {
 	 * @param string $name
 	 * @param array $value
 	 * @param array &$array
-	 * @throws InvalidArgumentException
+	 *
 	 */
 	protected function storeToArrayRecursive( $path, $name, $value, &$array ) {
 		if ( !is_array( $value ) ) {
@@ -895,6 +985,7 @@ class ExtensionProcessor implements Processor {
 	 * @param string $name
 	 * @param array $value
 	 * @param array &$array
+	 *
 	 * @throws InvalidArgumentException
 	 */
 	protected function storeToArray( $path, $name, $value, &$array ) {
@@ -908,23 +999,6 @@ class ExtensionProcessor implements Processor {
 		}
 	}
 
-	/**
-	 * @deprecated since 1.39, use getExtractedAutoloadInfo instead
-	 *
-	 * @param string $dir
-	 * @param array $info
-	 *
-	 * @return array
-	 */
-	public function getExtraAutoloaderPaths( $dir, array $info ) {
-		wfDeprecated( __METHOD__, '1.39' );
-		$paths = [];
-		if ( isset( $info['load_composer_autoloader'] ) && $info['load_composer_autoloader'] === true ) {
-			$paths[] = "$dir/vendor/autoload.php";
-		}
-		return $paths;
-	}
-
 	/**
 	 * Returns the extracted autoload info.
 	 * The autoload info is returned as an associative array with three keys:
@@ -962,10 +1036,6 @@ class ExtensionProcessor implements Processor {
 		return $autoload;
 	}
 
-	/**
-	 * @param array $info
-	 * @param string $dir
-	 */
 	private function extractAutoload( array $info, string $dir ) {
 		if ( isset( $info['load_composer_autoloader'] ) && $info['load_composer_autoloader'] === true ) {
 			$file = "$dir/vendor/autoload.php";
@@ -995,3 +1065,6 @@ class ExtensionProcessor implements Processor {
 		}
 	}
 }
+
+/** @deprecated class alias since 1.43 */
+class_alias( ExtensionProcessor::class, 'ExtensionProcessor' );
