@@ -1,5 +1,6 @@
 <?php
 
+use MediaWiki\FileRepo\File\File;
 use MediaWiki\Revision\RevisionRecord;
 use MediaWiki\Title\Title;
 
@@ -12,8 +13,8 @@ class FauxSearchResult extends SearchResult {
 
 	public function __construct(
 		Title $title,
-		RevisionRecord $revRecord = null,
-		File $image = null,
+		?RevisionRecord $revRecord = null,
+		?File $image = null,
 		$text = ''
 	) {
 		parent::__construct();
