@@ -21,7 +21,10 @@
  * @ingroup Search
  */
 
+use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Parser\Sanitizer;
+use MediaWiki\Registration\ExtensionRegistry;
 
 /**
  * Highlight bits of wikitext
@@ -35,6 +38,7 @@ class SearchHighlighter {
 	public const DEFAULT_CONTEXT_LINES = 2;
 	public const DEFAULT_CONTEXT_CHARS = 75;
 
+	/** @var bool */
 	protected $mCleanWikitext = true;
 
 	/**
@@ -63,13 +67,14 @@ class SearchHighlighter {
 		$contextlines = self::DEFAULT_CONTEXT_LINES,
 		$contextchars = self::DEFAULT_CONTEXT_CHARS
 	) {
-		global $wgSearchHighlightBoundaries;
+		$searchHighlightBoundaries = MediaWikiServices::getInstance()
+			->getMainConfig()->get( MainConfigNames::SearchHighlightBoundaries );
 
 		if ( $text == '' ) {
 			return '';
 		}
 
-		// spli text into text + templates/links/tables
+		// split text into text + templates/links/tables
 		$spat = "/(\\{\\{)|(\\[\\[[^\\]:]+:)|(\n\\{\\|)";
 		// first capture group is for detecting nested templates/links/tables/references
 		$endPatterns = [
@@ -79,7 +84,7 @@ class SearchHighlighter {
 
 		// @todo FIXME: This should prolly be a hook or something
 		// instead of hardcoding the name of the Cite extension
-		if ( \ExtensionRegistry::getInstance()->isLoaded( 'Cite' ) ) {
+		if ( ExtensionRegistry::getInstance()->isLoaded( 'Cite' ) ) {
 			$spat .= '|(<ref>)'; // references via cite extension
 			$endPatterns[4] = '/(<ref>)|(<\/ref>)/';
 		}
@@ -130,11 +135,11 @@ class SearchHighlighter {
 								break;
 							} else {
 								// end of nested element
-								$level -= 1;
+								$level--;
 							}
 						} else {
 							// nested
-							$level += 1;
+							$level++;
 						}
 						$offset = $endMatches[0][1] + strlen( $endMatches[0][0] );
 					}
@@ -168,7 +173,7 @@ class SearchHighlighter {
 			}
 		}
 		$anyterm = implode( '|', $terms );
-		$phrase = implode( "$wgSearchHighlightBoundaries+", $terms );
+		$phrase = implode( "{$searchHighlightBoundaries}+", $terms );
 		// @todo FIXME: A hack to scale contextchars, a correct solution
 		// would be to have contextchars actually be char and not byte
 		// length, and do proper utf-8 substrings and lengths everywhere,
@@ -176,8 +181,8 @@ class SearchHighlighter {
 		$scale = strlen( $anyterm ) / mb_strlen( $anyterm );
 		$contextchars = intval( $contextchars * $scale );
 
-		$patPre = "(^|$wgSearchHighlightBoundaries)";
-		$patPost = "($wgSearchHighlightBoundaries|$)";
+		$patPre = "(^|{$searchHighlightBoundaries})";
+		$patPost = "({$searchHighlightBoundaries}|$)";
 
 		$pat1 = "/(" . $phrase . ")/ui";
 		$pat2 = "/$patPre(" . $anyterm . ")$patPost/ui";
@@ -191,7 +196,7 @@ class SearchHighlighter {
 		$first = 0;
 		$firstText = '';
 		foreach ( $textExt as $index => $line ) {
-			if ( strlen( $line ) > 0 && $line[0] != ';' && $line[0] != ':' ) {
+			if ( $line !== '' && $line[0] != ';' && $line[0] != ':' ) {
 				$firstText = $this->extract( $line, 0, $contextchars * $contextlines );
 				$first = $index;
 				break;
@@ -247,12 +252,16 @@ class SearchHighlighter {
 		foreach ( $snippets as $index => $line ) {
 			$extended[$index] = $line;
 			$len = strlen( $line );
+			// @phan-suppress-next-next-line PhanPossiblyUndeclaredVariable
+			// $targetchars is set when $snippes contains anything
 			if ( $len < $targetchars - 20 ) {
 				// complete this line
 				if ( $len < strlen( $all[$index] ) ) {
 					$extended[$index] = $this->extract(
 						$all[$index],
 						$offsets[$index],
+						// @phan-suppress-next-next-line PhanPossiblyUndeclaredVariable
+						// $targetchars is set when $snippes contains anything
 						$offsets[$index] + $targetchars,
 						$offsets[$index]
 					);
@@ -261,10 +270,14 @@ class SearchHighlighter {
 
 				// add more lines
 				$add = $index + 1;
+				// @phan-suppress-next-next-line PhanPossiblyUndeclaredVariable
+				// $targetchars is set when $snippes contains anything
 				while ( $len < $targetchars - 20
 						&& array_key_exists( $add, $all )
 						&& !array_key_exists( $add, $snippets ) ) {
 					$offsets[$add] = 0;
+					// @phan-suppress-next-next-line PhanPossiblyUndeclaredVariable
+					// $targetchars is set when $snippes contains anything
 					$tt = "\n" . $this->extract( $all[$add], 0, $targetchars - $len, $offsets[$add] );
 					$extended[$add] = $tt;
 					$len += strlen( $tt );
@@ -283,7 +296,7 @@ class SearchHighlighter {
 			} elseif ( $last + 1 == $index
 				&& $offsets[$last] + strlen( $snippets[$last] ) >= strlen( $all[$last] )
 			) {
-				$extract .= " " . $line; // continous lines
+				$extract .= " " . $line; // continuous lines
 			} else {
 				$extract .= '<b> ... </b>' . $line;
 			}
@@ -462,7 +475,7 @@ class SearchHighlighter {
 	/**
 	 * Basic wikitext removal
 	 * @param string $text
-	 * @return mixed
+	 * @return string
 	 */
 	private function removeWiki( $text ) {
 		$text = preg_replace( "/\\{\\{([^|]+?)\\}\\}/", "", $text );
@@ -528,17 +541,14 @@ class SearchHighlighter {
 
 		$terms = implode( '|', $terms );
 		$max = intval( $contextchars ) + 1;
-		$pat1 = "/(.*)($terms)(.{0,$max})/i";
+		$pat1 = "/(.*)($terms)(.{0,$max})/ui";
 
-		$lineno = 0;
-
-		$extract = "";
+		$extract = '';
 		$contLang = MediaWikiServices::getInstance()->getContentLanguage();
 		foreach ( $lines as $line ) {
 			if ( $contextlines == 0 ) {
 				break;
 			}
-			++$lineno;
 			$m = [];
 			if ( !preg_match( $pat1, $line, $m ) ) {
 				continue;
@@ -555,12 +565,11 @@ class SearchHighlighter {
 
 			$found = $m[2];
 
-			// @phan-suppress-next-line SecurityCheck-DoubleEscaped Triggered by Language::truncateForVisual
 			$line = htmlspecialchars( $pre . $found . $post );
-			$pat2 = '/(' . $terms . ")/i";
-			$line = preg_replace( $pat2, "<span class='searchmatch'>\\1</span>", $line );
+			$pat2 = '/(' . $terms . ')/ui';
+			$line = preg_replace( $pat2, '<span class="searchmatch">\1</span>', $line );
 
-			$extract .= "${line}\n";
+			$extract .= "{$line}\n";
 		}
 
 		return $extract;
