@@ -1,12 +1,29 @@
 <?php
 
+namespace MediaWiki\SpecialPage;
+
+use InvalidArgumentException;
+use LogicException;
 use MediaWiki\Auth\AuthenticationRequest;
 use MediaWiki\Auth\AuthenticationResponse;
 use MediaWiki\Auth\AuthManager;
+use MediaWiki\Context\DerivativeContext;
+use MediaWiki\Deferred\DeferredUpdates;
+use MediaWiki\Exception\ErrorPageError;
+use MediaWiki\HTMLForm\Field\HTMLInfoField;
+use MediaWiki\HTMLForm\HTMLForm;
 use MediaWiki\Language\RawMessage;
 use MediaWiki\Logger\LoggerFactory;
+use MediaWiki\MainConfigNames;
+use MediaWiki\Message\Message;
 use MediaWiki\Request\DerivativeRequest;
+use MediaWiki\Request\WebRequest;
 use MediaWiki\Session\Token;
+use MediaWiki\Status\Status;
+use MWCryptRand;
+use Profiler;
+use StatusValue;
+use UnexpectedValueException;
 
 /**
  * A special page subclass for authentication-related special pages. It generates a form from
@@ -16,6 +33,7 @@ use MediaWiki\Session\Token;
  * @note Call self::setAuthManager from special page constructor when extending
  *
  * @stable to extend
+ * @ingroup Auth
  */
 abstract class AuthManagerSpecialPage extends SpecialPage {
 	/** @var string[] The list of actions this special page deals with. Subclasses should override
@@ -94,12 +112,7 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 		);
 	}
 
-	/**
-	 * @stable to override
-	 * @param string|null $subPage
-	 *
-	 * @return bool|void
-	 */
+	/** @inheritDoc */
 	protected function beforeExecute( $subPage ) {
 		$this->getOutput()->disallowUserJs();
 
@@ -112,11 +125,11 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 	 *
 	 * This is used in the redirect flow where we need
 	 * to be able to process data that was sent via a GET request. We set the /return subpage as
-	 * the reentry point so we know we need to treat GET as POST, but we don't want to handle all
-	 * future GETs as POSTs so we need to normalize the URL. (Also we don't want to show any
+	 * the reentry point, so we know we need to treat GET as POST, but we don't want to handle all
+	 * future GETs requests as POSTs, so we need to normalize the URL. (Also, we don't want to show any
 	 * received parameters around in the URL; they are ugly and might be sensitive.)
 	 *
-	 * Thus when on the /return subpage, we stash the request data in the session, redirect, then
+	 * Thus, when on the /return subpage, we stash the request data in the session, redirect, then
 	 * use the session to detect that we have been redirected, recover the data and replace the
 	 * real WebRequest with a fake one that contains the saved data.
 	 *
@@ -129,23 +142,29 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 
 		if ( $subPage === 'return' ) {
 			$this->loadAuth( $subPage );
-			$preservedParams = $this->getPreservedParams( false );
+			$preservedParams = $this->getPreservedParams();
 
 			// FIXME save POST values only from request
 			$authData = array_diff_key( $this->getRequest()->getValues(),
 				$preservedParams, [ 'title' => 1 ] );
+			$uniqueId = MWCryptRand::generateHex( 6 );
+			$preservedParams['authUniqueId'] = $uniqueId;
+			$key .= ':' . $uniqueId;
 			$authManager->setAuthenticationSessionData( $key, $authData );
 
 			$url = $this->getPageTitle()->getFullURL( $preservedParams, false, PROTO_HTTPS );
 			$this->getOutput()->redirect( $url );
 			return false;
-		}
-
-		$authData = $authManager->getAuthenticationSessionData( $key );
-		if ( $authData ) {
-			$authManager->removeAuthenticationSessionData( $key );
-			$this->isReturn = true;
-			$this->setRequest( $authData, true );
+		} elseif ( $this->getRequest()->getCheck( 'authUniqueId' ) ) {
+			$uniqueId = $this->getRequest()->getVal( 'authUniqueId' );
+			$key .= ':' . $uniqueId;
+			$authData = $authManager->getAuthenticationSessionData( $key );
+			if ( $authData ) {
+				$authManager->removeAuthenticationSessionData( $key );
+				$this->isReturn = true;
+				$this->setRequest( $authData, true );
+				$this->setPostTransactionProfilerExpectations( __METHOD__ );
+			}
 		}
 
 		return true;
@@ -179,20 +198,26 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 
 					$queryParams = [ 'authUniqueId' => $uniqueId ] + $queryParams;
 					$authData = array_diff_key( $request->getValues(),
-							$this->getPreservedParams( false ), [ 'title' => 1 ] );
+							$this->getPreservedParams(), [ 'title' => 1 ] );
 					$authManager->setAuthenticationSessionData( $key, $authData );
 				}
 
+				// Copied from RedirectSpecialPage::getRedirectQuery()
+				// Would using $this->getPreservedParams() be appropriate here?
+				$keepParams = [ 'uselang', 'useskin', 'useformat', 'variant', 'debug', 'safemode' ];
+
 				$title = SpecialPage::getTitleFor( 'Userlogin' );
 				$url = $title->getFullURL( [
 					'returnto' => $this->getFullTitle()->getPrefixedDBkey(),
 					'returntoquery' => wfArrayToCgi( $queryParams ),
 					'force' => $securityLevel,
-				], false, PROTO_HTTPS );
+				] + array_intersect_key( $queryParams, array_fill_keys( $keepParams, true ) ), false, PROTO_HTTPS );
 
 				$this->getOutput()->redirect( $url );
 				return false;
-			} elseif ( $securityStatus !== AuthManager::SEC_OK ) {
+			}
+
+			if ( $securityStatus !== AuthManager::SEC_OK ) {
 				throw new ErrorPageError( 'cannotauth-not-allowed-title', 'cannotauth-not-allowed' );
 			}
 		}
@@ -204,14 +229,24 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 			if ( $authData ) {
 				$authManager->removeAuthenticationSessionData( $key );
 				$this->setRequest( $authData, true );
+				$this->setPostTransactionProfilerExpectations( __METHOD__ );
 			}
 		}
 
 		return true;
 	}
 
+	private function setPostTransactionProfilerExpectations( string $fname ) {
+		$trxLimits = $this->getConfig()->get( MainConfigNames::TrxProfilerLimits );
+		$trxProfiler = Profiler::instance()->getTransactionProfiler();
+		$trxProfiler->redefineExpectations( $trxLimits['POST'], $fname );
+		DeferredUpdates::addCallableUpdate( static function () use ( $trxProfiler, $trxLimits, $fname ) {
+			$trxProfiler->redefineExpectations( $trxLimits['PostSend-POST'], $fname );
+		} );
+	}
+
 	/**
-	 * Get the default action for this special page, if none is given via URL/POST data.
+	 * Get the default action for this special page if none is given via URL/POST data.
 	 * Subclasses should override this (or override loadAuth() so this is never called).
 	 * @stable to override
 	 * @param string $subPage Subpage of the special page.
@@ -251,7 +286,7 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 	 */
 	protected function loadAuth( $subPage, $authAction = null, $reset = false ) {
 		// Do not load if already loaded, to cut down on the number of getAuthenticationRequests
-		// calls. This is important for requests which have hidden information so any
+		// calls. This is important for requests which have hidden information, so any
 		// getAuthenticationRequests call would mean putting data into some cache.
 		if (
 			!$reset && $this->subPage === $subPage && $this->authAction
@@ -318,7 +353,6 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 	 * the caller's responsibility.
 	 * @param string $action One of the AuthManager::ACTION_* constants in static::$allowedActions
 	 * @return bool
-	 * @throws LogicException if $action is invalid
 	 */
 	protected function isActionAllowed( $action ) {
 		$authManager = $this->getAuthManager();
@@ -367,7 +401,7 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 
 		$authManager = $this->getAuthManager();
 		$returnToUrl = $this->getPageTitle( 'return' )
-			->getFullURL( $this->getPreservedParams( true ), false, PROTO_HTTPS );
+			->getFullURL( $this->getPreservedParams( [ 'withToken' => true ] ), false, PROTO_HTTPS );
 
 		switch ( $action ) {
 			case AuthManager::ACTION_LOGIN:
@@ -388,7 +422,9 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 			case AuthManager::ACTION_UNLINK:
 				if ( count( $requests ) > 1 ) {
 					throw new InvalidArgumentException( 'only one auth request can be changed at a time' );
-				} elseif ( !$requests ) {
+				}
+
+				if ( !$requests ) {
 					throw new InvalidArgumentException( 'no auth request' );
 				}
 				$req = reset( $requests );
@@ -464,13 +500,13 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 					$status = $ret;
 				} else {
 					throw new UnexpectedValueException( 'invalid HTMLForm::trySubmit() return value: '
-						. 'first element of array is ' . gettype( reset( $status ) ) );
+						. 'first element of array is ' . get_debug_type( reset( $status ) ) );
 				}
 			} else {
-				// not supposed to happen but HTMLForm does not actually verify the return type
-				// from the submit callback; better safe then sorry
+				// not supposed to happen, but HTMLForm does not verify the return type
+				// from the submit callback; better safe then sorry!
 				throw new UnexpectedValueException( 'invalid HTMLForm::trySubmit() return type: '
-					. gettype( $status ) );
+					. get_debug_type( $status ) );
 			}
 
 			if ( ( !$status || !$status->isOK() ) && $this->isReturn ) {
@@ -510,22 +546,62 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 	}
 
 	/**
-	 * Returns URL query parameters which can be used to reload the page (or leave and return) while
-	 * preserving all information that is necessary for authentication to continue. These parameters
-	 * will be preserved in the action URL of the form and in the return URL for redirect flow.
+	 * Returns URL query parameters which should be preserved between authentication requests.
+	 * These should be used when generating links such as form submit or language switch.
+	 *
+	 * These parameters will be preserved in:
+	 * - successive authentication steps (the form submit URL and the return URL for redirecting
+	 *   providers);
+	 * - links that reload the same form somehow (e.g. language switcher links);
+	 * - links for switching between the login and create account forms.
+	 *
 	 * @stable to override
-	 * @param bool $withToken Include CSRF token
-	 * @return array
+	 * @param array $options (since 1.43)
+	 *   - reset (bool, default false): Reset the authentication process, i.e. omit parameters
+	 *     which are related to continuing in-progress authentication.
+	 *   - withToken (bool, default false): Include CSRF token
+	 *   Before 1.43, this was a boolean flag identical to the current 'withToken' option.
+	 *   That usage is deprecated.
+	 * @phan-param array{reset?: bool, withToken?: bool}|bool $options
+	 * @return array Array of parameter name => parameter value.
 	 */
-	protected function getPreservedParams( $withToken = false ) {
+	protected function getPreservedParams( $options = [] ) {
+		if ( is_bool( $options ) ) {
+			wfDeprecated( __METHOD__ . ' boolean $options', '1.43' );
+			$options = [ 'withToken' => $options ];
+		}
+		$options += [
+			'reset' => false,
+			'withToken' => false,
+		];
+		// Help Phan figure out that these fields are now definitely set - https://github.com/phan/phan/issues/4864
+		'@phan-var array{reset: bool, withToken: bool} $options';
 		$params = [];
-		if ( $this->authAction !== $this->getDefaultAction( $this->subPage ) ) {
+		$request = $this->getRequest();
+
+		$params += [
+			'uselang' => $request->getVal( 'uselang' ),
+			'variant' => $request->getVal( 'variant' ),
+			'returnto' => $request->getVal( 'returnto' ),
+			'returntoquery' => $request->getVal( 'returntoquery' ),
+			'returntoanchor' => $request->getVal( 'returntoanchor' ),
+		];
+
+		if ( !$options['reset'] && $this->authAction !== $this->getDefaultAction( $this->subPage ) ) {
 			$params['authAction'] = $this->getContinueAction( $this->authAction );
 		}
-		if ( $withToken ) {
+
+		if ( $options['withToken'] ) {
 			$params[$this->getTokenName()] = $this->getToken()->toString();
 		}
-		return $params;
+
+		// Allow authentication extensions like CentralAuth to preserve their own
+		// query params during and after the authentication process.
+		$this->getHookRunner()->onAuthPreserveQueryParams(
+			$params, [ 'request' => $request, 'reset' => $options['reset'] ]
+		);
+
+		return array_filter( $params, static fn ( $val ) => $val !== null );
 	}
 
 	/**
@@ -740,14 +816,14 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 	 * Sort the fields of a form descriptor by their 'weight' property. (Fields with higher weight
 	 * are shown closer to the bottom; weight defaults to 0. Negative weight is allowed.)
 	 * Keep order if weights are equal.
-	 * @param array &$formDescriptor
 	 */
 	protected static function sortFormDescriptorFields( array &$formDescriptor ) {
 		$i = 0;
 		foreach ( $formDescriptor as &$field ) {
 			$field['__index'] = $i++;
 		}
-		uasort( $formDescriptor, function ( $first, $second ) {
+		unset( $field );
+		uasort( $formDescriptor, static function ( $first, $second ) {
 			return self::getField( $first, 'weight', 0 ) <=> self::getField( $second, 'weight', 0 )
 				?: $first['__index'] <=> $second['__index'];
 		} );
@@ -773,9 +849,10 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 
 	/**
 	 * Maps AuthenticationRequest::getFieldInfo() types to HTMLForm types
+	 *
 	 * @param string $type
+	 *
 	 * @return string
-	 * @throws \LogicException
 	 */
 	protected static function mapFieldInfoTypeToFormDescriptorType( $type ) {
 		$map = [
@@ -789,7 +866,7 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 			'null' => 'info',
 		];
 		if ( !array_key_exists( $type, $map ) ) {
-			throw new \LogicException( 'invalid field type: ' . $type );
+			throw new InvalidArgumentException( 'invalid field type: ' . $type );
 		}
 		return $map[$type];
 	}
@@ -844,3 +921,6 @@ abstract class AuthManagerSpecialPage extends SpecialPage {
 		return array_filter( $defaultFormDescriptor + $formDescriptor );
 	}
 }
+
+/** @deprecated class alias since 1.41 */
+class_alias( AuthManagerSpecialPage::class, 'AuthManagerSpecialPage' );
