@@ -85,6 +85,7 @@ use MediaWiki\Specials\SpecialFileDuplicateSearch;
 use MediaWiki\Specials\SpecialFilepath;
 use MediaWiki\Specials\SpecialGoToInterwiki;
 use MediaWiki\Specials\SpecialImport;
+use MediaWiki\Specials\SpecialInterwiki;
 use MediaWiki\Specials\SpecialJavaScriptTest;
 use MediaWiki\Specials\SpecialLinkAccounts;
 use MediaWiki\Specials\SpecialLinkSearch;
@@ -139,7 +140,6 @@ use MediaWiki\Specials\SpecialRedirect;
 use MediaWiki\Specials\SpecialRemoveCredentials;
 use MediaWiki\Specials\SpecialRenameUser;
 use MediaWiki\Specials\SpecialResetTokens;
-use MediaWiki\Specials\SpecialRestSandbox;
 use MediaWiki\Specials\SpecialRevisionDelete;
 use MediaWiki\Specials\SpecialRunJobs;
 use MediaWiki\Specials\SpecialSearch;
@@ -455,6 +455,7 @@ class SpecialPageFactory {
 			'class' => SpecialUserLogin::class,
 			'services' => [
 				'AuthManager',
+				'UserIdentityUtils',
 			]
 		],
 		'Userlogout' => [
@@ -468,6 +469,7 @@ class SpecialPageFactory {
 			'services' => [
 				'AuthManager',
 				'FormatterFactory',
+				'UserIdentityUtils',
 			]
 		],
 		'LinkAccounts' => [
@@ -510,12 +512,13 @@ class SpecialPageFactory {
 				'UserGroupManager',
 				'UserIdentityLookup',
 				'HideUserUtils',
+				'TempUserConfig',
 			]
 		],
 		'Block' => [
 			'class' => SpecialBlock::class,
 			'services' => [
-				'BlockUtils',
+				'BlockTargetFactory',
 				'BlockPermissionCheckerFactory',
 				'BlockUserFactory',
 				'DatabaseBlockStore',
@@ -523,14 +526,16 @@ class SpecialPageFactory {
 				'UserNamePrefixSearch',
 				'BlockActionInfo',
 				'TitleFormatter',
-				'NamespaceInfo'
+				'NamespaceInfo',
+				'UserOptionsLookup',
+				'WatchlistManager'
 			]
 		],
 		'Unblock' => [
 			'class' => SpecialUnblock::class,
 			'services' => [
 				'UnblockUserFactory',
-				'BlockUtils',
+				'BlockTargetFactory',
 				'DatabaseBlockStore',
 				'UserNameUtils',
 				'UserNamePrefixSearch',
@@ -545,10 +550,11 @@ class SpecialPageFactory {
 				'BlockRestrictionStore',
 				'ConnectionProvider',
 				'CommentStore',
-				'BlockUtils',
+				'BlockTargetFactory',
 				'HideUserUtils',
 				'BlockActionInfo',
 				'RowCommentFormatter',
+				'TempUserConfig',
 			],
 		],
 		'AutoblockList' => [
@@ -558,7 +564,7 @@ class SpecialPageFactory {
 				'BlockRestrictionStore',
 				'ConnectionProvider',
 				'CommentStore',
-				'BlockUtils',
+				'BlockTargetFactory',
 				'HideUserUtils',
 				'BlockActionInfo',
 				'RowCommentFormatter',
@@ -652,6 +658,7 @@ class SpecialPageFactory {
 				'UserGroupManager',
 				'UserIdentityLookup',
 				'HideUserUtils',
+				'TempUserConfig',
 			]
 		],
 		'Listadmins' => [
@@ -741,7 +748,7 @@ class SpecialPageFactory {
 			'class' => SpecialRecentChanges::class,
 			'services' => [
 				'WatchedItemStore',
-				'MessageCache',
+				'MessageParser',
 				'UserOptionsLookup',
 				'ChangeTagsStore',
 				'UserIdentityUtils',
@@ -752,7 +759,7 @@ class SpecialPageFactory {
 			'class' => SpecialRecentChangesLinked::class,
 			'services' => [
 				'WatchedItemStore',
-				'MessageCache',
+				'MessageParser',
 				'UserOptionsLookup',
 				'SearchEngineFactory',
 				'ChangeTagsStore',
@@ -776,7 +783,7 @@ class SpecialPageFactory {
 				'CommentStore',
 				'UserNameUtils',
 				'UserNamePrefixSearch',
-				'CommentFormatter',
+				'RowCommentFormatter',
 				'LinkBatchFactory',
 			]
 		],
@@ -840,10 +847,14 @@ class SpecialPageFactory {
 		'ApiSandbox' => [
 			'class' => SpecialApiSandbox::class,
 		],
-		'RestSandbox' => [
-			'class' => SpecialRestSandbox::class,
+		'Interwiki' => [
+			'class' => SpecialInterwiki::class,
 			'services' => [
+				'ContentLanguage',
+				'InterwikiLookup',
+				'LanguageNameUtils',
 				'UrlUtils',
+				'ConnectionProvider',
 			]
 		],
 		'Statistics' => [
@@ -1044,6 +1055,7 @@ class SpecialPageFactory {
 				'ConnectionProvider',
 				'RevisionStore',
 				'CommentFormatter',
+				'ChangeTagsStore',
 			]
 		],
 		'ExpandTemplates' => [
@@ -1221,12 +1233,11 @@ class SpecialPageFactory {
 			'class' => SpecialRenameUser::class,
 			'services' => [
 				'ConnectionProvider',
-				'MovePageFactory',
 				'PermissionManager',
 				'TitleFactory',
 				'UserFactory',
 				'UserNamePrefixSearch',
-				'UserNameUtils',
+				'RenameUserFactory',
 			]
 		],
 		'Revisiondelete' => [
@@ -1255,7 +1266,6 @@ class SpecialPageFactory {
 		'TalkPage' => [
 			'class' => SpecialTalkPage::class,
 			'services' => [
-				'MainConfig',
 				'TitleParser',
 			],
 		],
@@ -1345,8 +1355,6 @@ class SpecialPageFactory {
 
 	/**
 	 * Get the special page list as an array
-	 *
-	 * @return array
 	 */
 	private function getPageList(): array {
 		if ( !is_array( $this->list ) ) {
@@ -1450,7 +1458,6 @@ class SpecialPageFactory {
 	 * Initialise and return the list of special page aliases. Returns an array where
 	 * the key is an alias, and the value is the canonical name of the special page.
 	 * All registered special pages are guaranteed to map to themselves.
-	 * @return array
 	 */
 	private function getAliasList(): array {
 		if ( $this->aliases === null ) {
@@ -1532,10 +1539,10 @@ class SpecialPageFactory {
 	 * @return bool True if a special page exists with this name
 	 */
 	public function exists( $name ) {
-		[ $title, /*...*/ ] = $this->resolveAlias( $name );
+		[ $title, ] = $this->resolveAlias( $name );
 
 		$specialPageList = $this->getPageList();
-		return isset( $specialPageList[$title] );
+		return $title !== null && isset( $specialPageList[$title] );
 	}
 
 	/**
@@ -1545,11 +1552,11 @@ class SpecialPageFactory {
 	 * @return SpecialPage|null SpecialPage object or null if the page doesn't exist
 	 */
 	public function getPage( $name ) {
-		[ $realName, /*...*/ ] = $this->resolveAlias( $name );
+		[ $realName, ] = $this->resolveAlias( $name );
 
 		$specialPageList = $this->getPageList();
 
-		if ( isset( $specialPageList[$realName] ) ) {
+		if ( $realName !== null && isset( $specialPageList[$realName] ) ) {
 			$rec = $specialPageList[$realName];
 
 			if ( is_array( $rec ) || is_string( $rec ) || is_callable( $rec ) ) {
