@@ -21,9 +21,14 @@
  * @ingroup Upload
  */
 
+use MediaWiki\Api\ApiMessage;
 use MediaWiki\Api\ApiResult;
 use MediaWiki\Api\ApiUpload;
 use MediaWiki\Context\RequestContext;
+use MediaWiki\FileRepo\File\ArchivedFile;
+use MediaWiki\FileRepo\File\File;
+use MediaWiki\FileRepo\File\LocalFile;
+use MediaWiki\FileRepo\FileRepo;
 use MediaWiki\HookContainer\HookRunner;
 use MediaWiki\HookContainer\ProtectedHookAccessorTrait;
 use MediaWiki\Logger\LoggerFactory;
@@ -37,12 +42,16 @@ use MediaWiki\Request\WebRequest;
 use MediaWiki\Shell\Shell;
 use MediaWiki\Status\Status;
 use MediaWiki\Title\Title;
+use MediaWiki\Upload\SVGCSSChecker;
 use MediaWiki\User\User;
 use MediaWiki\User\UserIdentity;
 use Wikimedia\AtEase\AtEase;
 use Wikimedia\FileBackend\FileBackend;
 use Wikimedia\FileBackend\FSFile\FSFile;
 use Wikimedia\FileBackend\FSFile\TempFSFile;
+use Wikimedia\Message\ListType;
+use Wikimedia\Message\MessageParam;
+use Wikimedia\Message\MessageSpecifier;
 use Wikimedia\Mime\XmlTypeCheck;
 use Wikimedia\ObjectCache\BagOStuff;
 use Wikimedia\Rdbms\IDBAccessObject;
@@ -101,7 +110,7 @@ abstract class UploadBase {
 	/** @var string|false */
 	protected $mSVGNSError;
 
-	private const SAFE_XML_ENCONDINGS = [
+	private const SAFE_XML_ENCODINGS = [
 		'UTF-8',
 		'US-ASCII',
 		'ISO-8859-1',
@@ -124,11 +133,9 @@ abstract class UploadBase {
 	public const EMPTY_FILE = 3;
 	public const MIN_LENGTH_PARTNAME = 4;
 	public const ILLEGAL_FILENAME = 5;
-	public const OVERWRITE_EXISTING_FILE = 7; # Not used anymore; handled by verifyTitlePermissions()
 	public const FILETYPE_MISSING = 8;
 	public const FILETYPE_BADTYPE = 9;
 	public const VERIFICATION_ERROR = 10;
-	public const HOOK_ABORTED = 11;
 	public const FILE_TOO_LARGE = 12;
 	public const WINDOWS_NONASCII_FILENAME = 13;
 	public const FILENAME_TOO_LONG = 14;
@@ -140,9 +147,7 @@ abstract class UploadBase {
 		self::FILETYPE_BADTYPE => 'filetype-banned',
 		self::MIN_LENGTH_PARTNAME => 'filename-tooshort',
 		self::ILLEGAL_FILENAME => 'illegal-filename',
-		self::OVERWRITE_EXISTING_FILE => 'overwrite',
 		self::VERIFICATION_ERROR => 'verification-error',
-		self::HOOK_ABORTED => 'hookaborted',
 		self::WINDOWS_NONASCII_FILENAME => 'windows-nonascii-filename',
 		self::FILENAME_TOO_LONG => 'filename-toolong',
 	];
@@ -188,7 +193,7 @@ abstract class UploadBase {
 	/**
 	 * Returns true if the user has surpassed the upload rate limit, false otherwise.
 	 *
-	 * @deprecated since 1.41, use verifyTitlePermissions() instead.
+	 * @deprecated since 1.41, use authorizeUpload() instead.
 	 * Rate limit checks are now implicit in permission checks.
 	 *
 	 * @param User $user
@@ -310,7 +315,7 @@ abstract class UploadBase {
 		$this->mTempPath = $tempPath ?? '';
 		$this->mFileSize = $fileSize ?: null;
 		$this->mFileProps = null;
-		if ( strlen( $this->mTempPath ) && file_exists( $this->mTempPath ) ) {
+		if ( $this->mTempPath !== '' && file_exists( $this->mTempPath ) ) {
 			$this->tempFileObj = new TempFSFile( $this->mTempPath );
 			if ( !$fileSize ) {
 				$this->mFileSize = filesize( $this->mTempPath );
@@ -644,48 +649,63 @@ abstract class UploadBase {
 	 * 'verifyPermissions', but that suggests it's checking the user, when it's
 	 * really checking the title + user combination.
 	 *
+	 * @deprecated since 1.44 Use authorizeUpload() instead
 	 * @param Authority $performer to verify the permissions against
 	 * @return array|bool An array as returned by getPermissionErrors or true
 	 *   in case the user has proper permissions.
 	 */
 	public function verifyPermissions( Authority $performer ) {
+		wfDeprecated( __METHOD__, '1.44' );
 		return $this->verifyTitlePermissions( $performer );
 	}
 
 	/**
-	 * Check whether the user can edit, upload and create the image. This
-	 * checks only against the current title; if it returns errors, it may
-	 * very well be that another title will not give errors. Therefore
-	 * isAllowed() should be called as well for generic is-user-blocked or
-	 * can-user-upload checking.
+	 * Check whether the user can upload the image. This method checks against the current title.
+	 * Use verifyUpload() or validateName() first to check that the title is valid.
 	 *
+	 * @deprecated since 1.44 Use authorizeUpload() instead
 	 * @param Authority $performer to verify the permissions against
 	 * @return array|bool An array as returned by getPermissionErrors or true
 	 *   in case the user has proper permissions.
 	 */
 	public function verifyTitlePermissions( Authority $performer ) {
-		/**
-		 * If the image is protected, non-sysop users won't be able
-		 * to modify it by uploading a new revision.
-		 */
-		$nt = $this->getTitle();
-		if ( $nt === null ) {
+		wfDeprecated( __METHOD__, '1.44' );
+
+		if ( $this->getTitle() === null ) {
 			return true;
 		}
+		$status = $this->authorizeUpload( $performer );
+		if ( !$status->isGood() ) {
+			return $status->toLegacyErrorArray();
+		}
+		return true;
+	}
 
+	/**
+	 * Check whether the user can upload the image. This method checks against the current title.
+	 * Use verifyUpload() or validateName() first to check that the title is valid.
+	 */
+	public function authorizeUpload( Authority $performer ): PermissionStatus {
 		$status = PermissionStatus::newEmpty();
+
+		$nt = $this->getTitle();
+		if ( $nt === null ) {
+			throw new LogicException( __METHOD__ . ' must only be called with valid title' );
+		}
+
 		$performer->authorizeWrite( 'edit', $nt, $status );
 		$performer->authorizeWrite( 'upload', $nt, $status );
 		if ( !$status->isGood() ) {
-			return $status->toLegacyErrorArray();
+			// If the user can't upload at all, don't display additional errors about re-uploading
+			return $status;
 		}
 
 		$overwriteError = $this->checkOverwrite( $performer );
 		if ( $overwriteError !== true ) {
-			return [ $overwriteError ];
+			$status->fatal( ...$overwriteError );
 		}
 
-		return true;
+		return $status;
 	}
 
 	/**
@@ -752,8 +772,9 @@ abstract class UploadBase {
 
 	/**
 	 * Convert the warnings array returned by checkWarnings() to something that
-	 * can be serialized. File objects will be converted to an associative array
-	 * with the following keys:
+	 * can be serialized, and that is suitable for inclusion directly in action API results.
+	 *
+	 * File objects will be converted to an associative array with the following keys:
 	 *
 	 *   - fileName: The name of the file
 	 *   - timestamp: The upload timestamp
@@ -768,6 +789,8 @@ abstract class UploadBase {
 					'fileName' => $param->getName(),
 					'timestamp' => $param->getTimestamp()
 				];
+			} elseif ( $param instanceof MessageParam ) {
+				// Do nothing (T390001)
 			} elseif ( is_object( $param ) ) {
 				throw new InvalidArgumentException(
 					'UploadBase::makeWarningsSerializable: ' .
@@ -901,7 +924,7 @@ abstract class UploadBase {
 		return $warnings;
 	}
 
-	private function checkLocalFileWasDeleted( LocalFile $localFile ) {
+	private function checkLocalFileWasDeleted( LocalFile $localFile ): bool {
 		return $localFile->wasDeleted() && !$localFile->exists();
 	}
 
@@ -1487,7 +1510,7 @@ abstract class UploadBase {
 
 		if ( preg_match( "!<\?xml\b(.*?)\?>!si", $contents, $matches ) ) {
 			if ( preg_match( $encodingRegex, $matches[1], $encMatch )
-				&& !in_array( strtoupper( $encMatch[1] ), self::SAFE_XML_ENCONDINGS )
+				&& !in_array( strtoupper( $encMatch[1] ), self::SAFE_XML_ENCODINGS )
 			) {
 				wfDebug( __METHOD__ . ": Found unsafe XML encoding '{$encMatch[1]}'" );
 
@@ -1507,7 +1530,7 @@ abstract class UploadBase {
 		}
 
 		// It's possible the file is encoded with multibyte encoding, so re-encode attempt to
-		// detect the encoding in case it specifies an encoding not allowed in self::SAFE_XML_ENCONDINGS
+		// detect the encoding in case it specifies an encoding not allowed in self::SAFE_XML_ENCODINGS
 		$attemptEncodings = [ 'UTF-16', 'UTF-16BE', 'UTF-32', 'UTF-32BE' ];
 		foreach ( $attemptEncodings as $encoding ) {
 			AtEase::suppressWarnings();
@@ -1515,7 +1538,7 @@ abstract class UploadBase {
 			AtEase::restoreWarnings();
 			if ( $str != '' && preg_match( "!<\?xml\b(.*?)\?>!si", $str, $matches ) ) {
 				if ( preg_match( $encodingRegex, $matches[1], $encMatch )
-					&& !in_array( strtoupper( $encMatch[1] ), self::SAFE_XML_ENCONDINGS )
+					&& !in_array( strtoupper( $encMatch[1] ), self::SAFE_XML_ENCODINGS )
 				) {
 					wfDebug( __METHOD__ . ": Found unsafe XML encoding '{$encMatch[1]}'" );
 
@@ -1710,15 +1733,17 @@ abstract class UploadBase {
 
 		// Check <style> css
 		if ( $strippedElement === 'style'
-			&& self::checkCssFragment( Sanitizer::normalizeCss( $data ) )
 		) {
-			wfDebug( __METHOD__ . ": hostile css in style element." );
+			$cssCheck = ( new SVGCSSChecker )->checkStyleTag( $data );
+			if ( $cssCheck !== true ) {
+				wfDebug( __METHOD__ . ": hostile css in style element." );
 
-			return [ 'uploaded-hostile-svg' ];
+				return [ 'uploaded-hostile-svg', $cssCheck[0], $cssCheck[1], $cssCheck[2] ];
+			}
 		}
 
 		static $cssAttrs = [ 'font', 'clip-path', 'fill', 'filter', 'marker',
-			'marker-end', 'marker-mid', 'marker-start', 'mask', 'stroke' ];
+			'marker-end', 'marker-mid', 'marker-start', 'mask', 'stroke', 'cursor' ];
 
 		foreach ( $attribs as $attrib => $value ) {
 			// If attributeNamespace is '', it is relative to its element's namespace
@@ -1821,7 +1846,7 @@ abstract class UploadBase {
 
 			// use CSS styles to bring in remote code.
 			if ( $stripped === 'style'
-				&& self::checkCssFragment( Sanitizer::normalizeCss( $value ) )
+				&& ( new SVGCSSChecker )->checkStyleAttribute( $value ) !== true
 			) {
 				wfDebug( __METHOD__ . ": Found svg setting a style with "
 					. "remote url '$attrib'='$value' in uploaded file." );
@@ -1830,7 +1855,7 @@ abstract class UploadBase {
 
 			// Several attributes can include css, css character escaping isn't allowed.
 			if ( in_array( $stripped, $cssAttrs, true )
-				&& self::checkCssFragment( $value )
+				&& ( new SVGCSSChecker )->checkPresentationalAttribute( $value ) !== true
 			) {
 				wfDebug( __METHOD__ . ": Found svg setting a style with "
 					. "remote url '$attrib'='$value' in uploaded file." );
@@ -1854,52 +1879,6 @@ abstract class UploadBase {
 		return false; // No scripts detected
 	}
 
-	/**
-	 * Check a block of CSS or CSS fragment for anything that looks like
-	 * it is bringing in remote code.
-	 * @param string $value a string of CSS
-	 * @return bool true if the CSS contains an illegal string, false if otherwise
-	 */
-	private static function checkCssFragment( $value ) {
-		# Forbid external stylesheets, for both reliability and to protect viewer's privacy
-		if ( stripos( $value, '@import' ) !== false ) {
-			return true;
-		}
-
-		# We allow @font-face to embed fonts with data: urls, so we snip the string
-		# 'url' out so that this case won't match when we check for urls below
-		$pattern = '!(@font-face\s*{[^}]*src:)url(\("data:;base64,)!im';
-		$value = preg_replace( $pattern, '$1$2', $value );
-
-		# Check for remote and executable CSS. Unlike in Sanitizer::checkCss, the CSS
-		# properties filter and accelerator don't seem to be useful for xss in SVG files.
-		# Expression and -o-link don't seem to work either, but filtering them here in case.
-		# Additionally, we catch remote urls like url("http:..., url('http:..., url(http:...,
-		# but not local ones such as url("#..., url('#..., url(#....
-		if ( preg_match( '!expression
-				| -o-link\s*:
-				| -o-link-source\s*:
-				| -o-replace\s*:!imx', $value ) ) {
-			return true;
-		}
-
-		if ( preg_match_all(
-				"!(\s*(url|image|image-set)\s*\(\s*[\"']?\s*[^#]+.*?\))!sim",
-				$value,
-				$matches
-			) !== 0
-		) {
-			# TODO: redo this in one regex. Until then, url("#whatever") matches the first
-			foreach ( $matches[1] as $match ) {
-				if ( !preg_match( "!\s*(url|image|image-set)\s*\(\s*(#|'#|\"#)!im", $match ) ) {
-					return true;
-				}
-			}
-		}
-
-		return (bool)preg_match( '/[\000-\010\013\016-\037\177]/', $value );
-	}
-
 	/**
 	 * Divide the element name passed by the XML parser to the callback into URI and prefix.
 	 * @param string $element
@@ -2028,6 +2007,7 @@ abstract class UploadBase {
 	 * @param Authority $performer
 	 *
 	 * @return bool|array
+	 * @phan-return true|non-empty-array
 	 */
 	private function checkOverwrite( Authority $performer ) {
 		// First check whether the local file can be overwritten
@@ -2229,15 +2209,89 @@ abstract class UploadBase {
 		return $apiUpload->getUploadImageInfo( $this );
 	}
 
-	/**
-	 * @param array $error
-	 * @return Status
-	 */
-	public function convertVerifyErrorToStatus( $error ) {
-		$code = $error['status'];
-		unset( $code['status'] );
+	public function convertVerifyErrorToStatus( array $error ): UploadVerificationStatus {
+		switch ( $error['status'] ) {
+			/** Statuses that only require name changing */
+			case self::MIN_LENGTH_PARTNAME:
+				return UploadVerificationStatus::newFatal( 'filename-tooshort' )
+					->setRecoverableError( true )
+					->setInvalidParameter( 'filename' );
 
-		return Status::newFatal( $this->getVerificationErrorCode( $code ), $error );
+			case self::ILLEGAL_FILENAME:
+				return UploadVerificationStatus::newFatal( 'illegal-filename' )
+					->setRecoverableError( true )
+					->setInvalidParameter( 'filename' )
+					->setApiData( [ 'filename' => $error['filtered'] ] );
+
+			case self::FILENAME_TOO_LONG:
+				return UploadVerificationStatus::newFatal( 'filename-toolong' )
+					->setRecoverableError( true )
+					->setInvalidParameter( 'filename' );
+
+			case self::FILETYPE_MISSING:
+				return UploadVerificationStatus::newFatal( 'filetype-missing' )
+					->setRecoverableError( true )
+					->setInvalidParameter( 'filename' );
+
+			case self::WINDOWS_NONASCII_FILENAME:
+				return UploadVerificationStatus::newFatal( 'windows-nonascii-filename' )
+					->setRecoverableError( true )
+					->setInvalidParameter( 'filename' );
+
+			/** Statuses that require reuploading */
+			case self::EMPTY_FILE:
+				return UploadVerificationStatus::newFatal( 'empty-file' );
+
+			case self::FILE_TOO_LARGE:
+				return UploadVerificationStatus::newFatal( 'file-too-large' );
+
+			case self::FILETYPE_BADTYPE:
+				$extensions = array_unique( MediaWikiServices::getInstance()
+					->getMainConfig()->get( MainConfigNames::FileExtensions ) );
+				$extradata = [
+					'filetype' => $error['finalExt'],
+					'allowed' => array_values( $extensions ),
+				];
+				ApiResult::setIndexedTagName( $extradata['allowed'], 'ext' );
+				if ( isset( $error['blacklistedExt'] ) ) {
+					$bannedTypes = $error['blacklistedExt'];
+					$extradata['blacklisted'] = array_values( $bannedTypes );
+					ApiResult::setIndexedTagName( $extradata['blacklisted'], 'ext' );
+				} else {
+					$bannedTypes = [ $error['finalExt'] ];
+				}
+				'@phan-var string[] $bannedTypes';
+				return UploadVerificationStatus::newFatal(
+						'filetype-banned-type',
+						Message::listParam( $bannedTypes, ListType::COMMA ),
+						Message::listParam( $extensions, ListType::COMMA ),
+						count( $extensions ),
+						// Add PLURAL support for the first parameter. This results
+						// in a bit unlogical parameter sequence, but does not break
+						// old translations
+						count( $bannedTypes )
+					)
+					->setApiCode( 'filetype-banned' )
+					->setApiData( $extradata );
+
+			case self::VERIFICATION_ERROR:
+				$msg = ApiMessage::create( $error['details'], 'verification-error' );
+				if ( $error['details'][0] instanceof MessageSpecifier ) {
+					$apiDetails = [ $msg->getKey(), ...$msg->getParams() ];
+				} else {
+					$apiDetails = $error['details'];
+				}
+				ApiResult::setIndexedTagName( $apiDetails, 'detail' );
+				$msg->setApiData( $msg->getApiData() + [ 'details' => $apiDetails ] );
+				return UploadVerificationStatus::newFatal( $msg );
+
+			default:
+				// @codeCoverageIgnoreStart
+				return UploadVerificationStatus::newFatal( 'upload-unknownerror-nocode' )
+					->setApiCode( 'unknown-error' )
+					->setApiData( [ 'details' => [ 'code' => $error['status'] ] ] );
+				// @codeCoverageIgnoreEnd
+		}
 	}
 
 	/**
