@@ -20,10 +20,16 @@
  * @file
  */
 
+namespace MediaWiki\Xml;
+
 use MediaWiki\Html\Html;
 use MediaWiki\Languages\LanguageNameUtils;
 use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Message\Message;
+use MediaWiki\Parser\Sanitizer;
+use MediaWiki\Utils\MWTimestamp;
+use UtfNormal\Validator;
 
 /**
  * Module of static functions for generating XML
@@ -36,10 +42,14 @@ class Xml {
 	 * characters (<, >, &) are escaped but illegals are not touched.
 	 *
 	 * @param string $element Element name
+	 * @param-taint $element tainted
 	 * @param array|null $attribs Name=>value pairs. Values will be escaped.
+	 * @param-taint $attribs escapes_html
 	 * @param string|null $contents Null to make an open tag only; '' for a contentless closed tag (default)
+	 * @param-taint $contents escapes_html
 	 * @param bool $allowShortTag Whether '' in $contents will result in a contentless closed tag
 	 * @return string
+	 * @return-taint escaped
 	 */
 	public static function element( $element, $attribs = null, $contents = '',
 		$allowShortTag = true
@@ -64,21 +74,17 @@ class Xml {
 	 * The values are passed to Sanitizer::encodeAttribute.
 	 * Returns null or empty string if no attributes given.
 	 * @param array|null $attribs Array of attributes for an XML element
-	 * @throws MWException
 	 * @return null|string
 	 */
-	public static function expandAttributes( $attribs ) {
-		$out = '';
+	public static function expandAttributes( ?array $attribs ) {
 		if ( $attribs === null ) {
 			return null;
-		} elseif ( is_array( $attribs ) ) {
-			foreach ( $attribs as $name => $val ) {
-				$out .= " {$name}=\"" . Sanitizer::encodeAttribute( $val ) . '"';
-			}
-			return $out;
-		} else {
-			throw new MWException( 'Expected attribute array, got something else in ' . __METHOD__ );
 		}
+		$out = '';
+		foreach ( $attribs as $name => $val ) {
+			$out .= " {$name}=\"" . Sanitizer::encodeAttribute( $val ) . '"';
+		}
+		return $out;
 	}
 
 	/**
@@ -94,7 +100,7 @@ class Xml {
 	 */
 	public static function elementClean( $element, $attribs = [], $contents = '' ) {
 		if ( $attribs ) {
-			$attribs = array_map( [ UtfNormal\Validator::class, 'cleanUp' ], $attribs );
+			$attribs = array_map( [ Validator::class, 'cleanUp' ], $attribs );
 		}
 		if ( $contents ) {
 			$contents =
@@ -128,9 +134,13 @@ class Xml {
 	 * content you have is already valid xml.
 	 *
 	 * @param string $element Element name
+	 * @param-taint $element tainted
 	 * @param array|null $attribs Array of attributes
+	 * @param-taint $attribs escapes_html
 	 * @param string $contents Content of the element
+	 * @param-taint $contents tainted
 	 * @return string
+	 * @return-taint escaped
 	 */
 	public static function tags( $element, $attribs, $contents ) {
 		return self::openElement( $element, $attribs ) . $contents . "</$element>";
@@ -144,8 +154,12 @@ class Xml {
 	 *   Null to not include (default).
 	 * @param string $id Element identifier
 	 * @return string Html string containing the month selector
+	 *
+	 * @deprecated since 1.42
 	 */
 	public static function monthSelector( $selected = '', $allmonths = null, $id = 'month' ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		global $wgLang;
 		$options = [];
 
@@ -167,8 +181,11 @@ class Xml {
 	 * @param int|string $month A month in the 1..12 range. Use '', 0 or -1 to start with no month
 	 *  preselected.
 	 * @return string Formatted HTML
+	 *
+	 * @deprecated since 1.42
 	 */
 	public static function dateMenu( $year, $month ) {
+		wfDeprecated( __METHOD__, '1.42' );
 		# Offset overrides year/month selection
 		if ( $month && $month !== -1 ) {
 			$encMonth = intval( $month );
@@ -204,10 +221,13 @@ class Xml {
 	 * @param array $overrideAttrs Override the attributes of the select tag (since 1.20)
 	 * @param Message|null $msg Label message key (since 1.20)
 	 * @return array Array containing 2 items: label HTML and select list HTML
+	 *
+	 * @deprecated since 1.42
 	 */
 	public static function languageSelector( $selected, $customisedOnly = true,
-		$inLanguage = null, $overrideAttrs = [], Message $msg = null
+		$inLanguage = null, $overrideAttrs = [], ?Message $msg = null
 	) {
+		wfDeprecated( __METHOD__, '1.42' );
 		$languageCode = MediaWikiServices::getInstance()->getMainConfig()
 			->get( MainConfigNames::LanguageCode );
 
@@ -238,9 +258,7 @@ class Xml {
 		$attrs = [ 'id' => 'wpUserLanguage', 'name' => 'wpUserLanguage' ];
 		$attrs = array_merge( $attrs, $overrideAttrs );
 
-		if ( $msg === null ) {
-			$msg = wfMessage( 'yourlanguage' );
-		}
+		$msg ??= wfMessage( 'yourlanguage' );
 		return [
 			self::label( $msg->text(), $attrs['id'] ),
 			self::tags( 'select', $attrs, $options )
@@ -253,20 +271,28 @@ class Xml {
 	 * @param string $class Class name of the span element
 	 * @param array $attribs Other attributes
 	 * @return string
+	 *
+	 * @deprecated since 1.42, use {@see Html::element} instead; emiting deprecation warnings since 1.44
 	 */
 	public static function span( $text, $class, $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::element( 'span', [ 'class' => $class ] + $attribs, $text );
 	}
 
 	/**
 	 * Shortcut to make a specific element with a class attribute
+	 *
 	 * @param string $text Content of the element, will be escaped
 	 * @param string $class Class name of the span element
 	 * @param string $tag Element name
 	 * @param array $attribs Other attributes
 	 * @return string
+	 *
+	 * @deprecated since 1.42, use {@see Xml::tags} instead
 	 */
 	public static function wrapClass( $text, $class, $tag = 'span', $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
 		return self::tags( $tag, [ 'class' => $class ] + $attribs, $text );
 	}
 
@@ -277,8 +303,12 @@ class Xml {
 	 * @param string|false $value Value of the value attribute
 	 * @param array $attribs Other attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::input} instead; emitting warnings since 1.44
 	 */
 	public static function input( $name, $size = false, $value = false, $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		$attributes = [ 'name' => $name ];
 
 		if ( $size ) {
@@ -289,8 +319,7 @@ class Xml {
 			$attributes['value'] = $value;
 		}
 
-		return self::element( 'input',
-			Html::getTextInputAttributes( $attributes + $attribs ) );
+		return self::element( 'input', $attributes + $attribs );
 	}
 
 	/**
@@ -300,10 +329,14 @@ class Xml {
 	 * @param string|false $value Value of the value attribute
 	 * @param array $attribs Other attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::input} instead; emiting deprecation warnings since 1.44
 	 */
 	public static function password( $name, $size = false, $value = false,
 		$attribs = []
 	) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::input( $name, $size, $value,
 			array_merge( $attribs, [ 'type' => 'password' ] ) );
 	}
@@ -315,6 +348,8 @@ class Xml {
 	 * @param bool $present
 	 *
 	 * @return array
+	 *
+	 * @deprecated since 1.42; only for use in methods being deprecated
 	 */
 	public static function attrib( $name, $present = true ) {
 		return $present ? [ $name => $name ] : [];
@@ -326,8 +361,12 @@ class Xml {
 	 * @param bool $checked Whether the checkbox is checked or not
 	 * @param array $attribs Array other attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::check} instead; emiting warnings since 1.44
 	 */
 	public static function check( $name, $checked = false, $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::element( 'input', array_merge(
 			[
 				'name' => $name,
@@ -344,8 +383,12 @@ class Xml {
 	 * @param bool $checked Whether the checkbox is checked or not
 	 * @param array $attribs Other attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::radio} instead; emiting warnings since 1.44
 	 */
 	public static function radio( $name, $value, $checked = false, $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::element( 'input', [
 			'name' => $name,
 			'type' => 'radio',
@@ -361,6 +404,8 @@ class Xml {
 	 *     so this function will cherry-pick appropriate attributes to
 	 *     apply to the label as well; only class and title are applied.
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::label} instead
 	 */
 	public static function label( $label, $id, $attribs = [] ) {
 		$a = [ 'for' => $id ];
@@ -383,10 +428,14 @@ class Xml {
 	 * @param string|false $value Value of the value attribute
 	 * @param array $attribs Other attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::input} and {@see Html::label} instead; emitting warnings since 1.44
 	 */
 	public static function inputLabel( $label, $name, $id, $size = false,
 		$value = false, $attribs = []
 	) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		[ $label, $input ] = self::inputLabelSep( $label, $name, $id, $size, $value, $attribs );
 		return $label . "\u{00A0}" . $input;
 	}
@@ -400,12 +449,15 @@ class Xml {
 	 * @param int|false $size
 	 * @param string|false $value
 	 * @param array $attribs
-	 *
 	 * @return array
+	 *
+	 * @deprecated since 1.42, use {@see Html::input} and {@see Html::label} instead; emitting warnings since 1.44
 	 */
 	public static function inputLabelSep( $label, $name, $id, $size = false,
 		$value = false, $attribs = []
 	) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return [
 			self::label( $label, $id, $attribs ),
 			self::input( $name, $size, $value, [ 'id' => $id ] + $attribs )
@@ -420,21 +472,16 @@ class Xml {
 	 * @param string $id
 	 * @param bool $checked
 	 * @param array $attribs
-	 *
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::check} and {@see Html::label} instead; emitting warnings since 1.44
 	 */
 	public static function checkLabel( $label, $name, $id, $checked = false, $attribs = [] ) {
-		$useMediaWikiUIEverywhere = MediaWikiServices::getInstance()->getMainConfig()
-			->get( MainConfigNames::UseMediaWikiUIEverywhere );
-		$chkLabel = self::check( $name, $checked, [ 'id' => $id ] + $attribs ) .
+		wfDeprecated( __METHOD__, '1.42' );
+
+		return self::check( $name, $checked, [ 'id' => $id ] + $attribs ) .
 			"\u{00A0}" .
 			self::label( $label, $id, $attribs );
-
-		if ( $useMediaWikiUIEverywhere ) {
-			$chkLabel = self::openElement( 'div', [ 'class' => 'mw-ui-checkbox' ] ) .
-				$chkLabel . self::closeElement( 'div' );
-		}
-		return $chkLabel;
 	}
 
 	/**
@@ -446,39 +493,36 @@ class Xml {
 	 * @param string $id
 	 * @param bool $checked
 	 * @param array $attribs
-	 *
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::radio} and {@see Html::label} instead; emitting warnings since 1.44
 	 */
 	public static function radioLabel( $label, $name, $value, $id,
 		$checked = false, $attribs = []
 	) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::radio( $name, $value, $checked, [ 'id' => $id ] + $attribs ) .
 			"\u{00A0}" .
 			self::label( $label, $id, $attribs );
 	}
 
 	/**
-	 * Convenience function to build an HTML submit button
-	 * When $wgUseMediaWikiUIEverywhere is true it will default to a progressive button
+	 * Convenience function to build an HTML submit button.
+	 *
 	 * @param string $value Label text for the button (unescaped)
 	 * @param array $attribs Optional custom attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::submitButton} instead; emitting deprecation warnings since 1.44
 	 */
 	public static function submitButton( $value, $attribs = [] ) {
-		$useMediaWikiUIEverywhere = MediaWikiServices::getInstance()->getMainConfig()
-			->get( MainConfigNames::UseMediaWikiUIEverywhere );
-		$baseAttrs = [
+		wfDeprecated( __METHOD__, '1.42' );
+
+		$attribs += [
 			'type' => 'submit',
 			'value' => $value,
 		];
-		// Done conditionally for time being as it is possible
-		// some submit forms
-		// might need to be mw-ui-destructive (e.g. delete a page)
-		if ( $useMediaWikiUIEverywhere ) {
-			$baseAttrs['class'] = 'mw-ui-button mw-ui-progressive';
-		}
-		// Any custom attributes will take precedence of anything in baseAttrs e.g. override the class
-		$attribs += $baseAttrs;
 		return Html::element( 'input', $attribs );
 	}
 
@@ -489,6 +533,8 @@ class Xml {
 	 * @param bool $selected If true, will be the default selected item
 	 * @param array $attribs Optional additional HTML attributes
 	 * @return string HTML
+	 *
+	 * @deprecated since 1.42, use {@see Html::element} instead
 	 */
 	public static function option( $text, $value = null, $selected = false,
 			$attribs = [] ) {
@@ -503,7 +549,7 @@ class Xml {
 
 	/**
 	 * Build a drop-down box from a textual list. This is a wrapper
-	 * for Xml::listDropDownOptions() plus the XmlSelect class.
+	 * for Xml::listDropdownOptions() plus the XmlSelect class.
 	 *
 	 * @param string $name Name and id for the drop-down
 	 * @param string $list Correctly formatted text (newline delimited) to be
@@ -513,11 +559,13 @@ class Xml {
 	 * @param string $class CSS classes for the drop-down
 	 * @param int|null $tabindex Value of the tabindex attribute
 	 * @return string
+	 *
+	 * @deprecated since 1.42; use the equivalent methods in Html without a wrapper
 	 */
-	public static function listDropDown( $name = '', $list = '', $other = '',
+	public static function listDropdown( $name = '', $list = '', $other = '',
 		$selected = '', $class = '', $tabindex = null
 	) {
-		$options = self::listDropDownOptions( $list, [ 'other' => $other ] );
+		$options = self::listDropdownOptions( $list, [ 'other' => $other ] );
 
 		$xmlSelect = new XmlSelect( $name, $name, $selected );
 		$xmlSelect->addOptions( $options );
@@ -536,7 +584,7 @@ class Xml {
 	 * Build options for a drop-down box from a textual list.
 	 *
 	 * The result of this function can be passed to XmlSelect::addOptions()
-	 * (to render a plain `<select>` dropdown box) or to Xml::listDropDownOptionsOoui()
+	 * (to render a plain `<select>` dropdown box) or to Xml::listDropdownOptionsOoui()
 	 * and then OOUI\DropdownInputWidget() (to render a pretty one).
 	 *
 	 * @param string $list Correctly formatted text (newline delimited) to be
@@ -544,8 +592,10 @@ class Xml {
 	 * @param array $params Extra parameters:
 	 *   - string $params['other'] If set, add an option with this as text and a value of 'other'
 	 * @return array Array keys are textual labels, values are internal values
+	 *
+	 * @deprecated since 1.42; use the equivalent method in Html
 	 */
-	public static function listDropDownOptions( $list, $params = [] ) {
+	public static function listDropdownOptions( $list, $params = [] ) {
 		$options = [];
 
 		if ( isset( $params['other'] ) ) {
@@ -592,10 +642,12 @@ class Xml {
 	 *
 	 * TODO Find a better home for this function.
 	 *
-	 * @param array $options Options, as returned e.g. by Xml::listDropDownOptions()
+	 * @param array $options Options, as returned e.g. by Xml::listDropdownOptions()
 	 * @return array
+	 *
+	 * @deprecated since 1.42; use the equivalent method in Html
 	 */
-	public static function listDropDownOptionsOoui( $options ) {
+	public static function listDropdownOptionsOoui( $options ) {
 		$optionsOoui = [];
 
 		foreach ( $options as $text => $value ) {
@@ -620,8 +672,9 @@ class Xml {
 	 * @param string|false $content Pre-escaped content for the fieldset. If false,
 	 *   only open fieldset is returned.
 	 * @param array $attribs Any attributes to fieldset-element.
-	 *
 	 * @return string
+	 *
+	 * @deprecated since 1.42, use {@see Html::element} instead
 	 */
 	public static function fieldset( $legend = false, $content = false, $attribs = [] ) {
 		$s = self::openElement( 'fieldset', $attribs ) . "\n";
@@ -646,38 +699,42 @@ class Xml {
 	 * @param int $cols The number of columns for the textarea
 	 * @param int $rows The number of rows for the textarea
 	 * @param array $attribs Any other attributes for the textarea
-	 *
 	 * @return string
+	 *
+	 * @deprecated since 1.42, use {@see Html::textarea} instead; emiting deprecation warnings since 1.44
 	 */
 	public static function textarea( $name, $content, $cols = 40, $rows = 5, $attribs = [] ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		return self::element( 'textarea',
-					Html::getTextInputAttributes(
-						[
-							'name' => $name,
-							'id' => $name,
-							'cols' => $cols,
-							'rows' => $rows
-						] + $attribs
-					),
+					[
+						'name' => $name,
+						'id' => $name,
+						'cols' => $cols,
+						'rows' => $rows
+					] + $attribs,
 					$content, false );
 	}
 
 	/**
 	 * Encode a variable of arbitrary type to JavaScript.
-	 * If the value is an XmlJsCode object, pass through the object's value verbatim.
+	 * If the value is an HtmlJsCode object, pass through the object's value verbatim.
 	 *
 	 * @note Only use this function for generating JavaScript code. If generating output
 	 *       for a proper JSON parser, just call FormatJson::encode() directly.
 	 *
 	 * @param mixed $value The value being encoded. Can be any type except a resource.
+	 * @param-taint $value escapes_html
 	 * @param bool $pretty If true, add non-significant whitespace to improve readability.
 	 * @return string|false String if successful; false upon failure
+	 * @return-taint none
+	 *
+	 * @deprecated since 1.41, use {@see Html::encodeJsVar} instead; emiting deprecation warnings since 1.44
 	 */
 	public static function encodeJsVar( $value, $pretty = false ) {
-		if ( $value instanceof XmlJsCode ) {
-			return $value->value;
-		}
-		return FormatJson::encode( $value, $pretty, FormatJson::UTF8_OK );
+		wfDeprecated( __METHOD__, '1.41' );
+
+		return Html::encodeJsVar( $value, $pretty );
 	}
 
 	/**
@@ -687,22 +744,19 @@ class Xml {
 	 * @since 1.17
 	 * @param string $name The name of the function to call, or a JavaScript expression
 	 *    which evaluates to a function object which is called.
+	 * @param-taint $name tainted
 	 * @param array $args The arguments to pass to the function.
+	 * @param-taint $args escapes_html
 	 * @param bool $pretty If true, add non-significant whitespace to improve readability.
 	 * @return string|false String if successful; false upon failure
+	 * @return-taint none
+	 *
+	 * @deprecated since 1.41, use {@see Html::encodeJsCall} instead; emiting deprecation warnings since 1.44
 	 */
 	public static function encodeJsCall( $name, $args, $pretty = false ) {
-		foreach ( $args as &$arg ) {
-			$arg = self::encodeJsVar( $arg, $pretty );
-			if ( $arg === false ) {
-				return false;
-			}
-		}
+		wfDeprecated( __METHOD__, '1.41' );
 
-		return "$name(" . ( $pretty
-			? ( ' ' . implode( ', ', $args ) . ' ' )
-			: implode( ',', $args )
-		) . ");";
+		return Html::encodeJsCall( $name, $args, $pretty );
 	}
 
 	/**
@@ -723,11 +777,6 @@ class Xml {
 		xml_parser_set_option( $parser, XML_OPTION_CASE_FOLDING, 0 );
 
 		if ( !xml_parse( $parser, $text, true ) ) {
-			// $err = xml_error_string( xml_get_error_code( $parser ) );
-			// $position = xml_get_current_byte_index( $parser );
-			// $fragment = $this->extractFragment( $html, $position );
-			// $this->mXmlError = "$err at byte $position:\n$fragment";
-			xml_parser_free( $parser );
 			return false;
 		}
 
@@ -778,6 +827,8 @@ class Xml {
 	 *   the submit button.
 	 * @param array $submitAttribs The attributes to add to the submit button
 	 * @return string HTML form.
+	 *
+	 * @deprecated since 1.42, use OOUI or Codex widgets instead
 	 */
 	public static function buildForm( $fields, $submitLabel = null, $submitAttribs = [] ) {
 		$form = '';
@@ -801,7 +852,13 @@ class Xml {
 			$form .= self::openElement( 'tr' );
 			$form .= self::tags( 'td', [], '' );
 			$form .= self::openElement( 'td', [ 'class' => 'mw-submit' ] )
-				. self::submitButton( wfMessage( $submitLabel )->text(), $submitAttribs )
+				. Html::element(
+					'input',
+					$submitAttribs + [
+						'type' => 'submit',
+						'value' => wfMessage( $submitLabel )->text(),
+					]
+				)
 				. self::closeElement( 'td' );
 			$form .= self::closeElement( 'tr' );
 		}
@@ -816,8 +873,12 @@ class Xml {
 	 * @param array|null $attribs An array of attributes to apply to the table tag
 	 * @param array|null $headers An array of strings to use as table headers
 	 * @return string
+	 *
+	 * @deprecated since 1.42; use OOUI or Codex widgets instead; emiting deprecation warnings since 1.44
 	 */
 	public static function buildTable( $rows, $attribs = [], $headers = null ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		$s = self::openElement( 'table', $attribs );
 
 		if ( is_array( $headers ) ) {
@@ -855,8 +916,12 @@ class Xml {
 	 * @param array|null $attribs An array of attributes to apply to the tr tag
 	 * @param string[] $cells An array of strings to put in <td>
 	 * @return string
+	 *
+	 * @deprecated since 1.42; use OOUI or Codex widgets instead; emiting deprecation warnings since 1.44
 	 */
 	public static function buildTableRow( $attribs, $cells ) {
+		wfDeprecated( __METHOD__, '1.42' );
+
 		$s = self::openElement( 'tr', $attribs );
 
 		foreach ( $cells as $id => $cell ) {
@@ -874,3 +939,5 @@ class Xml {
 		return $s;
 	}
 }
+/** @deprecated class alias since 1.43 */
+class_alias( Xml::class, 'Xml' );
