@@ -131,7 +131,7 @@ $magicWords = [
 	'noeditsection'             => [ '0', '__TANPASUNTINGANBAGIAN__', '__NIRSUBA__', '__NOEDITSECTION__' ],
 	'nogallery'                 => [ '0', '__TANPAGALERI__', '__NIRGAL__', '__NOGALLERY__' ],
 	'noindex'                   => [ '1', '__TANPAINDEKS__', '__NIRDEKS__', '__NOINDEX__' ],
-	'nonewsectionlink'          => [ '1', '_TANPAPRANALABAGIANBARU__', '__NIRPRABABA__', '__NONEWSECTIONLINK__' ],
+	'nonewsectionlink'          => [ '1', '__TANPAPRANALABAGIANBARU__', '_TANPAPRANALABAGIANBARU__', '__NIRPRABABA__', '__NONEWSECTIONLINK__' ],
 	'notitleconvert'            => [ '0', '__TANPAKONVERSIJUDUL__', '__NIRKODUL__', '__NOTITLECONVERT__', '__NOTC__' ],
 	'notoc'                     => [ '0', '__TANPADAFTARISI__', '__NIRDASI__', '__NOTOC__' ],
 	'ns'                        => [ '0', 'RN:', 'RUNAM:', 'NS:' ],
