@@ -12,18 +12,18 @@ $namespaceNames = [
 	NS_SPECIAL          => 'ߞߙߍߞߙߍߣߍ߲',
 	NS_TALK             => 'ߢߊߝߐߞߣߍ',
 	NS_USER             => 'ߟߊߓߊ߯ߙߟߊ',
-	NS_USER_TALK        => 'ߟߊߓߊ߯ߙߟߊ ߟߊ߫ ߢߊߝߐߞߣߍ',
-	NS_PROJECT_TALK     => '$1 ߢߊߝߐߞߣߍ',
+	NS_USER_TALK        => 'ߟߊߓߊ߯ߙߟߊ_ߟߊ߫_ߢߊߝߐߞߣߍ',
+	NS_PROJECT_TALK     => '$1_ߢߊߝߐߞߣߍ',
 	NS_FILE             => 'ߞߐߕߐ߮',
-	NS_FILE_TALK        => 'ߞߐߕߐ߮ ߢߊߝߐߞߣߍ',
+	NS_FILE_TALK        => 'ߞߐߕߐ߮_ߢߊߝߐߞߣߍ',
 	NS_MEDIAWIKI        => 'ߡߘߌߦߊߥߞߌ',
-	NS_MEDIAWIKI_TALK   => 'ߡߘߌߦߊߥߞߌ ߢߊߝߐߞߣߍ',
+	NS_MEDIAWIKI_TALK   => 'ߡߘߌߦߊߥߞߌ_ߢߊߝߐߞߣߍ',
 	NS_TEMPLATE         => 'ߞߙߊߞߏ',
-	NS_TEMPLATE_TALK    => 'ߞߙߊߞߏ ߢߊߝߐߞߣߍ',
+	NS_TEMPLATE_TALK    => 'ߞߙߊߞߏ_ߢߊߝߐߞߣߍ',
 	NS_HELP             => 'ߡߊ߬ߘߍ߬ߡߍ߲߬ߠߌ߲',
-	NS_HELP_TALK        => 'ߡߊ߬ߘߍ߬ߡߍ߲߬ߠߌ߲ ߢߊߝߐߞߣߍ',
+	NS_HELP_TALK        => 'ߡߊ߬ߘߍ߬ߡߍ߲߬ߠߌ߲_ߢߊߝߐߞߣߍ',
 	NS_CATEGORY         => 'ߦߌߟߡߊ',
-	NS_CATEGORY_TALK    => 'ߦߌߟߡߊ ߢߊߝߐߞߣߍ',
+	NS_CATEGORY_TALK    => 'ߦߌߟߡߊ_ߢߊߝߐߞߣߍ',
 ];
 
 $digitTransformTable = [
@@ -38,3 +38,5 @@ $digitTransformTable = [
 	'8' => '߈', # U+07C8
 	'9' => '߉', # U+07C9
 ];
+
+$numberingSystem = 'nkoo';
