@@ -4,6 +4,7 @@
  * @file
  * @ingroup Languages
  *
+ * @author Aca
  * @author FriedrickMILBarbarossa
  * @author Geitost
  * @author Kaganer
@@ -15,6 +16,7 @@
  * @author Red Baron
  * @author Reedy
  * @author Slaven Kosanovic
+ * @author Srđan
  * @author TheStefan12345
  * @author Жељко Тодоровић
  * @author Михајло Анђелковић
@@ -28,7 +30,7 @@ $namespaceNames = [
 	NS_SPECIAL          => 'Posebno',
 	NS_TALK             => 'Razgovor',
 	NS_USER             => 'Korisnik',
-	NS_USER_TALK        => 'Razgovor_sa_korisnikom',
+	NS_USER_TALK        => 'Razgovor_s_korisnikom',
 	NS_PROJECT_TALK     => 'Razgovor_o_$1',
 	NS_FILE             => 'Datoteka',
 	NS_FILE_TALK        => 'Razgovor_o_datoteci',
@@ -42,28 +44,31 @@ $namespaceNames = [
 	NS_CATEGORY_TALK    => 'Razgovor_o_kategoriji',
 ];
 
-# Aliases to cyrillic namespaces
 $namespaceAliases = [
-	"Медија"                  => NS_MEDIA,
-	"Посебно"                 => NS_SPECIAL,
-	"Разговор"                => NS_TALK,
-	"Корисник"                => NS_USER,
-	"Разговор_са_корисником"  => NS_USER_TALK,
-	"Разговор_о_$1"           => NS_PROJECT_TALK,
-	"Слика"                   => NS_FILE,
-	"Разговор_о_слици"        => NS_FILE_TALK,
-	"МедијаВики"              => NS_MEDIAWIKI,
-	"Разговор_о_МедијаВикију" => NS_MEDIAWIKI_TALK,
-	'Шаблон'                  => NS_TEMPLATE,
-	'Разговор_о_шаблону'      => NS_TEMPLATE_TALK,
-	'Помоћ'                   => NS_HELP,
-	'Разговор_о_помоћи'       => NS_HELP_TALK,
-	'Категорија'              => NS_CATEGORY,
-	'Разговор_о_категорији'   => NS_CATEGORY_TALK,
-
-	'Medija'                  => NS_MEDIA,
-	'Slika'                   => NS_FILE,
-	'Razgovor_o_slici'        => NS_FILE_TALK,
+	'Medija'                   => NS_MEDIA,
+	'Specijalno'               => NS_SPECIAL,
+	'Saradnik'                 => NS_USER,
+	'Suradnik'                 => NS_USER,
+	'Razgovor_sa_korisnikom'   => NS_USER_TALK,
+	'Razgovor_sa_saradnikom'   => NS_USER_TALK,
+	'Razgovor_s_saradnikom'    => NS_USER_TALK,
+	'Razgovor_sa_suradnikom'   => NS_USER_TALK,
+	'Razgovor_s_suradnikom'    => NS_USER_TALK,
+	'Fotografija'              => NS_FILE,
+	'Slika'                    => NS_FILE,
+	'Razgovor_o_fotografiji'   => NS_FILE_TALK,
+	'Razgovor_o_slici'         => NS_FILE_TALK,
+	'MedijaViki'               => NS_MEDIAWIKI,
+	'MedijaWiki'               => NS_MEDIAWIKI,
+	'Razgovor_o_MedijaVikiju'  => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_MedijaViki-ju' => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_MediaWiki-ju'  => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_MediaWikiju'   => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_MedijaWiki-ju' => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_MedijaWikiju'  => NS_MEDIAWIKI_TALK,
+	'Razgovor_o_Medijaviki-ju' => NS_MEDIAWIKI_TALK,
+	'Predložak'                => NS_TEMPLATE,
+	'Razgovor_o_predlošku'     => NS_TEMPLATE_TALK,
 ];
 
 $datePreferenceMigrationMap = [
@@ -90,7 +95,7 @@ $specialPageAliases = [
 	'Allpages'                  => [ 'Sve_stranice', 'SveStranice' ],
 	'Ancientpages'              => [ 'Najstarije_stranice', 'NajstarijeStranice', 'NajstarijiČlanci' ],
 	'ApiHelp'                   => [ 'Pomoć_oko_API-ja', 'API_pomoć' ],
-	'ApiSandbox'                => [ 'Pesak_API-ja', 'API_pesak' ],
+	'ApiSandbox'                => [ 'Pesak_API-ja', 'Pijesak_API-ja', 'API_pesak', 'API_pijesak' ],
 	'AutoblockList'             => [ 'Spisak_autoblokada', 'SpisakAutoblokova', 'Autoblokade', 'Autoblokovi' ],
 	'Badtitle'                  => [ 'Neispravan_naslov', 'Loš_naslov', 'LošNaslov' ],
 	'Blankpage'                 => [ 'Prazna_stranica', 'PraznaStranica' ],
@@ -98,12 +103,12 @@ $specialPageAliases = [
 	'BlockList'                 => [ 'Spisak_blokada', 'SpisakBlokada', 'SpisakBlokiranih', 'PopisBlokiranih' ],
 	'Booksources'               => [ 'Štampani_izvori', 'ŠtampaniIzvori', 'KnjiževniIzvori' ],
 	'BotPasswords'              => [ 'Lozinke_botova', 'LozinkeBotova' ],
-	'BrokenRedirects'           => [ 'Pokvarena_preusmerenja', 'Neispravna_preusmerenja', 'PokvarenaPreusmerenja' ],
+	'BrokenRedirects'           => [ 'Pokvarena_preusmerenja', 'Pokvarena_preusmjerenja', 'Neispravna_preusmerenja', 'Neispravna_preusmjerenja', 'PokvarenaPreusmerenja', 'PokvarenaPreusmjerenja' ],
 	'Categories'                => [ 'Kategorije' ],
-	'ChangeContentModel'        => [ 'Promeni_sadržinski_model', 'PromeniModelSadržaja', 'IzmeniModelSadržaja' ],
-	'ChangeCredentials'         => [ 'Promeni_akreditive', 'PromeniAkreditive' ],
-	'ChangeEmail'               => [ 'Promeni_imejl', 'PromeniImejl', 'PromeniImejlAdresu' ],
-	'ChangePassword'            => [ 'Promeni_lozinku', 'PromeniLozinku' ],
+	'ChangeContentModel'        => [ 'Promeni_sadržinski_model', 'Promijeni_sadržinski_model', 'PromeniModelSadržaja', 'PromijeniModelSadržaja', 'IzmeniModelSadržaja', 'IzmijeniModelSadržaja' ],
+	'ChangeCredentials'         => [ 'Promeni_akreditive', 'Promijeni_akreditive', 'PromeniAkreditive', 'PromijeniAkreditive' ],
+	'ChangeEmail'               => [ 'Promeni_imejl', 'Promijeni_imejl', 'PromeniImejl', 'PromijeniImejl', 'PromeniImejlAdresu', 'PromijeniImejlAdresu' ],
+	'ChangePassword'            => [ 'Promeni_lozinku', 'Promijeni_lozinku', 'PromeniLozinku', 'PromijeniLozinku' ],
 	'ComparePages'              => [ 'Uporedi_stranice', 'UporediStranice' ],
 	'Confirmemail'              => [ 'Potvrdi_imejl', 'PotvrdiImejl', 'PotvrdiE-poštu', 'Potvrda_e-pošte' ],
 	'Contributions'             => [ 'Doprinosi', 'Prilozi' ],
@@ -111,18 +116,19 @@ $specialPageAliases = [
 	'Deadendpages'              => [ 'Ćorsokaci', 'StraniceKojeNeVodeNikuda', 'SlepeStranice' ],
 	'DeletedContributions'      => [ 'Izbrisani_doprinosi', 'ObrisaniDoprinosi' ],
 	'Diff'                      => [ 'Razlike' ],
-	'DoubleRedirects'           => [ 'Dvostruka_preusmerenja', 'DvostrukaPreusmerenja' ],
+	'DoubleRedirects'           => [ 'Dvostruka_preusmerenja', 'Dvostruka_preusmjerenja', 'DvostrukaPreusmerenja', 'DvostrukaPreusmjerenja' ],
 	'EditPage'                  => [ 'Uredi_stranicu', 'UrediStranicu', 'Uredi' ],
 	'EditTags'                  => [ 'Uredi_oznake', 'UrediOznake' ],
 	'EditWatchlist'             => [ 'Uredi_spisak_nadgledanja', 'UrediSpisakNadgledanja' ],
 	'Emailuser'                 => [ 'Pošalji_imejl_korisniku', 'PošaljiImejlKorisniku' ],
 	'ExpandTemplates'           => [ 'Proširi_šablone', 'ProširiŠablone' ],
 	'Export'                    => [ 'Izvezi' ],
-	'Fewestrevisions'           => [ 'Najmanje_izmena', 'NajmanjeIzmena', 'ČlanciSaNajmanjeRevizija' ],
+	'Fewestrevisions'           => [ 'Najmanje_izmena', 'Najmanje_izmjena', 'NajmanjeIzmena', 'NajmanjeIzmjena', 'ČlanciSaNajmanjeRevizija' ],
 	'FileDuplicateSearch'       => [ 'Pretraga_duplikata_datoteka', 'PretragaDuplikataDatoteka' ],
 	'Filepath'                  => [ 'Putanja_do_datoteke', 'Putanja_datoteke', 'PutanjaDatoteke' ],
-	'GoToInterwiki'             => [ 'Poseti_međuviki', 'PosetiMeđuviki' ],
+	'GoToInterwiki'             => [ 'Poseti_međuviki', 'Posjeti_međuviki', 'PosetiMeđuviki', 'PosjetiMeđuviki' ],
 	'Import'                    => [ 'Uvezi' ],
+	'Interwiki'                 => [ 'Međuviki' ],
 	'Invalidateemail'           => [ 'Otkaži_potvrdu_imejla', 'PoništiImejl' ],
 	'JavaScriptTest'            => [ 'Testiranje_JavaScript-a', 'Testiranje_JavaScripta', 'TestiranjeJavaskripta' ],
 	'LinkAccounts'              => [ 'Poveži_naloge', 'PovežiNaloge' ],
@@ -133,10 +139,10 @@ $specialPageAliases = [
 	'Listfiles'                 => [ 'Spisak_datoteka', 'SpisakDatoteka', 'SpisakSlika' ],
 	'Listgrants'                => [ 'Spisak_dozvola', 'SpisakDozvola' ],
 	'Listgrouprights'           => [ 'Spisak_prava_korisničkih_grupa', 'Spisak_korisničkih_prava', 'SpisakKorisničkihPrava' ],
-	'Listredirects'             => [ 'Spisak_preusmerenja', 'SpisakPreusmerenja' ],
+	'Listredirects'             => [ 'Spisak_preusmerenja', 'Spisak_preusmjerenja', 'SpisakPreusmerenja', 'SpisakPreusmjerenja' ],
 	'Listusers'                 => [ 'Spisak_korisnika', 'SpisakKorisnika', 'Korisnički_spisak', 'KorisničkiSpisak' ],
 	'Lockdb'                    => [ 'Zaključaj_bazu_podataka', 'ZaključajBazuPodataka', 'Zaključaj_bazu', 'ZaključajBazu' ],
-	'Log'                       => [ 'Dnevnik', 'Izveštaj', 'Izveštaji' ],
+	'Log'                       => [ 'Dnevnik', 'Izveštaj', 'Izvještaj', 'Izveštaji', 'Izvještaji' ],
 	'Lonelypages'               => [ 'Siročići' ],
 	'Longpages'                 => [ 'Duge_stranice', 'DugačkeStranice', 'DugačkeStrane' ],
 	'MediaStatistics'           => [ 'Statistika_datoteka', 'StatistikeMedija' ],
@@ -148,8 +154,8 @@ $specialPageAliases = [
 	'Mostlinked'                => [ 'Najpovezanije_stranice', 'NajpovezanijeStranice', 'NajpovezanijeStrane' ],
 	'Mostlinkedcategories'      => [ 'Najpovezanije_kategorije', 'NajpovezanijeKategorije' ],
 	'Mostlinkedtemplates'       => [ 'Najpovezaniji_šabloni', 'NajpovezanijiŠabloni' ],
-	'Mostrevisions'             => [ 'Najviše_izmena', 'NajvišeIzmena', 'NajvišeRevizija', 'ČlanciSaNajvišeRevizija' ],
-	'Movepage'                  => [ 'Premesti_stranicu', 'PremestiStranicu', 'Premesti', 'Preusmeri' ],
+	'Mostrevisions'             => [ 'Najviše_izmena', 'Najviše_izmjena', 'NajvišeIzmena', 'NajvišeIzmjena', 'NajvišeRevizija', 'ČlanciSaNajvišeRevizija' ],
+	'Movepage'                  => [ 'Premesti_stranicu', 'Premjesti_stranicu', 'PremestiStranicu', 'PremjestiStranicu', 'Premesti', 'Premjesti', 'Preusmeri', 'Preusmjeri' ],
 	'Mycontributions'           => [ 'Moji_doprinosi', 'MojiDoprinosi', 'Moji_prilozi' ],
 	'MyLanguage'                => [ 'Moj_jezik', 'MojJezik' ],
 	'Mypage'                    => [ 'Moja_stranica', 'MojaStranica' ],
@@ -157,7 +163,7 @@ $specialPageAliases = [
 	'Myuploads'                 => [ 'Moja_otpremanja', 'MojaOtpremanja', 'Moja_slanja' ],
 	'Newimages'                 => [ 'Nove_datoteke', 'NoveDatoteke', 'NoviFajlovi', 'NoveSlike' ],
 	'Newpages'                  => [ 'Nove_stranice', 'NoveStranice', 'NoveStrane' ],
-	'NewSection'                => [ 'Novi_odeljak', 'NoviOdeljak' ],
+	'NewSection'                => [ 'Novi_odeljak', 'Novi_odjeljak', 'NoviOdeljak', 'NoviOdjeljak' ],
 	'PageData'                  => [ 'Podaci_stranice', 'PodaciStranice' ],
 	'PageHistory'               => [ 'Istorija_stranice', 'IstorijaStranice', 'Istorija' ],
 	'PageLanguage'              => [ 'Jezik_stranice', 'JezikStranice' ],
@@ -169,18 +175,18 @@ $specialPageAliases = [
 	'Prefixindex'               => [ 'Stranice_s_prefiksom', 'StraniceSaPrefiksom' ],
 	'Protectedpages'            => [ 'Zaštićene_stranice', 'ZaštićeneStranice' ],
 	'Protectedtitles'           => [ 'Zaštićeni_naslovi', 'ZaštićeniNaslovi' ],
-	'Purge'                     => [ 'Osveži' ],
+	'Purge'                     => [ 'Osveži', 'Osvježi' ],
 	'RandomInCategory'          => [ 'Nasumična_stranica_u_kategoriji', 'Slučajna_stranica_u_kategoriji', 'Slučajna_strana_u_kategoriji' ],
 	'Randompage'                => [ 'Nasumična_stranica', 'SlučajnaStranica', 'SlučajnaStrana' ],
-	'Randomredirect'            => [ 'Nasumično_preusmerenje', 'SlučajnoPreusmerenje' ],
-	'Randomrootpage'            => [ 'Nasumična_korenska_stranica', 'SlučajnaOsnovnaStranica', 'SlučajnaOsnovnaStrana' ],
-	'Recentchanges'             => [ 'Skorašnje_izmene', 'SkorašnjeIzmene' ],
-	'Recentchangeslinked'       => [ 'Povezane_izmene', 'SrodneIzmene' ],
-	'Redirect'                  => [ 'Preusmerenje' ],
+	'Randomredirect'            => [ 'Nasumično_preusmerenje', 'Nasumično_preusmjerenje', 'SlučajnoPreusmerenje', 'SlučajnoPreusmjerenje' ],
+	'Randomrootpage'            => [ 'Nasumična_korenska_stranica', 'Nasumična_korijenska_stranica', 'SlučajnaOsnovnaStranica', 'SlučajnaOsnovnaStrana' ],
+	'Recentchanges'             => [ 'Skorašnje_izmene', 'Skorašnje_izmjene', 'SkorašnjeIzmene', 'SkorašnjeIzmjene' ],
+	'Recentchangeslinked'       => [ 'Povezane_izmene', 'Povezane_izmjene', 'SrodneIzmene', 'SrodneIzmjene' ],
+	'Redirect'                  => [ 'Preusmerenje', 'Preusmjerenje' ],
 	'RemoveCredentials'         => [ 'Ukloni_akreditive', 'UkloniAkreditive' ],
 	'Renameuser'                => [ 'PreimenujKorisnika', 'Preimenuj_korisnika' ],
 	'ResetTokens'               => [ 'Resetuj_tokene', 'ResetujŽetone' ],
-	'Revisiondelete'            => [ 'Brisanje_izmene', 'BrisanjeIzmene', 'UklanjanjeIzmene' ],
+	'Revisiondelete'            => [ 'Brisanje_izmene', 'Brisanje_izmjene', 'BrisanjeIzmene', 'BrisanjeIzmjene', 'UklanjanjeIzmene', 'UklanjanjeIzmjene' ],
 	'RunJobs'                   => [ 'Izvrši_poslove', 'IzvršiPoslove' ],
 	'Search'                    => [ 'Pretraži' ],
 	'Shortpages'                => [ 'Kratke_stranice', 'KratkeStranice', 'KratkiČlanci' ],
@@ -211,7 +217,7 @@ $specialPageAliases = [
 	'Wantedpages'               => [ 'Tražene_stranice', 'TraženeStranice', 'TraženeStrane' ],
 	'Wantedtemplates'           => [ 'Traženi_šabloni', 'TraženiŠabloni' ],
 	'Watchlist'                 => [ 'Spisak_nadgledanja', 'SpisakNadgledanja' ],
-	'Whatlinkshere'             => [ 'Šta_vodi_dovde', 'ŠtaVodiOvde', 'Šta_je_povezano_ovde' ],
+	'Whatlinkshere'             => [ 'Šta_vodi_ovamo', 'Šta_vodi_dovde', 'ŠtaVodiOvde', 'ŠtaVodiOvdje', 'Šta_je_povezano_ovde', 'Šta_je_povezano_ovdje' ],
 	'Withoutinterwiki'          => [ 'Bez_međuvikija', 'BezMeđuvikija' ],
 ];
 
@@ -293,48 +299,48 @@ $dateFormats = [
 /* NOT USED IN STABLE VERSION */
 /** @phpcs-require-sorted-array */
 $magicWords = [
-	'anchorencode'              => [ '0', 'KODIRANJEVEZE', 'KODIRANJE_VEZE', 'ANCHORENCODE' ],
-	'articlepath'               => [ '0', 'PUTANJAČLANKA', 'PUTANJA_ČLANKA', 'ARTICLEPATH' ],
-	'basepagename'              => [ '1', 'IMEOSNOVE', 'IME_OSNOVE', 'BASEPAGENAME' ],
-	'basepagenamee'             => [ '1', 'IMENAOSNOVA', 'IMENA_OSNOVA', 'BASEPAGENAMEE' ],
-	'contentlanguage'           => [ '1', 'JEZIKSADRŽAJA', 'JEZIK_SADRŽAJA', 'CONTENTLANGUAGE', 'CONTENTLANG' ],
+	'anchorencode'              => [ '0', 'KODIRANJESIDRA', 'KODIRANJEVEZE', 'KODIRANJE_VEZE', 'ANCHORENCODE' ],
+	'articlepath'               => [ '0', 'PUTANJADOČLANKA', 'PUTANJAČLANKA', 'PUTANJA_ČLANKA', 'ARTICLEPATH' ],
+	'basepagename'              => [ '1', 'NAZIVOSNOVNESTRANICE', 'IMEOSNOVE', 'IME_OSNOVE', 'BASEPAGENAME' ],
+	'basepagenamee'             => [ '1', 'NAZIVOSNOVNESTRANICEK', 'IMENAOSNOVA', 'IMENA_OSNOVA', 'BASEPAGENAMEE' ],
+	'contentlanguage'           => [ '1', 'JEZIKSADRŽINE', 'JEZIKSADRŽAJA', 'JEZIK_SADRŽAJA', 'CONTENTLANGUAGE', 'CONTENTLANG' ],
 	'currentday'                => [ '1', 'TRENUTNIDAN', 'TEKUĆIDAN', 'TEKUĆI_DAN', 'CURRENTDAY' ],
-	'currentday2'               => [ '1', 'TEKUĆIDAN2', 'TEKUĆI_DAN_2', 'CURRENTDAY2' ],
-	'currentdayname'            => [ '1', 'TRENUTNIDANIME', 'IMETEKUĆEGDANA', 'IME_TEKUĆEG_DANA', 'CURRENTDAYNAME' ],
-	'currentdow'                => [ '1', 'TRENUTNIDOV', 'TEKUĆIDUN', 'CURRENTDOW' ],
-	'currenthour'               => [ '1', 'TEKUĆISAT', 'TEKUĆI_SAT', 'CURRENTHOUR' ],
-	'currentmonth'              => [ '1', 'TRENUTNIMESEC', 'TRENUTNI_MESEC', 'TEKUĆIMESEC', 'TEKUĆI_MESEC', 'CURRENTMONTH', 'CURRENTMONTH2' ],
-	'currentmonth1'             => [ '1', 'TRENUTNIMESEC1', 'TRENUTNI_MESEC1', 'TEKUĆIMESEC1', 'TEKUĆI_MESEC1', 'CURRENTMONTH1' ],
-	'currentmonthabbrev'        => [ '1', 'TRENUTNIMESECSKR', 'TEKUĆIMESECSKR', 'TEKUĆI_MESEC_SKR', 'CURRENTMONTHABBREV' ],
-	'currentmonthname'          => [ '1', 'TRENUTNIMESECIME', 'IMETEKUĆEGMESECA', 'IME_TEKUĆEG_MESECA', 'CURRENTMONTHNAME' ],
-	'currentmonthnamegen'       => [ '1', 'TRENUTNIMESECGEN', 'TEKUĆIMESECGEN', 'TEKUĆI_MESEC_GEN', 'CURRENTMONTHNAMEGEN' ],
-	'currenttime'               => [ '1', 'TRENUTNOVREME', 'TEKUĆEVREME', 'TEKUĆE_VREME', 'CURRENTTIME' ],
-	'currenttimestamp'          => [ '1', 'TEKUĆIOTISAKVREMENA', 'TEKUĆI_OTISAK_VREMENA', 'CURRENTTIMESTAMP' ],
-	'currentversion'            => [ '1', 'TEKUĆEIZDANJE', 'TEKUĆE_IZDANJE', 'CURRENTVERSION' ],
-	'currentweek'               => [ '1', 'TRENUTNANEDELJA', 'TRENUTNA_NEDELJA', 'TEKUĆANEDELJA', 'TEKUĆA_NEDELJA', 'CURRENTWEEK' ],
+	'currentday2'               => [ '1', 'TRENUTNIDAN2', 'TEKUĆIDAN2', 'TEKUĆI_DAN_2', 'CURRENTDAY2' ],
+	'currentdayname'            => [ '1', 'NAZIVTRENUTNOGDANA', 'TRENUTNIDANIME', 'IMETEKUĆEGDANA', 'IME_TEKUĆEG_DANA', 'CURRENTDAYNAME' ],
+	'currentdow'                => [ '1', 'TRENUTNIDUN', 'TRENUTNIDOV', 'TEKUĆIDUN', 'CURRENTDOW' ],
+	'currenthour'               => [ '1', 'TRENUTNISAT', 'TRENUTNIČAS', 'TEKUĆISAT', 'TEKUĆIČAS', 'TEKUĆI_SAT', 'TEKUĆI_ČAS', 'CURRENTHOUR' ],
+	'currentmonth'              => [ '1', 'TRENUTNIMESEC', 'TRENUTNIMJESEC', 'TRENUTNI_MESEC', 'TRENUTNI_MJESEC', 'TEKUĆIMESEC', 'TEKUĆIMJESEC', 'TEKUĆI_MESEC', 'TEKUĆI_MJESEC', 'CURRENTMONTH', 'CURRENTMONTH2' ],
+	'currentmonth1'             => [ '1', 'TRENUTNIMESEC1', 'TRENUTNIMJESEC1', 'TRENUTNI_MESEC1', 'TRENUTNI_MJESEC1', 'TEKUĆIMESEC1', 'TEKUĆIMJESEC1', 'TEKUĆI_MESEC1', 'TEKUĆI_MJESEC1', 'CURRENTMONTH1' ],
+	'currentmonthabbrev'        => [ '1', 'TRENUTNIMESECSKR', 'TRENUTNIMJESECSKR', 'TEKUĆIMESECSKR', 'TEKUĆIMJESECSKR', 'TEKUĆI_MESEC_SKR', 'TEKUĆI_MJESEC_SKR', 'CURRENTMONTHABBREV' ],
+	'currentmonthname'          => [ '1', 'NAZIVTRENUTNOGMESECA', 'NAZIVTRENUTNOGMJESECA', 'TRENUTNIMESECIME', 'TRENUTNIMJESECIME', 'IMETEKUĆEGMESECA', 'IMETEKUĆEGMJESECA', 'IME_TEKUĆEG_MESECA', 'IME_TEKUĆEG_MJESECA', 'CURRENTMONTHNAME' ],
+	'currentmonthnamegen'       => [ '1', 'TRENUTNIMESECGEN', 'TRENUTNIMJESECGEN', 'TEKUĆIMESECGEN', 'TEKUĆIMJESECGEN', 'TEKUĆI_MESEC_GEN', 'TEKUĆI_MJESEC_GEN', 'CURRENTMONTHNAMEGEN' ],
+	'currenttime'               => [ '1', 'TRENUTNOVREME', 'TRENUTNOVRIJEME', 'TEKUĆEVREME', 'TEKUĆEVRIJEME', 'TEKUĆE_VREME', 'TEKUĆE_VRIJEME', 'CURRENTTIME' ],
+	'currenttimestamp'          => [ '1', 'TRENUTNAVREMENSKAOZNAKA', 'TRENUTNIVREMENSKIPEČAT', 'TEKUĆIOTISAKVREMENA', 'TEKUĆI_OTISAK_VREMENA', 'CURRENTTIMESTAMP' ],
+	'currentversion'            => [ '1', 'TRENUTNAVERZIJA', 'TRENUTNOIZDANJE', 'TEKUĆAVERZIJA', 'TEKUĆEIZDANJE', 'TEKUĆE_IZDANJE', 'CURRENTVERSION' ],
+	'currentweek'               => [ '1', 'TRENUTNASEDMICA', 'TRENUTNANEDELJA', 'TRENUTNANEDJELJA', 'TRENUTNA_NEDELJA', 'TRENUTNA_NEDJELJA', 'TEKUĆANEDELJA', 'TEKUĆANEDJELJA', 'TEKUĆA_NEDELJA', 'TEKUĆA_NEDJELJA', 'CURRENTWEEK' ],
 	'currentyear'               => [ '1', 'TRENUTNAGODINA', 'TEKUĆAGODINA', 'TEKUĆA_GODINA', 'CURRENTYEAR' ],
-	'defaultsort'               => [ '1', 'PODRAZUMEVANOSORTIRANJE:', 'PODRAZUMEVANIKLJUČZASORTIRANJE:', 'PODRAZMEVANOSORTIRANJEKATEGORIJE:', 'DEFAULTSORT:' ],
-	'defaultsort_noerror'       => [ '0', 'bez_greške', 'noerror' ],
-	'defaultsort_noreplace'     => [ '0', 'bez_zamene', 'noreplace' ],
-	'directionmark'             => [ '1', 'SMEROZNAKE', 'SMER   _OZNAKE', 'DIRECTIONMARK', 'DIRMARK' ],
-	'displaytitle'              => [ '1', 'NAZIVPRIKAZA', 'NAZIV_PRIKAZA', 'DISPLAYTITLE' ],
-	'filepath'                  => [ '0', 'PUTANJADATOTEKE', 'PUTANJA_DATOTEKE', 'FILEPATH:' ],
-	'forcetoc'                  => [ '0', '__FORSIRANISADRŽAJ__', '__FORSIRANI_SADRŽAJ__', '__PRIMORANISADRŽAJ__', '__PRIMORANI_SADRŽAJ__', '__FORCETOC__' ],
+	'defaultsort'               => [ '1', 'PODRAZUMEVANOSORTIRANJE:', 'PODRAZUMIJEVANOSORTIRANJE:', 'PODRAZUMEVANIKLJUČZASORTIRANJE:', 'PODRAZUMIJEVANIKLJUČZASORTIRANJE:', 'PODRAZUMEVANOSORTIRANJEKATEGORIJE:', 'PODRAZUMIJEVANOSORTIRANJEKATEGORIJE:', 'PODRAZMEVANOSORTIRANJEKATEGORIJE:', 'PODRAZMIJEVANOSORTIRANJEKATEGORIJE:', 'DEFAULTSORT:', 'SORTIRANJE:' ],
+	'defaultsort_noerror'       => [ '0', 'bezgreške', 'bez_greške', 'noerror' ],
+	'defaultsort_noreplace'     => [ '0', 'bezzamene', 'bezzamjene', 'bez_zamene', 'bez_zamjene', 'noreplace' ],
+	'directionmark'             => [ '1', 'OZNAKAZASMER', 'OZNAKAZASMJER', 'SMEROZNAKE', 'SMJEROZNAKE', 'SMER   _OZNAKE', 'SMJER   _OZNAKE', 'SMER_OZNAKE', 'SMJER_OZNAKE', 'DIRECTIONMARK', 'DIRMARK' ],
+	'displaytitle'              => [ '1', 'NASLOVZAPRIKAZ', 'NAZIVZAPRIKAZ', 'NAZIVPRIKAZA', 'NAZIV_PRIKAZA', 'DISPLAYTITLE' ],
+	'filepath'                  => [ '0', 'PUTANJADODATOTEKE', 'PUTANJADATOTEKE', 'PUTANJA_DATOTEKE', 'FILEPATH:' ],
+	'forcetoc'                  => [ '0', '__FORSIRAJSADRŽAJ__', '__FORSIRANISADRŽAJ__', '__FORSIRANI_SADRŽAJ__', '__PRIMORANISADRŽAJ__', '__PRIMORANI_SADRŽAJ__', '__FORCETOC__' ],
 	'formatdate'                => [ '0', 'formatdatuma', 'format_datuma', 'formatdate', 'dateformat' ],
-	'fullpagename'              => [ '1', 'PUNOIMESTRANICE', 'PUNOIMESTRANE', 'PUNO_IME_STRANICE', 'PUNO_IME_STRANE', 'FULLPAGENAME' ],
-	'fullpagenamee'             => [ '1', 'PUNAIMENASTRANICA', 'PUNAIMENASTRANA', 'PUNA_IMENA_STRANICA', 'PUNA_IMENA_STRANA', 'FULLPAGENAMEE' ],
-	'fullurl'                   => [ '0', 'PUNURL:', 'CELAADRESA', 'CELA_ADRESA', 'FULLURL:' ],
-	'fullurle'                  => [ '0', 'PUNURLE:', 'CELEADRESE', 'CELE_ADRESE', 'FULLURLE:' ],
+	'fullpagename'              => [ '1', 'CEONAZIVSTRANICE', 'CIONAZIVSTRANICE', 'CELINAZIVSTRANICE', 'CIJELINAZIVSTRANICE', 'PUNOIMESTRANICE', 'PUNOIMESTRANE', 'PUNO_IME_STRANICE', 'PUNO_IME_STRANE', 'FULLPAGENAME' ],
+	'fullpagenamee'             => [ '1', 'CEONAZIVSTRANICEK', 'CIONAZIVSTRANICEK', 'CELINAZIVSTRANICEK', 'CIJELINAZIVSTRANICEK', 'PUNAIMENASTRANICA', 'PUNAIMENASTRANA', 'PUNA_IMENA_STRANICA', 'PUNA_IMENA_STRANA', 'FULLPAGENAMEE' ],
+	'fullurl'                   => [ '0', 'CELAADRESA:', 'CIJELAADRESA:', 'CEOURL:', 'CIOURL:', 'CELIURL:', 'CIJELIURL:', 'PUNURL:', 'CELAADRESA', 'CIJELAADRESA', 'CELA_ADRESA', 'CIJELA_ADRESA', 'FULLURL:' ],
+	'fullurle'                  => [ '0', 'CELAADRESAK:', 'CIJELAADRESAK:', 'CEOURLK:', 'CIOURLK:', 'CELIURLK:', 'CIJELIURLK:', 'PUNURLE:', 'CELEADRESE', 'CIJELEADRESE', 'CELE_ADRESE', 'CIJELE_ADRESE', 'FULLURLE:' ],
 	'gender'                    => [ '0', 'ROD:', 'LICE:', 'GENDER:' ],
 	'grammar'                   => [ '0', 'GRAMATIKA:', 'GRAMMAR:' ],
-	'hiddencat'                 => [ '1', '__SAKRIVENAKAT__', '__HIDDENCAT__' ],
+	'hiddencat'                 => [ '1', '__SAKRIVENAKAT__', '__SAKRIVENAKATEGORIJA__', '__SKRIVENAKAT__', '__SKRIVENAKATEGORIJA__', '__HIDDENCAT__' ],
 	'img_baseline'              => [ '1', 'osnova', 'baseline' ],
-	'img_border'                => [ '1', 'ivica', 'border' ],
+	'img_border'                => [ '1', 'ivica', 'oivičeno', 'border' ],
 	'img_bottom'                => [ '1', 'dno', 'bottom' ],
 	'img_center'                => [ '1', 'centar', 'c', 'center', 'centre' ],
 	'img_framed'                => [ '1', 'okvir', 'ram', 'framed', 'enframed', 'frame' ],
 	'img_frameless'             => [ '1', 'bezokvira', 'bez_okvira', 'bezrama', 'bez_rama', 'frameless' ],
-	'img_left'                  => [ '1', 'levo', 'l', 'left' ],
+	'img_left'                  => [ '1', 'levo', 'lijevo', 'l', 'left' ],
 	'img_link'                  => [ '1', 'veza=$1', 'link=$1' ],
 	'img_manualthumb'           => [ '1', 'mini=$1', 'umanjeno=$1', 'thumbnail=$1', 'thumb=$1' ],
 	'img_middle'                => [ '1', 'sredina', 'middle' ],
@@ -342,96 +348,100 @@ $magicWords = [
 	'img_page'                  => [ '1', 'stranica=$1', 'strana=$1', 'stranica_$1', 'strana_$1', 'page=$1', 'page $1' ],
 	'img_right'                 => [ '1', 'desno', 'd', 'right' ],
 	'img_sub'                   => [ '1', 'pod', 'sub' ],
+	'img_super'                 => [ '1', 'super', 'sup' ],
 	'img_text_bottom'           => [ '1', 'sredinateksta', 'sredina_teksta', 'text-bottom' ],
 	'img_text_top'              => [ '1', 'vrhteksta', 'vrh_teksta', 'text-top' ],
 	'img_thumbnail'             => [ '1', 'mini', 'umanjeno', 'thumbnail', 'thumb' ],
 	'img_top'                   => [ '1', 'vrh', 'top' ],
 	'img_upright'               => [ '1', 'uspravno', 'uspravno=$1', 'uspravno_$1', 'upright', 'upright=$1', 'upright $1' ],
-	'img_width'                 => [ '1', '$1piskel', '$1p', '$1px' ],
-	'index'                     => [ '1', '__INDEKS__', '__INDEX__' ],
-	'language'                  => [ '0', '#JEZIK:', '#LANGUAGE:' ],
+	'img_width'                 => [ '1', '$1p', '$1piksel', '$1px' ],
+	'index'                     => [ '1', '__INDEKSIRAJ__', '__INDEKSIRANJE__', '__INDEKS__', '__INDEX__' ],
+	'language'                  => [ '0', '#JEZIK', '#LANGUAGE' ],
 	'lcfirst'                   => [ '0', 'LCPRVI:', 'LCFIRST:' ],
 	'localday'                  => [ '1', 'LOKALNIDAN', 'LOKALNI_DAN', 'LOCALDAY' ],
 	'localday2'                 => [ '1', 'LOKALNIDAN2', 'LOKALNI_DAN2', 'LOCALDAY2' ],
-	'localdayname'              => [ '1', 'IMELOKALNOGDANA', 'IME_LOKALNOG_DANA', 'LOCALDAYNAME' ],
+	'localdayname'              => [ '1', 'NAZIVLOKALNOGDANA', 'IMELOKALNOGDANA', 'IME_LOKALNOG_DANA', 'LOCALDAYNAME' ],
 	'localdow'                  => [ '1', 'LOKALNIDUN', 'LOCALDOW' ],
-	'localhour'                 => [ '1', 'LOKALNISAT', 'LOKALNI_SAT', 'LOCALHOUR' ],
-	'localmonth'                => [ '1', 'LOKALNIMESEC', 'LOKALNI_MESEC', 'LOCALMONTH', 'LOCALMONTH2' ],
-	'localmonth1'               => [ '1', 'LOKALNIMESEC2', 'LOKALNI_MESEC2', 'LOCALMONTH1' ],
-	'localmonthabbrev'          => [ '1', 'LOKALNIMESECSKR', 'LOKALNI_MESEC_SKR', 'LOCALMONTHABBREV' ],
-	'localmonthname'            => [ '1', 'IMELOKALNOGMESECA', 'IME_LOKALNOG_MESECA', 'LOCALMONTHNAME' ],
-	'localmonthnamegen'         => [ '1', 'LOKALNIMESECGEN', 'LOKALNI_MESEC_GEN', 'LOCALMONTHNAMEGEN' ],
-	'localtime'                 => [ '1', 'LOKALNOVREME', 'LOKALNO_VREME', 'LOCALTIME' ],
-	'localtimestamp'            => [ '1', 'OTISAKVREMENA', 'OTISAK_VREMENA', 'LOCALTIMESTAMP' ],
-	'localurl'                  => [ '0', 'LOKALNAADRESA:', 'LOKALNA_ADRESA:', 'LOCALURL:' ],
-	'localurle'                 => [ '0', 'LOKALNEADRESE:', 'LOKALNE_ADRESE:', 'LOCALURLE:' ],
-	'localweek'                 => [ '1', 'LOKALNANEDELJA', 'LOKALNA_NEDELJA', 'LOCALWEEK' ],
+	'localhour'                 => [ '1', 'LOKALNISAT', 'LOKALNIČAS', 'LOKALNI_SAT', 'LOKALNI_ČAS', 'LOCALHOUR' ],
+	'localmonth'                => [ '1', 'LOKALNIMESEC', 'LOKALNIMJESEC', 'LOKALNI_MESEC', 'LOKALNI_MJESEC', 'LOCALMONTH', 'LOCALMONTH2' ],
+	'localmonth1'               => [ '1', 'LOKALNIMESEC2', 'LOKALNIMJESEC2', 'LOKALNI_MESEC2', 'LOKALNI_MJESEC2', 'LOCALMONTH1' ],
+	'localmonthabbrev'          => [ '1', 'LOKALNIMESECSKR', 'LOKALNIMJESECSKR', 'LOKALNI_MESEC_SKR', 'LOKALNI_MJESEC_SKR', 'LOCALMONTHABBREV' ],
+	'localmonthname'            => [ '1', 'NAZIVLOKALNOGMESECA', 'NAZIVLOKALNOGMJESECA', 'IMELOKALNOGMESECA', 'IMELOKALNOGMJESECA', 'IME_LOKALNOG_MESECA', 'IME_LOKALNOG_MJESECA', 'LOCALMONTHNAME' ],
+	'localmonthnamegen'         => [ '1', 'LOKALNIMESECGEN', 'LOKALNIMJESECGEN', 'LOKALNI_MESEC_GEN', 'LOKALNI_MJESEC_GEN', 'LOCALMONTHNAMEGEN' ],
+	'localtime'                 => [ '1', 'LOKALNOVREME', 'LOKALNOVRIJEME', 'LOKALNO_VREME', 'LOKALNO_VRIJEME', 'LOCALTIME' ],
+	'localtimestamp'            => [ '1', 'LOKALNAVREMENSKAOZNAKA', 'LOKALNIVREMENSKIPEČAT', 'OTISAKVREMENA', 'OTISAK_VREMENA', 'LOCALTIMESTAMP' ],
+	'localurl'                  => [ '0', 'LOKALNAADRESA:', 'LOKALNIURL:', 'LOKALNA_ADRESA:', 'LOCALURL:' ],
+	'localurle'                 => [ '0', 'LOKALNAADRESAK:', 'LOKALNIURLK:', 'LOKALNEADRESE:', 'LOKALNE_ADRESE:', 'LOCALURLE:' ],
+	'localweek'                 => [ '1', 'LOKALNASEDMICA', 'LOKALNANEDELJA', 'LOKALNANEDJELJA', 'LOKALNA_NEDELJA', 'LOKALNA_NEDJELJA', 'LOCALWEEK' ],
 	'localyear'                 => [ '1', 'LOKALNAGODINA', 'LOKALNA_GODINA', 'LOCALYEAR' ],
 	'msg'                       => [ '0', 'POR:', 'MSG:' ],
 	'msgnw'                     => [ '0', 'NVPOR:', 'MSGNW:' ],
 	'namespace'                 => [ '1', 'IMENSKIPROSTOR', 'IMENSKI_PROSTOR', 'NAMESPACE' ],
-	'namespacee'                => [ '1', 'IMENSKIPROSTORI', 'IMENSKI_PROSTORI', 'NAMESPACEE' ],
+	'namespacee'                => [ '1', 'IMENSKIPROSTORK', 'IMENSKIPROSTORI', 'IMENSKI_PROSTORI', 'NAMESPACEE' ],
 	'namespacenumber'           => [ '1', 'BROJIMENSKOGPROSTORA', 'NAMESPACENUMBER' ],
-	'newsectionlink'            => [ '1', '__NOVAVEZAODELJKA__', '__NOVA_VEZA_ODELJKA__', '__NEWSECTIONLINK__' ],
-	'nocontentconvert'          => [ '0', '__BEZCC__', '__NOCONTENTCONVERT__', '__NOCC__' ],
+	'newsectionlink'            => [ '1', '__LINKZANOVIODELJAK__', '__LINKZANOVIODJELJAK__', '__VEZAZANOVIODELJAK__', '__VEZAZANOVIODJELJAK__', '__NOVAVEZAODELJKA__', '__NOVAVEZAODJELJKA__', '__NOVA_VEZA_ODELJKA__', '__NOVA_VEZA_ODJELJKA__', '__NEWSECTIONLINK__' ],
+	'nocontentconvert'          => [ '0', '__BEZKS__', '__BEZCC__', '__NOCONTENTCONVERT__', '__NOCC__' ],
 	'noeditsection'             => [ '0', '__BEZIZMENA__', '__BEZ_IZMENA__', '__BEZIZMJENA__', '__BEZ_IZMJENA__', '__NOEDITSECTION__' ],
 	'nogallery'                 => [ '0', '__BEZGALERIJE__', '__BEZ_GALERIJE__', '__NOGALLERY__' ],
-	'noindex'                   => [ '1', '__BEZINDEKSA__', '__BEZ_INDEKSA__', '__NOINDEX__' ],
-	'nonewsectionlink'          => [ '1', '__BEZNOVEVEZEODELJKA__', '__BEZ_NOVE_VEZE_ODELJKA__', '__NONEWSECTIONLINK__' ],
+	'noindex'                   => [ '1', '__NEINDEKSIRAJ__', '__BEZINDEKSIRANJA__', '__BEZINDEKSA__', '__BEZ_INDEKSA__', '__NOINDEX__' ],
+	'nonewsectionlink'          => [ '1', '__BEZLINKAZANOVIODELJAK__', '__BEZLINKAZANOVIODJELJAK__', '__BEZVEZEZANOVIODELJAK__', '__BEZVEZEZANOVIODJELJAK__', '__BEZNOVEVEZEODELJKA__', '__BEZNOVEVEZEODJELJKA__', '__BEZ_NOVE_VEZE_ODELJKA__', '__BEZ_NOVE_VEZE_ODJELJKA__', '__NONEWSECTIONLINK__' ],
 	'notitleconvert'            => [ '0', '__BEZKN__', '__NOTITLECONVERT__', '__NOTC__' ],
 	'notoc'                     => [ '0', '__BEZSADRŽAJA__', '__BEZ_SADRŽAJA__', '__NOTOC__' ],
 	'ns'                        => [ '0', 'IP:', 'NS:' ],
 	'numberingroup'             => [ '1', 'BROJUGRUPI', 'BROJ_U_GRUPI', 'NUMBERINGROUP', 'NUMINGROUP' ],
 	'numberofactiveusers'       => [ '1', 'BROJAKTIVNIHKORISNIKA', 'BROJ_AKTIVNIH_KORISNIKA', 'NUMBEROFACTIVEUSERS' ],
-	'numberofadmins'            => [ '1', 'BROJADMINA', 'BROJ_ADMINA', 'NUMBEROFADMINS' ],
+	'numberofadmins'            => [ '1', 'BROJADMINA', 'BROJADMINISTRATORA', 'BROJ_ADMINA', 'NUMBEROFADMINS' ],
 	'numberofarticles'          => [ '1', 'BROJČLANAKA', 'BROJ_ČLANAKA', 'NUMBEROFARTICLES' ],
-	'numberofedits'             => [ '1', 'BROJIZMENA', 'BROJ_IZMENA', 'NUMBEROFEDITS' ],
+	'numberofedits'             => [ '1', 'BROJIZMENA', 'BROJIZMJENA', 'BROJ_IZMENA', 'BROJ_IZMJENA', 'NUMBEROFEDITS' ],
 	'numberoffiles'             => [ '1', 'BROJDATOTEKA', 'BROJ_DATOTEKA', 'BROJFAJLOVA', 'BROJ_FAJLOVA', 'NUMBEROFFILES' ],
 	'numberofpages'             => [ '1', 'BROJSTRANICA', 'BROJ_STRANICA', 'NUMBEROFPAGES' ],
 	'numberofusers'             => [ '1', 'BROJKORISNIKA', 'BROJ_KORISNIKA', 'NUMBEROFUSERS' ],
-	'padleft'                   => [ '0', 'PORAVNAJULEVO', 'PORAVNAJ_ULEVO', 'PADLEFT' ],
-	'padright'                  => [ '0', 'PORAVNAJUDESNO', 'PORAVNAJ_UDESNO', 'PADRIGHT' ],
-	'pagename'                  => [ '1', 'IMESTRANICE', 'IME_STRANICE', 'STRANICA', 'PAGENAME' ],
-	'pagenamee'                 => [ '1', 'IMENASTRANICA', 'IMENA_STRANICA', 'STRANICE', 'PAGENAMEE' ],
+	'padleft'                   => [ '0', 'POPUNILEVO', 'POPUNILIJEVO', 'POPUNALEVO', 'POPUNALIJEVO', 'PORAVNAJLEVO', 'PORAVNAJLIJEVO', 'PORAVNAJ_LEVO', 'PORAVNAJ_LIJEVO', 'PORAVNAJULEVO', 'PORAVNAJ_ULEVO', 'PORAVNAJULIJEVO', 'PORAVNAJ_ULIJEVO', 'PADLEFT' ],
+	'padright'                  => [ '0', 'POPUNIDESNO', 'POPUNADESNO', 'PORAVNAJDESNO', 'PORAVNAJ_DESNO', 'PORAVNAJUDESNO', 'PORAVNAJ_UDESNO', 'PADRIGHT' ],
+	'pageid'                    => [ '0', 'IDSTRANICE', 'BROJSTRANICE', 'PAGEID' ],
+	'pagename'                  => [ '1', 'NAZIVSTRANICE', 'IMESTRANICE', 'IME_STRANICE', 'STRANICA', 'PAGENAME' ],
+	'pagenamee'                 => [ '1', 'NAZIVSTRANICEK', 'IMENASTRANICA', 'IMENA_STRANICA', 'STRANICE', 'PAGENAMEE' ],
 	'pagesincategory'           => [ '1', 'STRANICAUKATEGORIJI', 'STRANAUKATEGORIJI', 'STRANICA_U_KATEGORIJI', 'STRANA_U_KATEGORIJI', 'PAGESINCATEGORY', 'PAGESINCAT' ],
+	'pagesincategory_all'       => [ '0', 'sve', 'all' ],
+	'pagesincategory_files'     => [ '0', 'datoteke', 'files' ],
 	'pagesinnamespace'          => [ '1', 'STRANICAUIMENSKOMPROSTORU', 'STRANAUIMENSKOMPROSTORU', 'STRANICA_U_IMENSKOM_PROSTORU', 'STRANA_U_IMENSKOM_PROSTORU', 'PAGESINNAMESPACE:', 'PAGESINNS:' ],
 	'pagesize'                  => [ '1', 'VELIČINASTRANICE', 'VELIČINASTRANE', 'VELIČINA_STRANICE', 'VELIČINA_STRANE', 'PAGESIZE' ],
 	'plural'                    => [ '0', 'MNOŽINA:', 'PLURAL:' ],
-	'protectionlevel'           => [ '1', 'NIVOZAŠTITE', 'NIVO_ZAŠTITE', 'PROTECTIONLEVEL' ],
+	'protectionlevel'           => [ '1', 'STEPENZAŠTITE', 'NIVOZAŠTITE', 'NIVO_ZAŠTITE', 'PROTECTIONLEVEL' ],
 	'raw'                       => [ '0', 'ČIST:', 'RAW:' ],
 	'redirect'                  => [ '0', '#Preusmeri', '#preusmeri', '#PREUSMERI', '#Preusmjeri', '#preusmjeri', '#PREUSMJERI', '#redirect', '#REDIRECT' ],
-	'revisionday'               => [ '1', 'DANIZMENE', 'DAN_IZMENE', 'REVISIONDAY' ],
-	'revisionday2'              => [ '1', 'DANIZMENE2', 'DAN_IZMENE2', 'REVISIONDAY2' ],
-	'revisionid'                => [ '1', 'IDREVIZIJE', 'ID_REVIZIJE', 'IB_IZMENE', 'REVISIONID' ],
-	'revisionmonth'             => [ '1', 'MESECIZMENE', 'MESEC_IZMENE', 'REVISIONMONTH' ],
-	'revisionmonth1'            => [ '1', 'MESECIZMENE1', 'MESEC_IZMENE1', 'REVISIONMONTH1' ],
-	'revisiontimestamp'         => [ '1', 'VREMEIZMENE', 'VREME_IZMENE', 'REVISIONTIMESTAMP' ],
-	'revisionuser'              => [ '1', 'KORISNIKIZMENE', 'KORISNIK_IZMENE', 'REVISIONUSER' ],
-	'revisionyear'              => [ '1', 'GODINAIZMENE', 'GODINA_IZMENE', 'REVISIONYEAR' ],
-	'safesubst'                 => [ '0', 'BEZBEDNAZAMENA', 'BEZBEDNA_ZAMENA', 'SAFESUBST:' ],
-	'scriptpath'                => [ '0', 'SKRIPTA', 'SKRIPT', 'SCRIPTPATH' ],
-	'servername'                => [ '0', 'IMESERVERA', 'IME_SERVERA', 'SERVERNAME' ],
-	'sitename'                  => [ '1', 'IMESAJTA', 'SITENAME' ],
+	'revisionday'               => [ '1', 'DANIZMENE', 'DANIZMJENE', 'DAN_IZMENE', 'DAN_IZMJENE', 'REVISIONDAY' ],
+	'revisionday2'              => [ '1', 'DANIZMENE2', 'DANIZMJENE2', 'DAN_IZMENE2', 'DAN_IZMJENE2', 'REVISIONDAY2' ],
+	'revisionid'                => [ '1', 'IDIZMENE', 'IDIZMJENE', 'IDREVIZIJE', 'ID_REVIZIJE', 'IB_IZMENE', 'IB_IZMJENE', 'REVISIONID' ],
+	'revisionmonth'             => [ '1', 'MESECIZMENE', 'MJESECIZMJENE', 'MESEC_IZMENE', 'MJESEC_IZMJENE', 'REVISIONMONTH' ],
+	'revisionmonth1'            => [ '1', 'MESECIZMENE1', 'MJESECIZMJENE1', 'MESEC_IZMENE1', 'MJESEC_IZMJENE1', 'REVISIONMONTH1' ],
+	'revisiontimestamp'         => [ '1', 'VREMENSKAOZNAKAIZMENE', 'VREMENSKAOZNAKAIZMJENE', 'VREMENSKIPEČATIZMENE', 'VREMENSKIPEČATIZMJENE', 'VREMEIZMENE', 'VRIJEMEIZMJENE', 'VREME_IZMENE', 'VRIJEME_IZMJENE', 'REVISIONTIMESTAMP' ],
+	'revisionuser'              => [ '1', 'KORISNIKIZMENE', 'KORISNIKIZMJENE', 'KORISNIK_IZMENE', 'KORISNIK_IZMJENE', 'REVISIONUSER' ],
+	'revisionyear'              => [ '1', 'GODINAIZMENE', 'GODINAIZMJENE', 'GODINA_IZMENE', 'GODINA_IZMJENE', 'REVISIONYEAR' ],
+	'safesubst'                 => [ '0', 'BEZBEDNAZAMENA', 'BEZBJEDNAZAMJENA', 'BEZBEDNA_ZAMENA', 'BEZBJEDNA_ZAMJENA', 'SAFESUBST:' ],
+	'scriptpath'                => [ '0', 'SKRIPTNAPUTANJA', 'SKRIPTA', 'SKRIPT', 'SCRIPTPATH' ],
+	'servername'                => [ '0', 'NAZIVSERVERA', 'IMESERVERA', 'IME_SERVERA', 'SERVERNAME' ],
+	'sitename'                  => [ '1', 'NAZIVSAJTA', 'IMESAJTA', 'SITENAME' ],
 	'special'                   => [ '0', 'posebno', 'special' ],
-	'staticredirect'            => [ '1', '__STATIČKOPREUSMERENJE__', 'STATIČKO_PREUSMERENJE', '__STATICREDIRECT__' ],
-	'stylepath'                 => [ '0', 'PUTANJASTILA', 'PUTANJA_STILA', 'STYLEPATH' ],
-	'subjectpagename'           => [ '1', 'IMEČLANKA', 'IME_ČLANKA', 'SUBJECTPAGENAME', 'ARTICLEPAGENAME' ],
-	'subjectpagenamee'          => [ '1', 'IMENAČLANAKA', 'IMENA_ČLANAKA', 'SUBJECTPAGENAMEE', 'ARTICLEPAGENAMEE' ],
-	'subjectspace'              => [ '1', 'IMENSKIPROSTORČLANKA', 'IMENSKI_PROSTOR_ČLANKA', 'SUBJECTSPACE', 'ARTICLESPACE' ],
-	'subjectspacee'             => [ '1', 'IMENSKIPROSTORČLANAKA', 'IMENSKI_PROSTOR_ČLANAKA', 'SUBJECTSPACEE', 'ARTICLESPACEE' ],
-	'subpagename'               => [ '1', 'IMEPODSTRANICE', 'IMEPODSTRANE', 'IME_PODSTRANICE', 'IME_PODSTRANE', 'SUBPAGENAME' ],
-	'subpagenamee'              => [ '1', 'IMENAPODSTRANICA', 'IMENAPODSTRANA', 'IMENA_PODSTRANICA', 'IMENA_PODSTRANA', 'SUBPAGENAMEE' ],
-	'subst'                     => [ '0', 'ZAMENI:', 'ZAMENA:', 'SUBST:' ],
+	'staticredirect'            => [ '1', '__STATIČNOPREUSMERENJE__', '__STATIČNOPREUSMJERENJE__', '__STATIČKOPREUSMERENJE__', '__STATIČKOPREUSMJERENJE__', 'STATIČKO_PREUSMERENJE', 'STATIČKO_PREUSMJERENJE', '__STATICREDIRECT__' ],
+	'stylepath'                 => [ '0', 'STILSKAPUTANJA', 'PUTANJASTILA', 'PUTANJA_STILA', 'STYLEPATH' ],
+	'subjectpagename'           => [ '1', 'NAZIVČLANKA', 'IMEČLANKA', 'IME_ČLANKA', 'SUBJECTPAGENAME', 'ARTICLEPAGENAME' ],
+	'subjectpagenamee'          => [ '1', 'NAZIVČLANKAK', 'IMENAČLANAKA', 'IMENA_ČLANAKA', 'SUBJECTPAGENAMEE', 'ARTICLEPAGENAMEE' ],
+	'subjectspace'              => [ '1', 'PROSTORČLANAKA', 'IMENSKIPROSTORČLANKA', 'IMENSKI_PROSTOR_ČLANKA', 'SUBJECTSPACE', 'ARTICLESPACE' ],
+	'subjectspacee'             => [ '1', 'PROSTORČLANAKAK', 'IMENSKIPROSTORČLANAKA', 'IMENSKI_PROSTOR_ČLANAKA', 'SUBJECTSPACEE', 'ARTICLESPACEE' ],
+	'subpagename'               => [ '1', 'NAZIVPODSTRANICE', 'IMEPODSTRANICE', 'IMEPODSTRANE', 'IME_PODSTRANICE', 'IME_PODSTRANE', 'SUBPAGENAME' ],
+	'subpagenamee'              => [ '1', 'NAZIVPODSTRANICEK', 'IMENAPODSTRANICA', 'IMENAPODSTRANA', 'IMENA_PODSTRANICA', 'IMENA_PODSTRANA', 'SUBPAGENAMEE' ],
+	'subst'                     => [ '0', 'ZAMENI:', 'ZAMIJENI:', 'ZAMENA:', 'ZAMJENA:', 'SUBST', 'SUBST:' ],
 	'tag'                       => [ '0', 'oznaka', 'tag' ],
-	'talkpagename'              => [ '1', 'IMERAZGOVORA', 'IME_RAZGOVORA', 'TALKPAGENAME' ],
-	'talkpagenamee'             => [ '1', 'IMENARAZGOVORA', 'IMENA_RAZGOVORA', 'TALKPAGENAMEE' ],
-	'talkspace'                 => [ '1', 'RAZGOVOR', 'TALKSPACE' ],
-	'talkspacee'                => [ '1', 'RAZGOVORI', 'TALKSPACEE' ],
+	'talkpagename'              => [ '1', 'NAZIVSTRANICEZARAZGOVOR', 'IMERAZGOVORA', 'IME_RAZGOVORA', 'TALKPAGENAME' ],
+	'talkpagenamee'             => [ '1', 'NAZIVSTRANICEZARAZGOVORK', 'IMENARAZGOVORA', 'IMENA_RAZGOVORA', 'TALKPAGENAMEE' ],
+	'talkspace'                 => [ '1', 'PROSTORRAZGOVORA', 'RAZGOVOR', 'TALKSPACE' ],
+	'talkspacee'                => [ '1', 'PROSTORRAZGOVORAK', 'RAZGOVORK', 'RAZGOVORI', 'TALKSPACEE' ],
 	'toc'                       => [ '0', '__SADRŽAJ__', '__TOC__' ],
 	'ucfirst'                   => [ '0', 'UCPRVI:', 'UCFIRST:' ],
-	'urlencode'                 => [ '0', 'KODIRANJEADRESE', 'KODIRANJE_ADRESE', 'URLENCODE:' ],
+	'urlencode'                 => [ '0', 'KODIRANJEADRESE', 'KODIRANJEURLA', 'KODIRANJEURL-A', 'KODIRANJE_ADRESE', 'URLENCODE:' ],
 	'url_path'                  => [ '0', 'PUTANJA', 'PATH' ],
-	'url_query'                 => [ '0', 'REDOSLED', 'QUERY' ],
+	'url_query'                 => [ '0', 'UPIT', 'REDOSLED', 'REDOSLIJED', 'REDOSLJED', 'QUERY' ],
 	'url_wiki'                  => [ '0', 'VIKI', 'WIKI' ],
 ];
 
