@@ -1,15 +1,29 @@
 <?php
 
+use MediaWiki\ChangeTags\ChangeTags;
+use MediaWiki\ChangeTags\ChangeTagsStore;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\Deferred\DeferredUpdates;
 use MediaWiki\Language\RawMessage;
 use MediaWiki\MainConfigNames;
+use MediaWiki\MediaWikiServices;
 use Wikimedia\Rdbms\Platform\ISQLPlatform;
 
 /**
- * @covers \ChangeTags
+ * @covers \MediaWiki\ChangeTags\ChangeTagsStore
+ * @covers \MediaWiki\ChangeTags\ChangeTags
  * @group Database
  */
 class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 
+	private ChangeTagsStore $changeTags;
+
+	protected function setUp(): void {
+		parent::setUp();
+
+		$this->changeTags = $this->getServiceContainer()->getChangeTagsStore();
+	}
+
 	protected function tearDown(): void {
 		parent::tearDown();
 	}
@@ -28,6 +42,97 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 
 	// TODO most methods are not tested
 
+	public function testBuildTagFilterSelector_allTags() {
+		// Set `activeOnly` to false
+		// Expect that at least all the software defined tags are returned
+		$allTags = MediaWikiServices::getInstance()->getChangeTagsStore()->listDefinedTags();
+		$allTagsList = ChangeTags::getChangeTagListSummary(
+			RequestContext::getMain(),
+			RequestContext::getMain()->getLanguage(),
+			ChangeTags::TAG_SET_ALL
+		);
+		$this->assertTrue(
+			count( $allTagsList ) >= count( $allTags ),
+			'`activeOnly` is false, expect all software tags'
+		);
+	}
+
+	public function testBuildTagFilterSelector_allSoftwareTags() {
+		// Set both `activeOnly` and `useAllTags` to false
+		// Expect that only software defined tags are returned
+		$allSoftwareTags = MediaWikiServices::getInstance()->getChangeTagsStore()->getSoftwareTags( true );
+		$allSoftwareTagsList = ChangeTags::getChangeTagListSummary(
+			RequestContext::getMain(),
+			RequestContext::getMain()->getLanguage(),
+			ChangeTags::TAG_SET_ALL,
+			ChangeTags::USE_SOFTWARE_TAGS_ONLY
+		);
+		$this->assertTrue(
+			count( $allSoftwareTagsList ) == count( $allSoftwareTags ),
+			'`activeOnly` and `useAllTags` are false, expect only software tags'
+		);
+	}
+
+	public function testBuildTagFilterSelector_activeOnlyNoHits() {
+		// Enable and test `activeOnly` and expect no tags returned,
+		// as there are currently no tagged edits in the test database
+		$emptyTagListSummary = ChangeTags::getChangeTagListSummary(
+			RequestContext::getMain(),
+			RequestContext::getMain()->getLanguage(),
+			ChangeTags::TAG_SET_ACTIVE_ONLY
+		);
+		$this->assertCount( 0, $emptyTagListSummary, '`activeOnly` is true and no hits, expect no tags' );
+
+		// Assert that by default, an empty select is returned, as no tags have been used yet
+		$this->assertEquals(
+			[
+				'<label for="tagfilter"><a href="/wiki/Special:Tags" title="Special:Tags">Tag</a> filter:</label>',
+				'<input class="mw-tagfilter-input mw-ui-input mw-ui-input-inline" size="20" id="tagfilter" list="tagfilter-datalist" name="tagfilter"><datalist id="tagfilter-datalist"></datalist>'
+			],
+			ChangeTags::buildTagFilterSelector(
+				'', false, RequestContext::getMain()
+			)
+		);
+	}
+
+	public function testBuildTagFilterSelector_activeOnly() {
+		// Disable patrolling so reverts will happen without approval
+		$this->overrideConfigValues( [ MainConfigNames::UseRCPatrol => false ] );
+
+		// Make an edit and replace the content, adding the `mw-replace` tag to the revision
+		$page = $this->getExistingTestPage();
+		$this->editPage( $page, '1' );
+		$this->editPage(
+			$page, '0', '', NS_MAIN, $this->getTestUser()->getUser()
+		);
+
+		// Ensure all deferred updates are run
+		DeferredUpdates::doUpdates();
+
+		// Assert that only the `mw-replace` tag is returned
+		$replaceOnlyTagList = ChangeTags::getChangeTagListSummary(
+			RequestContext::getMain(),
+			RequestContext::getMain()->getLanguage()
+		);
+		$this->assertCount( 1, $replaceOnlyTagList, '`activeOnly` is true with 1 hit, return 1 tag' );
+		$this->assertEquals(
+			'mw-replace', $replaceOnlyTagList[0]['name'],
+			'`activeOnly` is true with 1 hit, return expected tag'
+		);
+
+		// Assert that the tag is reflected in the default markup returned
+		$this->assertEquals(
+			[
+				'<label for="tagfilter"><a href="/wiki/Special:Tags" title="Special:Tags">Tag</a> filter:</label>',
+				'<input class="mw-tagfilter-input mw-ui-input mw-ui-input-inline" size="20" id="tagfilter" list="tagfilter-datalist" name="tagfilter"><datalist id="tagfilter-datalist"><option value="mw-replace">Replaced</option></datalist>'
+			],
+			ChangeTags::buildTagFilterSelector(
+				'', false, RequestContext::getMain()
+			),
+			'`activeOnly` is true with 1 hit, return expected tag markup'
+		);
+	}
+
 	/** @dataProvider provideModifyDisplayQuery */
 	public function testModifyDisplayQuery(
 		$origQuery,
@@ -38,6 +143,9 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$exclude = false
 	) {
 		$this->overrideConfigValue( MainConfigNames::UseTagFilter, $useTags );
+		// Reset the ChangeTagsStore after the config change
+		$this->changeTags = $this->getServiceContainer()->getChangeTagsStore();
+
 		if (
 			$avoidReopeningTables &&
 			$this->getDb()->getType() === 'mysql' &&
@@ -47,7 +155,7 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		}
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'foo', 'bar', '0' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'foo', 'bar', '0' ], [], $rcId );
 		// HACK resolve deferred group concats (see comment in provideModifyDisplayQuery)
 		if ( isset( $modifiedQuery['fields']['ts_tags'] ) ) {
 			$modifiedQuery['fields']['ts_tags'] = $this->getDb()
@@ -56,7 +164,7 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		if ( isset( $modifiedQuery['exception'] ) ) {
 			$this->expectException( $modifiedQuery['exception'] );
 		}
-		ChangeTags::modifyDisplayQuery(
+		$this->changeTags->modifyDisplayQuery(
 			$origQuery['tables'],
 			$origQuery['fields'],
 			$origQuery['conds'],
@@ -409,7 +517,7 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		];
 	}
 
-	public static function dataGetSoftwareTags() {
+	public static function provideGetSoftwareTags() {
 		return [
 			[
 				[
@@ -459,13 +567,15 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @dataProvider dataGetSoftwareTags
-	 * @covers \ChangeTags::getSoftwareTags
+	 * @dataProvider provideGetSoftwareTags
+	 * @covers \MediaWiki\ChangeTags\ChangeTagsStore::getSoftwareTags
 	 */
 	public function testGetSoftwareTags( $softwareTags, $expected ) {
 		$this->overrideConfigValue( MainConfigNames::SoftwareTags, $softwareTags );
+		// Reset the ChangeTagsStore after the config change
+		$this->changeTags = $this->getServiceContainer()->getChangeTagsStore();
 
-		$actual = ChangeTags::getSoftwareTags();
+		$actual = $this->changeTags->getSoftwareTags();
 		// Order of tags in arrays is not important
 		sort( $expected );
 		sort( $actual );
@@ -477,7 +587,7 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 
 		$rcId = 123;
 		$revId = 341;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId, $revId );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId, $revId );
 
 		$this->newSelectQueryBuilder()
 			->select( [ 'ctd_name', 'ctd_id', 'ctd_count' ] )
@@ -498,8 +608,8 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 
 		$rcId = 124;
 		$revId = 342;
-		ChangeTags::updateTags( [ 'tag1' ], [], $rcId, $revId );
-		ChangeTags::updateTags( [ 'tag3' ], [], $rcId, $revId );
+		$this->changeTags->updateTags( [ 'tag1' ], [], $rcId, $revId );
+		$this->changeTags->updateTags( [ 'tag3' ], [], $rcId, $revId );
 
 		$this->newSelectQueryBuilder()
 			->select( [ 'ctd_name', 'ctd_id', 'ctd_count' ] )
@@ -526,8 +636,8 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$this->emptyChangeTagsTables();
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId );
-		ChangeTags::updateTags( [ 'tag2', 'tag3' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'tag2', 'tag3' ], [], $rcId );
 
 		$this->newSelectQueryBuilder()
 			->select( [ 'ctd_name', 'ctd_id', 'ctd_count' ] )
@@ -553,8 +663,8 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$this->emptyChangeTagsTables();
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId );
-		$res = ChangeTags::updateTags( [ 'tag2', 'tag1' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId );
+		$res = $this->changeTags->updateTags( [ 'tag2', 'tag1' ], [], $rcId );
 		$this->assertEquals( [ [], [], [ 'tag1', 'tag2' ] ], $res );
 
 		$this->newSelectQueryBuilder()
@@ -580,8 +690,8 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$this->getServiceContainer()->resetServiceForTesting( 'NameTableStoreFactory' );
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId );
-		ChangeTags::updateTags( [], [ 'tag2' ], $rcId );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId );
+		$this->changeTags->updateTags( [], [ 'tag2' ], $rcId );
 
 		$this->newSelectQueryBuilder()
 			->select( [ 'ctd_name', 'ctd_id', 'ctd_count' ] )
@@ -617,9 +727,9 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 	 * @dataProvider provideTags
 	 */
 	public function testGetTags( array $tags, $rcId, $revId, $logId ) {
-		ChangeTags::addTags( $tags, $rcId, $revId, $logId );
+		$this->changeTags->addTags( $tags, $rcId, $revId, $logId );
 
-		$actualTags = ChangeTags::getTags( $this->getDb(), $rcId, $revId, $logId );
+		$actualTags = $this->changeTags->getTags( $this->getDb(), $rcId, $revId, $logId );
 
 		$this->assertSame( $tags, $actualTags );
 	}
@@ -629,33 +739,33 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$revId = 456;
 		$logId = 789;
 
-		ChangeTags::addTags( [ 'tag 1' ], $rcId );
-		ChangeTags::addTags( [ 'tag 2' ], $rcId, $revId );
-		ChangeTags::addTags( [ 'tag 3' ], $rcId, $revId, $logId );
+		$this->changeTags->addTags( [ 'tag 1' ], $rcId );
+		$this->changeTags->addTags( [ 'tag 2' ], $rcId, $revId );
+		$this->changeTags->addTags( [ 'tag 3' ], $rcId, $revId, $logId );
 
 		$tags3 = [ 'tag 3' ];
 		$tags2 = array_merge( $tags3, [ 'tag 2' ] );
 		$tags1 = array_merge( $tags2, [ 'tag 1' ] );
-		$this->assertArrayEquals( $tags3, ChangeTags::getTags( $this->getDb(), $rcId, $revId, $logId ) );
-		$this->assertArrayEquals( $tags2, ChangeTags::getTags( $this->getDb(), $rcId, $revId ) );
-		$this->assertArrayEquals( $tags1, ChangeTags::getTags( $this->getDb(), $rcId ) );
+		$this->assertArrayEquals( $tags3, $this->changeTags->getTags( $this->getDb(), $rcId, $revId, $logId ) );
+		$this->assertArrayEquals( $tags2, $this->changeTags->getTags( $this->getDb(), $rcId, $revId ) );
+		$this->assertArrayEquals( $tags1, $this->changeTags->getTags( $this->getDb(), $rcId ) );
 	}
 
 	public function testGetTagsWithData() {
 		$rcId1 = 123;
 		$rcId2 = 456;
 		$rcId3 = 789;
-		ChangeTags::addTags( [ 'tag 1' ], $rcId1, null, null, 'data1' );
-		ChangeTags::addTags( [ 'tag 3_1' ], $rcId3, null, null );
-		ChangeTags::addTags( [ 'tag 3_2' ], $rcId3, null, null, 'data3_2' );
+		$this->changeTags->addTags( [ 'tag 1' ], $rcId1, null, null, 'data1' );
+		$this->changeTags->addTags( [ 'tag 3_1' ], $rcId3, null, null );
+		$this->changeTags->addTags( [ 'tag 3_2' ], $rcId3, null, null, 'data3_2' );
 
-		$data = ChangeTags::getTagsWithData( $this->getDb(), $rcId1 );
+		$data = $this->changeTags->getTagsWithData( $this->getDb(), $rcId1 );
 		$this->assertSame( [ 'tag 1' => 'data1' ], $data );
 
-		$data = ChangeTags::getTagsWithData( $this->getDb(), $rcId2 );
+		$data = $this->changeTags->getTagsWithData( $this->getDb(), $rcId2 );
 		$this->assertSame( [], $data );
 
-		$data = ChangeTags::getTagsWithData( $this->getDb(), $rcId3 );
+		$data = $this->changeTags->getTagsWithData( $this->getDb(), $rcId3 );
 		$this->assertArrayEquals( [ 'tag 3_1' => null, 'tag 3_2' => 'data3_2' ], $data, false, true );
 	}
 
@@ -664,22 +774,22 @@ class ChangeTagsTest extends MediaWikiIntegrationTestCase {
 		$this->getServiceContainer()->resetServiceForTesting( 'NameTableStoreFactory' );
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId );
 
 		$rcId = 124;
-		ChangeTags::updateTags( [ 'tag1' ], [], $rcId );
+		$this->changeTags->updateTags( [ 'tag1' ], [], $rcId );
 
-		$this->assertEquals( [ 'tag1' => 2, 'tag2' => 1 ], ChangeTags::tagUsageStatistics() );
+		$this->assertEquals( [ 'tag1' => 2, 'tag2' => 1 ], $this->changeTags->tagUsageStatistics() );
 	}
 
 	public function testListExplicitlyDefinedTags() {
 		$this->emptyChangeTagsTables();
 
 		$rcId = 123;
-		ChangeTags::updateTags( [ 'tag1', 'tag2' ], [], $rcId );
-		ChangeTags::defineTag( 'tag2' );
+		$this->changeTags->updateTags( [ 'tag1', 'tag2' ], [], $rcId );
+		$this->changeTags->defineTag( 'tag2' );
 
-		$this->assertEquals( [ 'tag2' ], ChangeTags::listExplicitlyDefinedTags() );
+		$this->assertEquals( [ 'tag2' ], $this->changeTags->listExplicitlyDefinedTags() );
 
 		$this->newSelectQueryBuilder()
 			->select( [ 'ctd_name', 'ctd_user_defined' ] )
