@@ -1,20 +1,24 @@
 <?php
 
-use Wikimedia\LightweightObjectStore\StorageAwareness;
+use MediaWiki\Deferred\DeferredUpdates;
+use MediaWiki\MainConfigNames;
+use Wikimedia\ObjectCache\BagOStuff;
+use Wikimedia\ObjectCache\HashBagOStuff;
+use Wikimedia\ObjectCache\MultiWriteBagOStuff;
 use Wikimedia\ScopedCallback;
 use Wikimedia\TestingAccessWrapper;
 
 /**
  * @author Matthias Mullie <mmullie@wikimedia.org>
  * @group BagOStuff
- * @covers BagOStuff
+ * @covers \Wikimedia\ObjectCache\BagOStuff
+ * @covers \Wikimedia\ObjectCache\MediumSpecificBagOStuff
  */
 abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 	/** @var BagOStuff */
-	private $cache;
+	protected $cache;
 
-	private const TEST_KEY = 'test';
-	private const TEST_TIME = 1563892142;
+	protected const TEST_TIME = 1563892142;
 
 	protected function setUp(): void {
 		parent::setUp();
@@ -26,38 +30,38 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 				': the configuration is presumably missing from $wgObjectCaches' );
 		}
 		$this->cache->deleteMulti( [
-			$this->cache->makeKey( self::TEST_KEY ),
-			$this->cache->makeKey( self::TEST_KEY ) . ':lock'
+			$this->cache->makeKey( $this->testKey() ),
+			$this->cache->makeKey( $this->testKey() ) . ':lock'
 		] );
 	}
 
+	private function testKey() {
+		return 'test-' . static::class;
+	}
+
 	/**
 	 * @return BagOStuff
 	 */
 	abstract protected function newCacheInstance();
 
 	protected function getCacheByClass( $className ) {
-		$caches = $this->getConfVar( 'ObjectCaches' );
+		$caches = $this->getConfVar( MainConfigNames::ObjectCaches );
 		foreach ( $caches as $id => $cache ) {
 			if ( ( $cache['class'] ?? '' ) === $className ) {
-				return ObjectCache::getInstance( $id );
+				return $this->getServiceContainer()->getObjectCacheFactory()->getInstance( $id );
 			}
 		}
 		$this->markTestSkipped( "No $className is configured" );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::makeGlobalKey
-	 * @covers MediumSpecificBagOStuff::makeKeyInternal
-	 */
 	public function testMakeKey() {
-		$cache = new HashBagOStuff();
+		$cache = new HashBagOStuff( [ 'keyspace' => 'local_prefix' ] );
 
 		$localKey = $cache->makeKey( 'first', 'second', 'third' );
 		$globalKey = $cache->makeGlobalKey( 'first', 'second', 'third' );
 
 		$this->assertSame(
-			'local:first:second:third',
+			'local_prefix:first:second:third',
 			$localKey,
 			'Local key interpolates parameters'
 		);
@@ -75,21 +79,18 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		);
 
 		$this->assertNotEquals(
-			$cache->makeKeyInternal( 'prefix', [ 'a', 'bc:', 'de' ] ),
-			$cache->makeKeyInternal( 'prefix', [ 'a', 'bc', ':de' ] )
+			$cache->makeKey( 'a', 'bc:', 'de' ),
+			$cache->makeKey( 'a', 'bc', ':de' )
 		);
 
 		$keyEmptyCollection = $cache->makeKey( '', 'second', 'third' );
 		$this->assertSame(
-			'local::second:third',
+			'local_prefix::second:third',
 			$keyEmptyCollection,
 			'Local key interpolates empty parameters'
 		);
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::isKeyGlobal
-	 */
 	public function testKeyIsGlobal() {
 		$cache = new HashBagOStuff();
 
@@ -100,12 +101,8 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertTrue( $cache->isKeyGlobal( $globalKey ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::merge
-	 * @covers MediumSpecificBagOStuff::mergeViaCas
-	 */
 	public function testMerge() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 
 		$calls = 0;
 		$casRace = false; // emulate a race
@@ -144,11 +141,8 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		}
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::changeTTL
-	 */
 	public function testChangeTTLRenew() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$value = 'meow';
 
 		$this->cache->add( $key, $value, 60 );
@@ -162,11 +156,8 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertFalse( $this->cache->changeTTL( $key, 15 ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::changeTTL
-	 */
 	public function testChangeTTLExpireRel() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$value = 'meow';
 
 		$this->cache->add( $key, $value, 5 );
@@ -176,11 +167,8 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertFalse( $this->cache->changeTTL( $key, -3600 ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::changeTTL
-	 */
 	public function testChangeTTLExpireAbs() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$value = 'meow';
 
 		$this->cache->add( $key, $value, 5 );
@@ -192,9 +180,6 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertFalse( $this->cache->changeTTL( $key, (int)$now - 3600 ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::changeTTLMulti
-	 */
 	public function testChangeTTLMulti() {
 		$key1 = $this->cache->makeKey( 'test-key1' );
 		$key2 = $this->cache->makeKey( 'test-key2' );
@@ -233,45 +218,42 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertTrue( $ok, "Expiry set for all keys" );
 		$this->assertSame( 1, $this->cache->get( $key1 ), "Key still live" );
 
-		$this->assertEquals( 2, $this->cache->incr( $key1 ) );
-		$this->assertEquals( 3, $this->cache->incr( $key2 ) );
-		$this->assertEquals( 4, $this->cache->incr( $key3 ) );
-
 		// cleanup
 		$this->cache->deleteMulti( [ $key1, $key2, $key3, $key4 ] );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::add
-	 */
 	public function testAdd() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$this->assertFalse( $this->cache->get( $key ) );
 		$this->assertTrue( $this->cache->add( $key, 'test', 5 ) );
 		$this->assertFalse( $this->cache->add( $key, 'test', 5 ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::get
-	 */
+	public function testAddBackground() {
+		$key = $this->cache->makeKey( $this->testKey() );
+		$this->assertFalse( $this->cache->get( $key ) );
+		$this->assertTrue(
+			$this->cache->add( $key, 'test', 5, BagOStuff::WRITE_BACKGROUND )
+		);
+		for ( $i = 0; $i < 100 && $this->cache->get( $key ) !== 'test'; $i++ ) {
+			usleep( 1000 );
+		}
+		$this->assertSame( 'test', $this->cache->get( $key ) );
+	}
+
 	public function testGet() {
 		$value = [ 'this' => 'is', 'a' => 'test' ];
 
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$this->cache->add( $key, $value, 5 );
 		$this->assertSame( $this->cache->get( $key ), $value );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::get
-	 * @covers MediumSpecificBagOStuff::set
-	 * @covers MediumSpecificBagOStuff::getWithSetCallback
-	 */
 	public function testGetWithSetCallback() {
 		$now = self::TEST_TIME;
 		$cache = new HashBagOStuff( [] );
 		$cache->setMockTime( $now );
-		$key = $cache->makeKey( self::TEST_KEY );
+		$key = $cache->makeKey( $this->testKey() );
 
 		$this->assertFalse( $cache->get( $key ), "No value" );
 
@@ -293,41 +275,49 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertFalse( $cache->get( $key ), "Value expired" );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::incr
-	 */
-	public function testIncr() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
-		$this->cache->add( $key, 0, 5 );
-		$this->cache->incr( $key );
-		$expectedValue = 1;
-		$actualValue = $this->cache->get( $key );
-		$this->assertEquals( $expectedValue, $actualValue, 'Value should be 1 after incrementing' );
-	}
-
-	/**
-	 * @covers MediumSpecificBagOStuff::incrWithInit
-	 */
 	public function testIncrWithInit() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 
 		$val = $this->cache->get( $key );
 		$this->assertFalse( $val, "No value yet" );
 
 		$val = $this->cache->incrWithInit( $key, 0, 1, 3 );
-		$this->assertEquals( 3, $val, "Correct init value" );
+		$this->assertSame( 3, $val, "Correct init value" );
 
 		$val = $this->cache->incrWithInit( $key, 0, 1, 3 );
-		$this->assertEquals( 4, $val, "Correct init value" );
+		$this->assertSame( 4, $val, "Correct incremented value" );
 		$this->cache->delete( $key );
 
 		$val = $this->cache->incrWithInit( $key, 0, 5 );
-		$this->assertEquals( 5, $val, "Correct init value" );
+		$this->assertSame( 5, $val, "Correct incremented value" );
+	}
+
+	public function testIncrWithInitAsync() {
+		$key = $this->cache->makeKey( $this->testKey() );
+		$val = $this->cache->get( $key );
+		$this->assertFalse( $val, "No value yet" );
+
+		$val = $this->cache->incrWithInit( $key, 0, 1, 3, BagOStuff::WRITE_BACKGROUND );
+		if ( $val === true ) {
+			$val = $this->cache->get( $key );
+			for ( $i = 0; $i < 1000 && $val !== 3; $i++ ) {
+				usleep( 1000 );
+				$val = $this->cache->get( $key );
+			}
+		}
+		$this->assertSame( 3, $val );
+
+		$val = $this->cache->incrWithInit( $key, 0, 1, 3, BagOStuff::WRITE_BACKGROUND );
+		if ( $val === true ) {
+			$val = $this->cache->get( $key );
+			for ( $i = 0; $i < 1000 && $val !== 4; $i++ ) {
+				usleep( 1000 );
+				$val = $this->cache->get( $key );
+			}
+		}
+		$this->assertSame( 4, $val );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::getMulti
-	 */
 	public function testGetMulti() {
 		$value1 = [ 'this' => 'is', 'a' => 'test' ];
 		$value2 = [ 'this' => 'is', 'another' => 'test' ];
@@ -365,10 +355,6 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->cache->delete( $key4 );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::setMulti
-	 * @covers MediumSpecificBagOStuff::deleteMulti
-	 */
 	public function testSetDeleteMulti() {
 		$map = [
 			$this->cache->makeKey( 'test-1' ) => 'Siberian',
@@ -397,14 +383,15 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		);
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::get
-	 * @covers MediumSpecificBagOStuff::getMulti
-	 * @covers MediumSpecificBagOStuff::merge
-	 * @covers MediumSpecificBagOStuff::delete
-	 */
+	public function testDelete() {
+		// Delete of non-existent key should return true
+		$key = $this->cache->makeKey( 'nonexistent' );
+		$this->assertTrue( $this->cache->delete( $key ) );
+		$this->assertTrue( $this->cache->delete( $key, BagOStuff::WRITE_BACKGROUND ) );
+	}
+
 	public function testSetSegmentable() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$tiny = 418;
 		$small = wfRandomString( 32 );
 		// 64 * 8 * 32768 = 16777216 bytes
@@ -418,7 +405,7 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		foreach ( $cases as $case => $value ) {
 			$this->cache->set( $key, $value, 10, BagOStuff::WRITE_ALLOW_SEGMENTS );
 			$this->assertEquals( $value, $this->cache->get( $key ), "get $case" );
-			$this->assertEquals( $value, $this->cache->getMulti( [ $key ] )[$key], "get $case" );
+			$this->assertEquals( [ $key => $value ], $this->cache->getMulti( [ $key ] ), "get $case" );
 
 			$this->assertTrue(
 				$this->cache->merge( $key, $callback, 5, 1, BagOStuff::WRITE_ALLOW_SEGMENTS ),
@@ -442,7 +429,7 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 			$this->cache->set( $key, "@$value", 10, BagOStuff::WRITE_ALLOW_SEGMENTS );
 			$this->assertEquals( "@$value", $this->cache->get( $key ), "get $case" );
 			$this->assertTrue(
-				$this->cache->delete( $key, BagOStuff::WRITE_PRUNE_SEGMENTS ),
+				$this->cache->delete( $key, BagOStuff::WRITE_ALLOW_SEGMENTS ),
 				"prune $case"
 			);
 			$this->assertFalse( $this->cache->get( $key ), "pruned $case" );
@@ -452,21 +439,21 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->cache->set( $key, 666, 10, BagOStuff::WRITE_ALLOW_SEGMENTS );
 
 		$this->assertEquals( 666, $this->cache->get( $key ) );
-		$this->assertEquals( 667, $this->cache->incr( $key ) );
+		$this->assertEquals( 667, $this->cache->incrWithInit( $key, 10 ) );
 		$this->assertEquals( 667, $this->cache->get( $key ) );
 
-		$this->assertEquals( 664, $this->cache->decr( $key, 3 ) );
-		$this->assertEquals( 664, $this->cache->get( $key ) );
-
 		$this->assertTrue( $this->cache->delete( $key ) );
 		$this->assertFalse( $this->cache->get( $key ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::getScopedLock
-	 */
+	public function testSetBackground() {
+		$key = $this->cache->makeKey( $this->testKey() );
+		$this->assertTrue(
+			$this->cache->set( $key, 'background', BagOStuff::WRITE_BACKGROUND ) );
+	}
+
 	public function testGetScopedLock() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$value1 = $this->cache->getScopedLock( $key, 0 );
 		$value2 = $this->cache->getScopedLock( $key, 0 );
 
@@ -486,10 +473,6 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertInstanceOf( ScopedCallback::class, $value2, 'Second reentrant call returned lock' );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::__construct
-	 * @covers MediumSpecificBagOStuff::trackDuplicateKeys
-	 */
 	public function testReportDupes() {
 		$logger = $this->createMock( Psr\Log\NullLogger::class );
 		$logger->expects( $this->once() )
@@ -511,12 +494,8 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		DeferredUpdates::doUpdates();
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::lock()
-	 * @covers MediumSpecificBagOStuff::unlock()
-	 */
 	public function testLocking() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$this->assertTrue( $this->cache->lock( $key ) );
 		$this->assertFalse( $this->cache->lock( $key ) );
 		$this->assertTrue( $this->cache->unlock( $key ) );
@@ -528,34 +507,29 @@ abstract class BagOStuffTestBase extends MediaWikiIntegrationTestCase {
 		$this->assertTrue( $this->cache->unlock( $key ) );
 	}
 
-	/**
-	 * @covers MediumSpecificBagOStuff::watchErrors()
-	 * @covers MediumSpecificBagOStuff::getLastError()
-	 * @covers MediumSpecificBagOStuff::setLastError()
-	 */
 	public function testErrorHandling() {
-		$key = $this->cache->makeKey( self::TEST_KEY );
+		$key = $this->cache->makeKey( $this->testKey() );
 		$wrapper = TestingAccessWrapper::newFromObject( $this->cache );
 
 		$wp = $this->cache->watchErrors();
 		$this->cache->get( $key );
-		$this->assertSame( StorageAwareness::ERR_NONE, $this->cache->getLastError() );
-		$this->assertSame( StorageAwareness::ERR_NONE, $this->cache->getLastError( $wp ) );
+		$this->assertSame( BagOStuff::ERR_NONE, $this->cache->getLastError() );
+		$this->assertSame( BagOStuff::ERR_NONE, $this->cache->getLastError( $wp ) );
 
-		$wrapper->setLastError( StorageAwareness::ERR_UNREACHABLE );
-		$this->assertSame( StorageAwareness::ERR_UNREACHABLE, $this->cache->getLastError() );
-		$this->assertSame( StorageAwareness::ERR_UNREACHABLE, $this->cache->getLastError( $wp ) );
+		$wrapper->setLastError( BagOStuff::ERR_UNREACHABLE );
+		$this->assertSame( BagOStuff::ERR_UNREACHABLE, $this->cache->getLastError() );
+		$this->assertSame( BagOStuff::ERR_UNREACHABLE, $this->cache->getLastError( $wp ) );
 
 		$wp = $this->cache->watchErrors();
-		$wrapper->setLastError( StorageAwareness::ERR_UNEXPECTED );
+		$wrapper->setLastError( BagOStuff::ERR_UNEXPECTED );
 		$wp2 = $this->cache->watchErrors();
-		$this->assertSame( StorageAwareness::ERR_UNEXPECTED, $this->cache->getLastError() );
-		$this->assertSame( StorageAwareness::ERR_UNEXPECTED, $this->cache->getLastError( $wp ) );
-		$this->assertSame( StorageAwareness::ERR_NONE, $this->cache->getLastError( $wp2 ) );
+		$this->assertSame( BagOStuff::ERR_UNEXPECTED, $this->cache->getLastError() );
+		$this->assertSame( BagOStuff::ERR_UNEXPECTED, $this->cache->getLastError( $wp ) );
+		$this->assertSame( BagOStuff::ERR_NONE, $this->cache->getLastError( $wp2 ) );
 
 		$this->cache->get( $key );
-		$this->assertSame( StorageAwareness::ERR_UNEXPECTED, $this->cache->getLastError() );
-		$this->assertSame( StorageAwareness::ERR_UNEXPECTED, $this->cache->getLastError( $wp ) );
-		$this->assertSame( StorageAwareness::ERR_NONE, $this->cache->getLastError( $wp2 ) );
+		$this->assertSame( BagOStuff::ERR_UNEXPECTED, $this->cache->getLastError() );
+		$this->assertSame( BagOStuff::ERR_UNEXPECTED, $this->cache->getLastError( $wp ) );
+		$this->assertSame( BagOStuff::ERR_NONE, $this->cache->getLastError( $wp2 ) );
 	}
 }
