@@ -7,6 +7,7 @@ use MediaWiki\Debug\MWDebug;
 use MediaWiki\Json\JsonCodec;
 use MediaWiki\Parser\CacheTime;
 use MediaWiki\Parser\ParserOutput;
+use MediaWiki\Parser\ParserOutputFlags;
 use MediaWiki\Tests\Json\JsonDeserializableSubClass;
 use MediaWiki\Title\Title;
 use MediaWiki\Title\TitleValue;
@@ -316,6 +317,11 @@ abstract class ParserCacheSerializationTestCases {
 		$parserOutputWithMetadataPost1_44 = clone $parserOutputWithMetadata;
 		$parserOutputWithMetadataPost1_44->setLanguageLinks( [ 'm:link1', 'mw:link2' ] );
 
+		$parserOutputWithMetadataPost1_45 = clone $parserOutputWithMetadataPost1_44;
+		// Non-standard output flags were deprecated in MW 1.45
+		$parserOutputWithMetadataPost1_45->setOutputFlag( 'test', false );
+		$parserOutputWithMetadataPost1_45->setOutputFlag( ParserOutputFlags::VARY_REVISION_ID );
+
 		$parserOutputWithSections = new ParserOutput( '' );
 		$parserOutputWithSections->setSections( self::SECTIONS );
 
@@ -598,6 +604,61 @@ abstract class ParserCacheSerializationTestCases {
 					$testCase->assertArrayEquals( [ 'test' ], $object->getAllFlags() );
 				}
 			],
+			'withMetadataPost1_45' => [
+				'instance' => $parserOutputWithMetadataPost1_45,
+				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
+					$testCase->assertSame( 42, $object->getSpeculativeRevIdUsed() );
+					$testCase->assertArrayEquals( [ 'm:link1', 'mw:link2' ], $object->getLanguageLinks() );
+					$testCase->assertArrayEquals( [ 'enwiki' => [
+						'interwiki1' => 1,
+						'interwiki2' => 1
+					] ], $object->getInterwikiLinks() );
+					$testCase->assertArrayEquals( [ 'category1', 'category2' ], $object->getCategoryNames() );
+					$testCase->assertArrayEquals( [
+						'category1' => '2',
+						'category2' => '1'
+					], $object->getCategoryMap() );
+					$testCase->assertArrayEquals( [ 'indicator1' => 'indicator1_value' ], $object->getIndicators() );
+					$testCase->assertSame( 'title_text1', $object->getTitleText() );
+					$testCase->assertArrayEquals( self::SECTIONS, $object->getSections() );
+					$testCase->assertNotNull( $object->getTOCData() );
+					$testCase->assertArrayEquals( [
+						NS_MAIN => [ 'Link1' => 42 ],
+						NS_USER => [ 'Link2' => 43 ]
+					], $object->getLinks() );
+					$testCase->assertArrayEquals( [
+						NS_SPECIAL => [ 'Template1' => 42 ]
+					], $object->getTemplates() );
+					$testCase->assertArrayEquals( [
+						NS_SPECIAL => [ 'Template1' => 4242 ]
+					], $object->getTemplateIds() );
+					$testCase->assertArrayEquals( [ 'Image1' => 1 ], $object->getImages() );
+					$testCase->assertArrayEquals( [ 'Image1' => [
+						'time' => MWTimestamp::convert( TS_MW, 123456789 ), 'sha1' => 'test_sha1'
+					] ], $object->getFileSearchOptions() );
+					$testCase->assertArrayEquals( [ 'https://test.com' => 1 ], $object->getExternalLinks() );
+					$testCase->assertArrayEquals( [ 'tag1' => 'head_item1' ], $object->getHeadItems() );
+					$testCase->assertArrayEquals( [ 'module1' ], $object->getModules() );
+					$testCase->assertArrayEquals( [ 'module_style1' ], $object->getModuleStyles() );
+					$testCase->assertArrayEquals( [ 'key1' => 'value1' ], $object->getJsConfigVars() );
+					$testCase->assertArrayEquals( [ 'warning1' ], $object->getWarnings() );
+					$testCase->assertSame( 'noindex', $object->getIndexPolicy() );
+					$testCase->assertSame( MWTimestamp::convert( TS_MW, 987654321 ), $object->getRevisionTimestamp() );
+					$testCase->assertArrayEquals(
+						[ 'limit_report_key1' => 'value1' ],
+						$object->getLimitReportData()
+					);
+					$testCase->assertArrayEquals(
+						[ 'limit_report_key1' => 'value1' ],
+						$object->getLimitReportJSData()
+					);
+					$testCase->assertTrue( $object->getEnableOOUI() );
+					$testCase->assertTrue( $object->getHideNewSection() );
+					$testCase->assertTrue( $object->getNewSection() );
+					$testCase->assertTrue( $object->getOutputFlag( ParserOutputFlags::VARY_REVISION_ID ) );
+					$testCase->assertArrayEquals( [ 'vary-revision-id' ], $object->getAllFlags() );
+				}
+			],
 			'withSections' => [
 				'instance' => $parserOutputWithSections,
 				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
@@ -733,6 +794,7 @@ abstract class ParserCacheSerializationTestCases {
 			$oldDeserializer = $d;
 			$d = static function ( $data ) use ( $oldDeserializer ) {
 				MWDebug::filterDeprecationForTest( '/::addLanguageLink without prefix/' );
+				MWDebug::filterDeprecationForTest( '/::setOutputFlag with non-standard flag/' );
 				return $oldDeserializer( $data );
 			};
 		}
